@@ -1,8 +1,8 @@
 #!/bin/sh
 
-aclocal
+aclocal$AMSUFFIX
 autoconf
 libtoolize --automake --force
 #aclocal
-automake -a
+automake$AMSUFFIX -a
 
