@@ -2,7 +2,7 @@
  * log-cmd.c -- Display log messages
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -35,7 +35,7 @@
 #include "svn_sorts.h"
 #include "svn_xml.h"
 #include "svn_time.h"
-#include "svn_utf.h"
+#include "svn_cmdline.h"
 #include "svn_subst.h"
 #include "cl.h"
 
@@ -75,22 +75,25 @@ num_lines (const char *msg)
 }
 
 
-/* Baton for log_message_receiver(). */
+/* Baton for log_message_receiver() and log_message_receiver_xml(). */
 struct log_receiver_baton
 {
   /* Check for cancellation on each invocation of a log receiver. */
   svn_cancel_func_t cancel_func;
   void *cancel_baton;
 
-  /* Don't print log message body nor line count.  This is ignored for
-     xml output. */
-  svn_boolean_t quiet;
+  /* Don't print log message body nor its line count. */
+  svn_boolean_t omit_log_message;
+
+  /* Output stream */
+  svn_stream_t *out;
 };
 
 
 /* The separator between log messages. */
 #define SEP_STRING \
-  "------------------------------------------------------------------------\n"
+  "------------------------------------------------------------------------" \
+  APR_EOL_STR
 
 
 /* Implement `svn_log_message_receiver_t', printing the logs in
@@ -101,7 +104,7 @@ struct log_receiver_baton
  * First, print a header line.  Then if CHANGED_PATHS is non-null,
  * print all affected paths in a list headed "Changed paths:\n",
  * immediately following the header line.  Then print a newline
- * followed by the message body, unless BATON->quiet is true.
+ * followed by the message body, unless BATON->omit_log_message is true.
  *
  * Here are some examples of the output:
  *
@@ -180,8 +183,8 @@ log_message_receiver (void *baton,
                       apr_pool_t *pool)
 {
   struct log_receiver_baton *lb = baton;
-  const char *author_native, *date_native;
-  const char *msg_native = NULL;     /* Silence a gcc uninitialized warning */
+  const char *author_stdout, *date_stdout;
+  const char *msg_stdout = NULL;     /* Silence a gcc uninitialized warning */
   svn_error_t *err;
 
   /* Number of lines in the msg. */
@@ -192,8 +195,8 @@ log_message_receiver (void *baton,
 
   if (rev == 0)
     {
-      printf ("No commit for revision 0.\n");
-      return SVN_NO_ERROR;
+      return svn_stream_printf (lb->out, pool,
+                                "No commit for revision 0." APR_EOL_STR);
     }
 
   /* ### See http://subversion.tigris.org/issues/show_bug.cgi?id=807
@@ -202,9 +205,12 @@ log_message_receiver (void *baton,
   if (author == NULL)
     author = "(no author)";
 
-  err = svn_utf_cstring_from_utf8 (&author_native, author, pool);
+  err = svn_cmdline_cstring_from_utf8 (&author_stdout, author, pool);
   if (err && (APR_STATUS_IS_EINVAL (err->apr_err)))
-    author_native = svn_utf_cstring_from_utf8_fuzzy (author, pool);
+    {
+      author_stdout = svn_cmdline_cstring_from_utf8_fuzzy (author, pool);
+      svn_error_clear (err);
+    }
   else if (err)
     return err;
 
@@ -212,14 +218,16 @@ log_message_receiver (void *baton,
     {
       /* Convert date to a format for humans. */
       apr_time_t time_temp;
+      const char *date_utf8;
       
       SVN_ERR (svn_time_from_cstring (&time_temp, date, pool));
-      date_native = svn_time_to_human_cstring(time_temp, pool);
+      date_utf8 = svn_time_to_human_cstring(time_temp, pool);
+      SVN_ERR (svn_cmdline_cstring_from_utf8 (&date_stdout, date_utf8, pool));
     }
   else
-    date_native = "(no date)";
+    date_stdout = "(no date)";
   
-  if (! lb->quiet)
+  if (! lb->omit_log_message)
     {
       if (msg == NULL)
         msg = "";
@@ -227,23 +235,26 @@ log_message_receiver (void *baton,
       {
         /* Convert log message from UTF8/LF to native locale and eol-style. */
         svn_string_t *logmsg = svn_string_create (msg, pool);
-        svn_subst_detranslate_string (&logmsg, logmsg, pool);
-        msg_native = logmsg->data;
+        svn_subst_detranslate_string (&logmsg, logmsg, TRUE, pool);
+        msg_stdout = logmsg->data;
       }
     }
 
-  printf (SEP_STRING);
+  SVN_ERR (svn_stream_printf (lb->out, pool, SEP_STRING));
 
-  printf ("rev %" SVN_REVNUM_T_FMT ":  %s | %s",
-          rev, author_native, date_native);
+  SVN_ERR (svn_stream_printf (lb->out, pool,
+                              "r%" SVN_REVNUM_T_FMT " | %s | %s",
+                              rev, author_stdout, date_stdout));
 
-  if (! lb->quiet)
+  if (! lb->omit_log_message)
     {
-      lines = num_lines (msg_native);
-      printf (" | %d line%s", lines, (lines > 1) ? "s" : "");
+      lines = num_lines (msg_stdout);
+      SVN_ERR (svn_stream_printf (lb->out, pool,
+                                  " | %d line%s", lines,
+                                  (lines > 1) ? "s" : ""));
     }
 
-  printf ("\n");
+  SVN_ERR (svn_stream_printf (lb->out, pool, APR_EOL_STR));
 
   if (changed_paths)
     {
@@ -251,15 +262,16 @@ log_message_receiver (void *baton,
       int i;
 
       /* Get an array of sorted hash keys. */
-      sorted_paths = apr_hash_sorted_keys (changed_paths,
-                                           svn_sort_compare_items_as_paths, 
-                                           pool);
+      sorted_paths = svn_sort__hash (changed_paths,
+                                     svn_sort_compare_items_as_paths, pool);
 
-      printf ("Changed paths:\n");
+      SVN_ERR (svn_stream_printf (lb->out, pool,
+                                  "Changed paths:" APR_EOL_STR));
       for (i = 0; i < sorted_paths->nelts; i++)
         {
-          svn_item_t *item = &(APR_ARRAY_IDX (sorted_paths, i, svn_item_t));
-          const char *path_native, *path = item->key;
+          svn_sort__item_t *item = &(APR_ARRAY_IDX (sorted_paths, i,
+                                                    svn_sort__item_t));
+          const char *path_stdout, *path = item->key;
           svn_log_changed_path_t *log_item 
             = apr_hash_get (changed_paths, item->key, item->klen);
           const char *copy_data = "";
@@ -267,32 +279,35 @@ log_message_receiver (void *baton,
           if (log_item->copyfrom_path 
               && SVN_IS_VALID_REVNUM (log_item->copyfrom_rev))
             {
-              SVN_ERR (svn_utf_cstring_from_utf8 (&path_native, 
-                                                  log_item->copyfrom_path, 
-                                                  pool));
+              SVN_ERR (svn_cmdline_cstring_from_utf8
+                       (&path_stdout, log_item->copyfrom_path, pool));
               copy_data 
                 = apr_psprintf (pool, 
                                 " (from %s:%" SVN_REVNUM_T_FMT ")",
-                                path_native,
+                                path_stdout,
                                 log_item->copyfrom_rev);
             }
-          SVN_ERR (svn_utf_cstring_from_utf8 (&path_native, path, pool));
-          printf ("   %c %s%s\n", log_item->action, path_native, copy_data);
+          SVN_ERR (svn_cmdline_cstring_from_utf8 (&path_stdout, path, pool));
+          SVN_ERR (svn_stream_printf (lb->out, pool, "   %c %s%s" APR_EOL_STR,
+                                      log_item->action, path_stdout,
+                                      copy_data));
         }
     }
 
-  if (! lb->quiet)
+  if (! lb->omit_log_message)
     {
-      printf ("\n");  /* A blank line always precedes the log message. */
-      printf ("%s\n", msg_native);
+      /* A blank line always precedes the log message. */
+      SVN_ERR (svn_stream_printf (lb->out, pool, APR_EOL_STR "%s" APR_EOL_STR,
+                                  msg_stdout));
     }
 
   return SVN_NO_ERROR;
 }
 
 
-/* This implements `svn_log_message_receiver_t', printing the logs in
- * XML.  BATON is ignored.
+/* This implements `svn_log_message_receiver_t', printing the logs in XML.
+ *
+ * BATON is of type `struct log_receiver_baton'.
  *
  * Here is an example of the output; note that the "<log>" and
  * "</log>" tags are not emitted by this function:
@@ -352,24 +367,24 @@ log_message_receiver_xml (void *baton,
   svn_xml_make_open_tag (&sb, pool, svn_xml_normal, "logentry",
                          "revision", revstr, NULL);
 
-  if (author == NULL)
-    author = "(no author)";
+  if (author)
+    {
+      /* <author>xxx</author> */
+      svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "author",
+                             NULL);
+      svn_xml_escape_cdata_cstring (&sb, author, pool);
+      svn_xml_make_close_tag (&sb, pool, "author");
+    }
 
-  /* <author>xxx</author> */
-  svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "author",
-                         NULL);
-  svn_xml_escape_cdata_cstring (&sb, author, pool);
-  svn_xml_make_close_tag (&sb, pool, "author");
-
-  if (date == NULL)
-    date = "(no date)";
-
-  /* Print the full, uncut, date.  This is machine output. */
-  /* <date>xxx</date> */
-  svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "date",
-                         NULL);
-  svn_xml_escape_cdata_cstring (&sb, date, pool);
-  svn_xml_make_close_tag (&sb, pool, "date");
+  if (date)
+    {
+      /* Print the full, uncut, date.  This is machine output. */
+      /* <date>xxx</date> */
+      svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "date",
+                             NULL);
+      svn_xml_escape_cdata_cstring (&sb, date, pool);
+      svn_xml_make_close_tag (&sb, pool, "date");
+    }
 
   if (changed_paths)
     {
@@ -422,24 +437,27 @@ log_message_receiver_xml (void *baton,
       svn_xml_make_close_tag (&sb, pool, "paths");
     }
 
-  if (msg == NULL)
-    msg = "";
+  if (! lb->omit_log_message)
+    {
+      if (msg == NULL)
+        msg = "";
+
+      /* <msg>xxx</msg> */
+      svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "msg", NULL);
+      SVN_ERR (svn_subst_translate_cstring (msg, &msg_native_eol,
+                                            APR_EOL_STR, /* the 'native' eol */
+                                            FALSE,       /* no need to repair */
+                                            NULL,        /* no keywords */
+                                            FALSE,       /* no expansion */
+                                            pool));
+      svn_xml_escape_cdata_cstring (&sb, msg_native_eol, pool);
+      svn_xml_make_close_tag (&sb, pool, "msg");
+    }
 
-  /* <msg>xxx</msg> */
-  svn_xml_make_open_tag (&sb, pool, svn_xml_protect_pcdata, "msg", NULL);
-  SVN_ERR (svn_subst_translate_cstring (msg, &msg_native_eol,
-                                        APR_EOL_STR, /* the 'native' eol */
-                                        FALSE,       /* no need to repair */
-                                        NULL,        /* no keywords */
-                                        FALSE,       /* no expansion */
-                                        pool));
-  svn_xml_escape_cdata_cstring (&sb, msg_native_eol, pool);
-  svn_xml_make_close_tag (&sb, pool, "msg");
-  
   /* </logentry> */
   svn_xml_make_close_tag (&sb, pool, "logentry");
 
-  printf ("%s", sb->data);
+  SVN_ERR (svn_stream_printf (lb->out, pool, "%s", sb->data));
 
   return SVN_NO_ERROR;
 }
@@ -477,19 +495,18 @@ svn_cl__log (apr_getopt_t *os,
          extra code complexity to get around this slight inefficiency
          doesn't seem worth it, however.  */
 
-      opt_state->end_revision.kind = opt_state->start_revision.kind;
-
-      opt_state->end_revision.value = opt_state->start_revision.value;
-
-      opt_state->end_revision.value.number
-        = opt_state->start_revision.value.number;
-
-      opt_state->end_revision.value.date
-        = opt_state->start_revision.value.date;
+      opt_state->end_revision = opt_state->start_revision;
     }
   else if (opt_state->start_revision.kind == svn_opt_revision_unspecified)
     {
-      opt_state->start_revision.kind = svn_opt_revision_head;
+      const char *target = APR_ARRAY_IDX (targets, 0, const char *);
+
+      /* If the first target is a URL, then we default to HEAD:1.
+         Otherwise, the default is BASE:1 since WC@HEAD may not exist. */
+      if (svn_path_is_url (target))
+        opt_state->start_revision.kind = svn_opt_revision_head;
+      else
+        opt_state->start_revision.kind = svn_opt_revision_base;
 
       if (opt_state->end_revision.kind == svn_opt_revision_unspecified)
         {
@@ -500,7 +517,12 @@ svn_cl__log (apr_getopt_t *os,
 
   lb.cancel_func = ctx->cancel_func;
   lb.cancel_baton = ctx->cancel_baton;
-  lb.quiet = opt_state->quiet;
+  lb.omit_log_message = opt_state->quiet;
+  SVN_ERR (svn_stream_for_stdout (&lb.out, pool));
+
+  if (! opt_state->quiet)
+    svn_cl__get_notifier (&ctx->notify_func, &ctx->notify_baton, FALSE, FALSE,
+                          FALSE, pool);
 
   if (opt_state->xml)
     {
@@ -517,14 +539,14 @@ svn_cl__log (apr_getopt_t *os,
           /* "<log>" */
           svn_xml_make_open_tag (&sb, pool, svn_xml_normal, "log", NULL);
 
-          printf ("%s", sb->data);  
+          SVN_ERR (svn_stream_printf (lb.out, pool, "%s", sb->data));
         }
       
       SVN_ERR (svn_client_log (targets,
                                &(opt_state->start_revision),
                                &(opt_state->end_revision),
                                opt_state->verbose,
-                               opt_state->strict,
+                               opt_state->stop_on_copy,
                                log_message_receiver_xml,
                                &lb,
                                ctx,
@@ -537,7 +559,7 @@ svn_cl__log (apr_getopt_t *os,
           /* "</log>" */
           svn_xml_make_close_tag (&sb, pool, "log");
 
-          printf ("%s", sb->data);
+          SVN_ERR (svn_stream_printf (lb.out, pool, "%s", sb->data));
         }
     }
   else  /* default output format */
@@ -553,14 +575,14 @@ svn_cl__log (apr_getopt_t *os,
                                &(opt_state->start_revision),
                                &(opt_state->end_revision),
                                opt_state->verbose,
-                               opt_state->strict,
+                               opt_state->stop_on_copy,
                                log_message_receiver,
                                &lb,
                                ctx,
                                pool));
 
       if (! opt_state->incremental)
-        printf (SEP_STRING);
+        SVN_ERR (svn_stream_printf (lb.out, pool, SEP_STRING));
     }
 
   return SVN_NO_ERROR;
