@@ -4,7 +4,7 @@
  * in here.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -87,12 +87,6 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   svn_boolean_t remove_file = TRUE;
   struct svn_config_t *cfg;
 
-  /* ### TODO: This function should, if it fails to create a temporary
-     file in the requested directory, fall back to the use of a
-     temporary directory (like /tmp, or C:\TEMP, or ...) as described
-     by a not-yet-existant APR function.  See issue #929. */
-
-     
   /* Look for the Subversion specific environment variable. */
   editor = getenv ("SVN_EDITOR");
 
@@ -127,7 +121,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
     return svn_error_create 
       (SVN_ERR_CL_NO_EXTERNAL_EDITOR, NULL,
        "None of the environment variables SVN_EDITOR, VISUAL or EDITOR is "
-       "set, and no 'editor-cmd' run-time configuration option was found.");
+       "set, and no 'editor-cmd' run-time configuration option was found");
 
   /* Convert file contents from UTF-8 */
   SVN_ERR (svn_utf_cstring_from_utf8 (&contents_native, contents, pool));
@@ -136,10 +130,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
      into tmpfile_name */
   apr_err = apr_filepath_get (&old_cwd, APR_FILEPATH_NATIVE, pool);
   if (apr_err)
-    {
-      return svn_error_create
-        (apr_err, NULL, "failed to get current working directory");
-    }
+    return svn_error_wrap_apr (apr_err, "Can't get working directory");
 
   /* APR doesn't like "" directories */
   if (base_dir[0] == '\0')
@@ -149,9 +140,8 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   apr_err = apr_filepath_set (base_dir_apr, pool);
   if (apr_err)
     {
-      return svn_error_createf
-        (apr_err, NULL,
-         "failed to change working directory to '%s'", base_dir);
+      return svn_error_wrap_apr
+        (apr_err, "Can't change working directory to '%s'", base_dir);
     }
 
   /*** From here on, any problems that occur require us to cd back!! ***/
@@ -175,11 +165,9 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
     apr_err = apr_err2;
   
   /* Make sure the whole CONTENTS were written, else return an error. */
-  if (apr_err || (written != strlen (contents_native)))
+  if (apr_err)
     {
-      err = svn_error_createf
-        (apr_err ? apr_err : SVN_ERR_INCOMPLETE_DATA, NULL,
-         "failed writing '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, "Can't write to '%s'", tmpfile_name);
       goto cleanup;
     }
 
@@ -193,8 +181,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
                       APR_FINFO_MTIME | APR_FINFO_SIZE, pool);
   if (apr_err)
     {
-      err =  svn_error_createf (apr_err, NULL,
-                                "failed to stat '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, "Can't stat '%s'", tmpfile_name);
       goto cleanup;
     }
 
@@ -218,8 +205,7 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
                       APR_FINFO_MTIME | APR_FINFO_SIZE, pool);
   if (apr_err)
     {
-      err = svn_error_createf (apr_err, NULL,
-                               "failed to stat '%s'", tmpfile_name);
+      err = svn_error_wrap_apr (apr_err, "Can't stat '%s'", tmpfile_name);
       goto cleanup;
     }
   
@@ -265,9 +251,8 @@ svn_cl__edit_externally (const char **edited_contents /* UTF-8! */,
   apr_err = apr_filepath_set (old_cwd, pool);
   if (apr_err)
     {
-      svn_handle_error (svn_error_create
-                        (apr_err, NULL,
-                         "failed to restore current working directory"),
+      svn_handle_error (svn_error_wrap_apr
+                        (apr_err, "Can't restore working directory"),
                         stderr, TRUE /* fatal */);
     }
 
@@ -305,7 +290,7 @@ svn_cl__make_log_msg_baton (void **baton,
              a deliberate encoding, and even if it is, we still
              can't handle it. */
           return svn_error_create (SVN_ERR_CL_BAD_LOG_MESSAGE, NULL,
-                                   "Log message contains a zero byte.");
+                                   "Log message contains a zero byte");
         }
       lmb->message = opt_state->filedata->data;
     }      
@@ -533,7 +518,7 @@ svn_cl__get_log_message (const char **log_msg,
             err = svn_error_quick_wrap 
               (err, "Could not use external editor to fetch log message; "
                "consider setting the $SVN_EDITOR environment variable "
-               "or using the --message (-m) or --file (-F) options.");
+               "or using the --message (-m) or --file (-F) options");
           return err;
         }
 
@@ -551,7 +536,7 @@ svn_cl__get_log_message (const char **log_msg,
              white space as we will consider white space only as empty */
           int len;
 
-          for (len = message->len; len >= 0; len--)
+          for (len = message->len - 1; len >= 0; len--)
             {
               /* FIXME: should really use an UTF-8 whitespace test
                  rather than apr_isspace, which is locale dependant */
@@ -567,9 +552,7 @@ svn_cl__get_log_message (const char **log_msg,
           const char *reply;
           svn_cl__prompt_user (&reply,
                                "\nLog message unchanged or not specified\n"
-                               "a)bort, c)ontinue, e)dit\n",
-                               FALSE, /* don't hide the reply */
-                               NULL, pool);
+                               "a)bort, c)ontinue, e)dit\n", pool);
           if (reply)
             {
               char letter = apr_tolower (reply[0]);
