@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -37,6 +37,34 @@ extern "C" {
 
 /* ---------------------------------------------------------------*/
 
+
+/** Callback type for checking authorization on paths produced by (at
+ * least) svn_repos_dir_delta().
+ *
+ * Set @a *allowed to TRUE to indicate that some operation is
+ * authorized for @a path in @a root, or set it to FALSE to indicate
+ * unauthorized (presumably according to state stored in @a baton).
+ * 
+ * Do not assume @a pool has any lifetime beyond this call.
+ *
+ * The exact operation being authorized depends on the callback
+ * implementation.  For read authorization, for example, the caller
+ * would implement an instance that does read checking, and pass it as
+ * a parameter named [perhaps] 'authz_read_func'.  The receiver of
+ * that parameter might also take another parameter named
+ * 'authz_write_func', which although sharing this type, would be a
+ * different implementation.
+ *
+ * Note: If someday we want more sophisticated authorization states
+ * than just yes/no, @a allowed can become an enum type.
+ */
+typedef svn_error_t *(*svn_repos_authz_func_t) (svn_boolean_t *allowed,
+                                                svn_fs_root_t *root,
+                                                const char *path,
+                                                void *baton,
+                                                apr_pool_t *pool);
+
+
 
 /** The repository object. */
 typedef struct svn_repos_t svn_repos_t;
@@ -44,6 +72,15 @@ typedef struct svn_repos_t svn_repos_t;
 /* Opening and creating repositories. */
 
 
+/** Find the root path of the repository that contains @a path.
+ *
+ * If a repository was found, the path to the root of the repository
+ * is returned, else @c NULL. The pointer to the returned path may be
+ * equal to @a path argument.
+ */
+const char *svn_repos_find_root_path (const char *path,
+                                      apr_pool_t *pool);
+
 /** Set @a *repos_p to a repository object for the repository at @a path.
  *
  * Allocate @a *repos_p in @a pool.
@@ -62,29 +99,17 @@ svn_error_t *svn_repos_open (svn_repos_t **repos_p,
  * and so on.  Return the (open) repository object in @a *repos_p,
  * allocated in @a pool.
  *
- * The directory structure will be created based on the @a on_disk_template
- * argument. If NULL, then the default template will be used (or an
- * internal, hard-coded fallback if the default cannot be found). If
- * the argument contains a path separate ('/'), then it is assumed to
- * specify a template directory. Otherwise, it is assumed to be the name
- * of a template from the standard set of installed templates.
- *
- * Once the repository directory has been constructed, and the Berkeley DB
- * filesystem set up, then the @a in_repos_template will be used to
- * specify an initial structure to import into the repository. Hooks will
- * not be run for this import. The argument specifies a template name or
- * a path to a template, similar to @a on_disk_template. If NULL is passed,
- * then nothing will be imported.
- *
  * @a config is a client configuration hash of @c svn_config_t * items
  * keyed on config category names, and may be NULL.
  *
  * @a fs_config is passed to the filesystem, and may be NULL.
+ *
+ * @a unused_1 and @a unused_2 are not used and should be NULL.
  */
 svn_error_t *svn_repos_create (svn_repos_t **repos_p, 
                                const char *path,
-                               const char *on_disk_template,
-                               const char *in_repos_template,
+                               const char *unused_1,
+                               const char *unused_2,
                                apr_hash_t *config,
                                apr_hash_t *fs_config,
                                apr_pool_t *pool);
@@ -97,6 +122,17 @@ svn_error_t *svn_repos_delete (const char *path, apr_pool_t *pool);
 /** Return the filesystem associated with repository object @a repos. */
 svn_fs_t *svn_repos_fs (svn_repos_t *repos);
 
+
+/** Make a hot copy of the Subversion repository found at @a src_path 
+ * to @a dst_path. 
+ *
+ * @copydoc svn_fs_hotcopy_berkeley()
+ */
+svn_error_t * svn_repos_hotcopy (const char *src_path,
+                                 const char *dst_path,
+                                 svn_boolean_t clean_logs,
+                                 apr_pool_t *pool);
+
 /** Run database recovery procedures on the repository at @a path,
  * returning the database to a consistent state.  Use @a pool for all
  * allocation.
@@ -108,6 +144,17 @@ svn_fs_t *svn_repos_fs (svn_repos_t *repos);
 svn_error_t *svn_repos_recover (const char *path, apr_pool_t *pool);
 
 
+/** This function is a wrapper around svn_fs_berkeley_logfiles(),
+ * returning log file paths relative to the root of the repository.
+ *
+ * @copydoc svn_fs_berkeley_logfiles()
+ */
+svn_error_t *svn_repos_db_logfiles (apr_array_header_t **logfiles,
+                                    const char *path,
+                                    svn_boolean_t only_unused,
+                                    apr_pool_t *pool);
+
+
 
 /* Repository Paths */
 
@@ -119,12 +166,21 @@ const char *svn_repos_path (svn_repos_t *repos, apr_pool_t *pool);
  */
 const char *svn_repos_db_env (svn_repos_t *repos, apr_pool_t *pool);
 
+/** Return path to @a repos's config directory, allocated in @a pool. */
+const char *svn_repos_conf_dir (svn_repos_t *repos, apr_pool_t *pool);
+
+/** Return path to @a repos's svnserve.conf, allocated in @a pool. */
+const char *svn_repos_svnserve_conf (svn_repos_t *repos, apr_pool_t *pool);
+
 /** Return path to @a repos's lock directory, allocated in @a pool. */
 const char *svn_repos_lock_dir (svn_repos_t *repos, apr_pool_t *pool);
 
 /** Return path to @a repos's db lockfile, allocated in @a pool. */
 const char *svn_repos_db_lockfile (svn_repos_t *repos, apr_pool_t *pool);
 
+/** Return path to @a repos's db logs lockfile, allocated in @a pool. */
+const char *svn_repos_db_logs_lockfile (svn_repos_t *repos, apr_pool_t *pool);
+
 /** Return the path to @a repos's hook directory, allocated in @a pool. */
 const char *svn_repos_hook_dir (svn_repos_t *repos, apr_pool_t *pool);
 
@@ -165,8 +221,8 @@ const char *svn_repos_post_revprop_change_hook (svn_repos_t *repos,
  * Specifically, the report will create a transaction made by @a username, 
  * relative to @a fs_base in the filesystem.  @a target is a single path 
  * component, used to limit the scope of the report to a single entry of 
- * @a fs_base, or @c NULL if all of @a fs_base itself is the main subject 
- * of the report.
+ * @a fs_base, or "" if all of @a fs_base itself is the main subject of
+ * the report.
  *
  * @a tgt_path and @a revnum is the fs path/revision pair that is the
  * "target" of @c dir_delta.  In other words, a tree delta will be
@@ -180,7 +236,7 @@ const char *svn_repos_post_revprop_change_hook (svn_repos_t *repos,
  * transform the reported hierarchy to revision @a revnum, preserving the
  * reported hierarchy.
  *
- * @a text_deltas instructs the driver of the @a editor to enable to disable
+ * @a text_deltas instructs the driver of the @a editor to enable
  * the generation of text deltas.
  *
  * @a recurse instructs the driver of the @a editor to send a recursive
@@ -189,6 +245,9 @@ const char *svn_repos_post_revprop_change_hook (svn_repos_t *repos,
  * @a ignore_ancestry instructs the driver to ignore node ancestry
  * when determining how to transmit differences.
  *
+ * The @a authz_read_func and @a authz_read_baton are passed along to
+ * @c svn_repos_dir_delta(); see that function for how they are used.
+ *
  * All allocation for the context and collected state will occur in
  * @a pool.
  */
@@ -205,6 +264,8 @@ svn_repos_begin_report (void **report_baton,
                         svn_boolean_t ignore_ancestry,
                         const svn_delta_editor_t *editor,
                         void *edit_baton,
+                        svn_repos_authz_func_t authz_read_func,
+                        void *authz_read_baton,
                         apr_pool_t *pool);
 
 
@@ -265,13 +326,21 @@ svn_error_t *svn_repos_delete_path (void *report_baton,
  * it drive an update editor (using @c svn_repos_delta_dirs()), then
  * abort the transaction.  If an error occurs during the driving of
  * the editor, we do NOT abort the edit; that responsibility belongs
- * to the caller, if it happens at all.
+ * to the caller, if it happens at all.  The fs transaction will be
+ * aborted even if the editor drive fails, so the caller does not need
+ * to clean up.
  */
-svn_error_t *svn_repos_finish_report (void *report_baton);
+svn_error_t *svn_repos_finish_report (void *report_baton,
+                                      apr_pool_t *pool);
 
 
-/** The report-driver is bailing, so abort the fs transaction. */
-svn_error_t *svn_repos_abort_report (void *report_baton);
+/** The report-driver is bailing, so abort the fs transaction.  This
+ * function can be called anytime before @c svn_repos_finish_report() is
+ * called.  No other reporting functions should be called after calling
+ * this function.
+ */
+svn_error_t *svn_repos_abort_report (void *report_baton,
+                                     apr_pool_t *pool);
 
 
 /* ---------------------------------------------------------------*/
@@ -281,19 +350,34 @@ svn_error_t *svn_repos_abort_report (void *report_baton);
 /** Use the provided @a editor and @a edit_baton to describe the changes
  * necessary for making a given node (and its descendants, if it is a
  * directory) under @a src_root look exactly like @a tgt_path under
- * @a tgt_root.  @a src_entry is the node to update, and is either @c NULL 
- * or a single path component.  If @a src_entry is @c NULL, then compute 
- * the difference between the entire tree anchored at @a src_parent_dir 
- * under @a src_root and @a tgt_path under @a target_root.  Else, describe 
- * the changes needed to update only that entry in @a src_parent_dir.
- * Typically, callers of this function will use a @a tgt_path that is the
- * concatenation of @a src_parent_dir and @a src_entry.
+ * @a tgt_root.  @a src_entry is the node to update.  If @a src_entry
+ * is empty, then compute the difference between the entire tree
+ * anchored at @a src_parent_dir under @a src_root and @a tgt_path
+ * under @a target_root.  Else, describe the changes needed to update
+ * only that entry in @a src_parent_dir.  Typically, callers of this
+ * function will use a @a tgt_path that is the concatenation of @a
+ * src_parent_dir and @a src_entry.
  *
  * @a src_root and @a tgt_root can both be either revision or transaction
  * roots.  If @a tgt_root is a revision, @a editor's @c set_target_revision()
  * will be called with the @a tgt_root's revision number, else it will
  * not be called at all.
  *
+ * If @a authz_read_func is non-null, invoke it before any call to
+ *
+ *    @a editor->open_root
+ *    @a editor->add_directory
+ *    @a editor->open_directory
+ *    @a editor->add_file
+ *    @a editor->open_file
+ *
+ * passing @a tgt_root, the same path that would be passed to the
+ * editor function in question, and @a authz_read_baton.  If the
+ * @a *allowed parameter comes back TRUE, then proceed with the planned
+ * editor call; else if FALSE, then invoke @a editor->absent_file or
+ * @a editor->absent_directory as appropriate, except if the planned
+ * editor call was open_root, throw SVN_ERR_AUTHZ_ROOT_UNREADABLE.
+ *
  * If @a text_deltas is @c FALSE, send a single @c NULL txdelta window to 
  * the window handler returned by @a editor->apply_textdelta().
  *
@@ -321,6 +405,8 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
                      const char *tgt_path,
                      const svn_delta_editor_t *editor,
                      void *edit_baton,
+                     svn_repos_authz_func_t authz_read_func,
+                     void *authz_read_baton,
                      svn_boolean_t text_deltas,
                      svn_boolean_t recurse,
                      svn_boolean_t entry_props,
@@ -336,6 +422,16 @@ svn_repos_dir_delta (svn_fs_root_t *src_root,
  * apply_textdelta() functions will not contain meaningful data, and
  * merely serve as indications that properties or textual contents
  * were changed. 
+ *
+ * NOTE: this editor driver passes SVN_INVALID_REVNUM for all
+ * revision parameters in the editor interface except the copyfrom
+ * parameter of the add_file() and add_directory() editor functions.
+ *
+ * ### TODO: This ought to take an svn_repos_authz_func_t too.
+ * The only reason it doesn't yet is the difficulty of implementing
+ * that correctly, plus lack of strong present need -- it's currently
+ * only used in creating a DAV MERGE response, in 'svnadmin dump', and
+ * in svnlook.
  */
 svn_error_t *
 svn_repos_replay (svn_fs_root_t *root,
@@ -348,18 +444,6 @@ svn_repos_replay (svn_fs_root_t *root,
 
 /* Making commits. */
 
-/** Callback function type for commits.
- *
- * When a filesystem commit succeeds, an instance of this is invoked on 
- * the @a new_revision, @a date, and @a author of the commit, along with 
- * the @a baton closure.
- */
-typedef svn_error_t * (*svn_repos_commit_callback_t) (
-    svn_revnum_t new_revision,
-    const char *date,
-    const char *author,
-    void *baton);
-
 /** Return an @a editor and @a edit_baton to commit changes to @a session->fs,
  * beginning at location 'rev:@a base_path', where "rev" is the argument
  * given to @c open_root().  Store @a user as the author of the commit and
@@ -373,9 +457,9 @@ typedef svn_error_t * (*svn_repos_commit_callback_t) (
  * invoke @a callback with the new revision number, the commit date (as a
  * <tt>const char *</tt>), commit author (as a <tt>const char *</tt>), and
  * @a callback_baton as arguments.  If @a callback returns an error, that
- * error will be returned from @c close_edit, otherwise @c close_edit will
- * return successfully (unless it encountered an error before invoking
- * @a callback).
+ * error will be returned from @c close_edit, otherwise if there was a
+ * post-commit hook failure, then that error will be returned and will
+ * have code SVN_ERR_REPOS_POST_COMMIT_HOOK_FAILED.
  */
 svn_error_t *svn_repos_get_commit_editor (const svn_delta_editor_t **editor,
                                           void **edit_baton,
@@ -384,7 +468,7 @@ svn_error_t *svn_repos_get_commit_editor (const svn_delta_editor_t **editor,
                                           const char *base_path,
                                           const char *user,
                                           const char *log_msg,
-                                          svn_repos_commit_callback_t hook,
+                                          svn_commit_callback_t callback,
                                           void *callback_baton,
                                           apr_pool_t *pool);
 
@@ -424,6 +508,35 @@ svn_repos_get_committed_info (svn_revnum_t *committed_rev,
                               const char *path,
                               apr_pool_t *pool);
 
+
+/** Callback type for use with svn_repos_history().  @a path and @a
+ * revision represent interesting history locations in the lifetime
+ * of the path passed to svn_repos_history().  @a baton is the same
+ * baton given to svn_repos_history().  @a pool is provided for the
+ * convenience of the implementor, who should not expect it to live
+ * longer than a single callback call.
+ */
+typedef svn_error_t *(*svn_repos_history_func_t) (void *baton,
+                                                  const char *path,
+                                                  svn_revnum_t revision,
+                                                  apr_pool_t *pool);
+
+/** Call @a history_func (with @a history_baton) for each interesting
+ * history location in the lifetime of @a path in @a fs, from the
+ * youngest of @a end and @ start to the oldest.  Only cross
+ * filesystem copy history if @a cross_copies is @c TRUE.  And do all
+ * of this in @a pool.
+ */
+svn_error_t *
+svn_repos_history (svn_fs_t *fs,
+                   const char *path,
+                   svn_repos_history_func_t history_func,
+                   void *history_baton,
+                   svn_revnum_t start,
+                   svn_revnum_t end,
+                   svn_boolean_t cross_copies,
+                   apr_pool_t *pool);
+
 /* ### other queries we can do someday --
 
      * fetch the last revision created by <user>
@@ -501,15 +614,22 @@ svn_repos_get_logs (svn_repos_t *repos,
  */
 
 /** Like @c svn_fs_commit_txn(), but invoke the @a repos's pre- and
- * post-commit hooks around the commit.  Use @a txn's pool for temporary
+ * post-commit hooks around the commit.  Use @a pool for any necessary
  * allocations.
  *
+ * If the pre-commit hook or svn_fs_commit_txn() fails, throw the
+ * original error to caller.  If an error occurs when running the
+ * post-commit hook, return the original error wrapped with
+ * SVN_ERR_REPOS_POST_COMMIT_HOOK_FAILED.  If the caller sees this
+ * error, it knows that the commit succeeded anyway.
+ *
  * @a conflict_p, @a new_rev, and @a txn are as in @c svn_fs_commit_txn().
  */
 svn_error_t *svn_repos_fs_commit_txn (const char **conflict_p,
                                       svn_repos_t *repos,
                                       svn_revnum_t *new_rev,
-                                      svn_fs_txn_t *txn);
+                                      svn_fs_txn_t *txn,
+                                      apr_pool_t *pool);
 
 /** Like @c svn_fs_begin_txn(), but use @a author and @a log_msg to set the
  * corresponding properties on transaction @a *txn_p.  @a repos is the
@@ -639,6 +759,9 @@ typedef struct svn_repos_node_t
   /** Pointer to the first child of this node */
   struct svn_repos_node_t *child;
 
+  /** Pointer to the parent of this node */
+  struct svn_repos_node_t *parent;
+
 } svn_repos_node_t;
 
 
@@ -731,13 +854,14 @@ enum svn_repos_load_uuid
 {
   svn_repos_load_uuid_default,
   svn_repos_load_uuid_ignore,
-  svn_repos_load_uuid_force,
+  svn_repos_load_uuid_force
 };
 
 /** Dump the contents of the filesystem within already-open @a repos into
  * writable @a dumpstream.  Begin at revision @a start_rev, and dump every
  * revision up through @a end_rev.  Use @a pool for all allocation.  If
- * non-@c NULL, send feedback to @a feedback_stream.
+ * non-@c NULL, send feedback to @a feedback_stream. @a dumpstream can be
+ * @c NULL for the purpose of verifying the repository.
  *
  * If @a start_rev is @c SVN_INVALID_REVNUM, then start dumping at revision 
  * 0.  If @a end_rev is @c SVN_INVALID_REVNUM, then dump through the @c HEAD 
@@ -746,6 +870,10 @@ enum svn_repos_load_uuid
  * If @a incremental is @c TRUE, the first revision dumped will be a diff
  * against the previous revision (usually it looks like a full dump of
  * the tree).
+ *
+ * If @a cancel_func is not @c NULL, it is called periodically with
+ * @a cancel_baton as argument to see if the client wishes to cancel
+ * the dump.
  */
 svn_error_t *svn_repos_dump_fs (svn_repos_t *repos,
                                 svn_stream_t *dumpstream,
@@ -753,6 +881,8 @@ svn_error_t *svn_repos_dump_fs (svn_repos_t *repos,
                                 svn_revnum_t start_rev,
                                 svn_revnum_t end_rev,
                                 svn_boolean_t incremental,
+                                svn_cancel_func_t cancel_func,
+                                void *cancel_baton,
                                 apr_pool_t *pool);
 
 
@@ -775,12 +905,18 @@ svn_error_t *svn_repos_dump_fs (svn_repos_t *repos,
  *
  * If the dumpstream contains no UUID, then @a uuid_action is
  * ignored and the repository UUID is not touched.
+ *
+ * If @a cancel_func is not @c NULL, it is called periodically with
+ * @a cancel_baton as argument to see if the client wishes to cancel
+ * the load.
  */
 svn_error_t *svn_repos_load_fs (svn_repos_t *repos,
                                 svn_stream_t *dumpstream,
                                 svn_stream_t *feedback_stream,
                                 enum svn_repos_load_uuid uuid_action,
                                 const char *parent_dir,
+                                svn_cancel_func_t cancel_func,
+                                void *cancel_baton,
                                 apr_pool_t *pool);
 
 
@@ -860,6 +996,10 @@ typedef struct svn_repos_parse_fns_t
 /** Read and parse dumpfile-formatted @a stream, calling callbacks in
  * @a parse_fns/@a parse_baton, and using @a pool for allocations.
  *
+ * If @a cancel_func is not @c NULL, it is called periodically with
+ * @a cancel_baton as argument to see if the client wishes to cancel
+ * the dump.
+ *
  * This parser has built-in knowledge of the dumpfile format, but only
  * in a general sense:
  *
@@ -879,6 +1019,8 @@ svn_error_t *
 svn_repos_parse_dumpstream (svn_stream_t *stream,
                             const svn_repos_parser_fns_t *parse_fns,
                             void *parse_baton,
+                            svn_cancel_func_t cancel_func,
+                            void *cancel_baton,
                             apr_pool_t *pool);
 
 
