@@ -2,7 +2,7 @@
  * copy.c:  copy/move wrappers around wc 'copy' functionality.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -60,15 +60,10 @@
 /* Copy SRC_PATH into DST_PATH as DST_BASENAME, deleting SRC_PATH
    afterwards if IS_MOVE is TRUE.  Use POOL for all necessary
    allocations.
-
-   ### 838 OPTIONAL_ADM_ACCESS is an access baton with a write lock for the
-   ### parent of DST_PATH. This parameter should be removed when issue 838
-   ### stops using svn_client_copy.
 */
 static svn_error_t *
 wc_to_wc_copy (const char *src_path,
                const char *dst_path,
-               svn_wc_adm_access_t *optional_adm_access,
                svn_boolean_t is_move,
                svn_boolean_t force,
                svn_client_ctx_t *ctx,
@@ -82,7 +77,7 @@ wc_to_wc_copy (const char *src_path,
   SVN_ERR (svn_io_check_path (src_path, &src_kind, pool));
   if (src_kind == svn_node_none)
     return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                              "path `%s' does not exist.", src_path);
+                              "Path '%s' does not exist", src_path);
 
   /* If DST_PATH does not exist, then its basename will become a new
      file or dir added to its parent (possibly an implicit '.').  If
@@ -101,14 +96,11 @@ wc_to_wc_copy (const char *src_path,
     }
   else
     return svn_error_createf (SVN_ERR_ENTRY_EXISTS, NULL,
-                              "file `%s' already exists.", dst_path);
+                              "File '%s' already exists", dst_path);
 
   if (is_move)
     {
-      /* We don't handle optional_adm_access here.  The merge code that
-         sets it calls svn_client_copy rather than svn_client_move */
       const char *src_parent;
-      assert (! optional_adm_access);
 
       svn_path_split (src_path, &src_parent, NULL, pool);
 
@@ -144,14 +136,14 @@ wc_to_wc_copy (const char *src_path,
 
       if (!force)
         /* Ensure there are no "awkward" files. */
-        SVN_ERR_W (svn_client__can_delete (src_path, src_access, ctx, pool),
-                   "Pass --force to override this restriction");
+        SVN_ERR_W (svn_client__can_delete (src_path, ctx, pool),
+                   "Move will not be attempted unless forced");
+    }
+  else 
+    {
+      SVN_ERR (svn_wc_adm_open (&adm_access, NULL, dst_parent, TRUE, FALSE,
+                                pool));
     }
-  else if (! optional_adm_access)
-    SVN_ERR (svn_wc_adm_open (&adm_access, NULL, dst_parent, TRUE, FALSE,
-                              pool));
-  else
-    adm_access = optional_adm_access;
                               
   /* Perform the copy and (optionally) delete. */
 
@@ -174,8 +166,10 @@ wc_to_wc_copy (const char *src_path,
         SVN_ERR (svn_wc_adm_close (adm_access));
       SVN_ERR (svn_wc_adm_close (src_access));
     }
-  else if (! optional_adm_access)
-    SVN_ERR (svn_wc_adm_close (adm_access));
+  else
+    {
+      SVN_ERR (svn_wc_adm_close (adm_access));
+    }
 
   return SVN_NO_ERROR;
 }
@@ -208,6 +202,11 @@ path_driver_cb_func (void **dir_baton,
   /* Initialize return value. */
   *dir_baton = NULL;
 
+  /* This function should never get an empty PATH.  We can neither
+     create nor delete the empty PATH, so if someone is calling us
+     with such, the code is just plain wrong. */
+  assert (! svn_path_is_empty (path));
+
   /* If this is a resurrection, we know the source and dest paths are
      the same, and that our driver will only be calling us once.  */
   if (cb_baton->resurrection)
@@ -269,25 +268,22 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
                      const svn_opt_revision_t *src_revision, 
                      const char *dst_url, 
                      svn_client_ctx_t *ctx,
-                     const char *message,
                      svn_boolean_t is_move,
                      apr_pool_t *pool)
 {
   apr_array_header_t *paths = apr_array_make (pool, 2, sizeof (const char *));
-  const char *top_url, *src_rel, *dst_rel;
+  const char *top_url, *src_rel, *dst_rel, *message;
   svn_revnum_t youngest;
   void *ra_baton, *sess;
   svn_ra_plugin_t *ra_lib;
   svn_node_kind_t src_kind, dst_kind;
   const svn_delta_editor_t *editor;
   void *edit_baton;
-  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-  const char *committed_date = NULL;
-  const char *committed_author = NULL;
+  void *commit_baton;
   svn_revnum_t src_revnum;
-  const char *auth_dir;
   svn_boolean_t resurrection = FALSE;
   struct path_driver_cb_baton cb_baton;
+  svn_error_t *err;
 
   /* We have to open our session to the longest path common to both
      SRC_URL and DST_URL in the repository so we can do existence
@@ -319,20 +315,52 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
   else
     dst_rel = "";
 
+  /* We can't move something into itself, period. */
+  if (svn_path_is_empty (src_rel) && is_move)
+    return svn_error_createf (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                              "Cannot move URL '%s' into itself", src_url);
+
   /* Get the RA vtable that matches URL. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, top_url, pool));
+  err = svn_ra_get_ra_library (&ra_lib, ra_baton, top_url, pool);
 
-  /* Get the auth dir. */
-  SVN_ERR (svn_client__dir_if_wc (&auth_dir, "", pool));
+  /* If the two URLs appear not to be in the same repository, then
+     top_url will be empty and the call to svn_ra_get_ra_library()
+     above will have failed.  Below we check for that, and propagate a
+     descriptive error back to the user.
+   
+     Ideally, we'd contact the repositories and compare their UUIDs to
+     determine whether or not src and dst are in the same repository,
+     instead of depending on an essentially textual comparison.
+     However, it is simpler to assume that if someone is using the
+     same repository, then they will use the same hostname/path to
+     refer to it both times.  Conversely, if the repositories are
+     different, then they can't share a non-empty prefix, so top_url
+     would still be "" and svn_ra_get_library() would still error.
+     Thus we can get this check without extra network turnarounds to
+     fetch the UUIDs.
+   */
+  if (err)
+    {
+      if ((err->apr_err == SVN_ERR_RA_ILLEGAL_URL)
+          && ((top_url == NULL) || (top_url[0] == '\0')))
+        {
+          return svn_error_createf
+            (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+             "Source and dest appear not to be in the same repository "
+	     "(src: '%s'; dst: '%s')",
+             src_url, dst_url);
+        }
+      else
+        return err;
+    }
 
   /* Open an RA session for the URL. Note that we don't have a local
-     directory, nor a place to put temp files or store the auth data. */
+     directory, nor a place to put temp files. */
   SVN_ERR (svn_client__open_ra_session (&sess, ra_lib, top_url,
-                                        auth_dir,
+                                        NULL,
                                         NULL, NULL, FALSE, TRUE, 
                                         ctx, pool));
-
   /* Pass NULL for the path, to ensure error if trying to get a
      revision based on the working copy. */
   SVN_ERR (svn_client__get_revision_number
@@ -346,15 +374,15 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
     src_revnum = youngest;
   
   /* Verify that SRC_URL exists in the repository. */
-  SVN_ERR (ra_lib->check_path (&src_kind, sess, src_rel, src_revnum, pool));
+  SVN_ERR (ra_lib->check_path (sess, src_rel, src_revnum, &src_kind, pool));
   if (src_kind == svn_node_none)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FOUND, NULL,
-       "path `%s' does not exist in revision `%" SVN_REVNUM_T_FMT "'",
+       "Path '%s' does not exist in revision '%" SVN_REVNUM_T_FMT "'",
        src_url, src_revnum);
 
   /* Figure out the basename that will result from this operation. */
-  SVN_ERR (ra_lib->check_path (&dst_kind, sess, dst_rel, youngest, pool));
+  SVN_ERR (ra_lib->check_path (sess, dst_rel, youngest, &dst_kind, pool));
   if (dst_kind == svn_node_none)
     {
       /* do nothing */
@@ -363,7 +391,7 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
     {
       /* We disallow the overwriting of files. */
       return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                "fs path `%s' already exists.", dst_rel);
+                                "Path '%s' already exists.", dst_rel);
     }
   else if (dst_kind == svn_node_dir)
     {
@@ -375,24 +403,45 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
 
       bname = svn_path_uri_decode (svn_path_basename (src_url, pool), pool);
       dst_rel = svn_path_join (dst_rel, bname, pool);
-      SVN_ERR (ra_lib->check_path (&attempt_kind, sess, 
-                                   dst_rel, youngest, pool));
+      SVN_ERR (ra_lib->check_path (sess, dst_rel, youngest,
+                                   &attempt_kind, pool));
       if (attempt_kind != svn_node_none)
         return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                  "fs path `%s' already exists.", dst_rel);
+                                  "Path '%s' already exists", dst_rel);
     }
   else
     {
       return svn_error_createf (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-                                "unrecognized node kind of '%s'.", dst_url);
+                                "Unrecognized node kind of '%s'", dst_url);
     }
 
+  /* Create a new commit item and add it to the array. */
+  if (ctx->log_msg_func)
+    {
+      svn_client_commit_item_t *item;
+      const char *tmp_file;
+      apr_array_header_t *commit_items 
+        = apr_array_make (pool, 1, sizeof (item));
+      
+      item = apr_pcalloc (pool, sizeof (*item));
+      item->url = svn_path_join (top_url, dst_rel, pool);
+      item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
+      (*((svn_client_commit_item_t **) apr_array_push (commit_items))) = item;
+      
+      SVN_ERR ((*ctx->log_msg_func) (&message, &tmp_file, commit_items, 
+                                     ctx->log_msg_baton, pool));
+      if (! message)
+        return SVN_NO_ERROR;
+    }
+  else
+    message = "";
+
+
   /* Fetch RA commit editor. */
-  SVN_ERR (ra_lib->get_commit_editor (sess, &editor, &edit_baton,
-                                      &committed_rev,
-                                      &committed_date,
-                                      &committed_author,
-                                      message, pool));
+  SVN_ERR (svn_client__commit_get_baton (&commit_baton, commit_info, pool));
+  SVN_ERR (ra_lib->get_commit_editor (sess, &editor, &edit_baton, message,
+                                      svn_client__commit_callback,
+                                      commit_baton, pool));
 
   /* Setup our PATHS for the path-based editor drive. */
   APR_ARRAY_PUSH (paths, const char *) = dst_rel;
@@ -411,18 +460,18 @@ repos_to_repos_copy (svn_client_commit_info_t **commit_info,
   cb_baton.resurrection = resurrection;
 
   /* Call the path-based editor driver. */
-  SVN_ERR (svn_delta_path_driver (editor, edit_baton, youngest, paths,
-                                  path_driver_cb_func, &cb_baton, pool));
+  err = svn_delta_path_driver (editor, edit_baton, youngest, paths,
+                               path_driver_cb_func, &cb_baton, pool);
+  if (err)
+    {
+      /* At least try to abort the edit (and fs txn) before throwing err. */
+      svn_error_clear (editor->abort_edit (edit_baton, pool));
+      return err;
+    }
 
   /* Close the edit. */
   SVN_ERR (editor->close_edit (edit_baton, pool));
 
-  /* Fill in the commit_info structure. */
-  *commit_info = svn_client__make_commit_info (committed_rev,
-                                               committed_author,
-                                               committed_date,
-                                               pool);
-
   return SVN_NO_ERROR;
 }
 
@@ -521,18 +570,15 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
                   const char *src_path, 
                   const char *dst_url, 
                   svn_client_ctx_t *ctx,
-                  const char *message,
                   apr_pool_t *pool)
 {
-  const char *anchor, *target, *parent, *base_name;
+  const char *anchor, *target, *base_name, *message;
   void *ra_baton, *session;
   svn_ra_plugin_t *ra_lib;
   const svn_delta_editor_t *editor;
   void *edit_baton;
   svn_node_kind_t src_kind, dst_kind;
-  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-  const char *committed_date = NULL;
-  const char *committed_author = NULL;
+  void *commit_baton;
   apr_hash_t *committables, *tempfiles = NULL;
   svn_wc_adm_access_t *adm_access, *dir_access;
   apr_array_header_t *commit_items;
@@ -542,15 +588,14 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
   svn_boolean_t commit_in_progress = FALSE;
   const char *base_path;
   const char *base_url;
-  const char *auth_dir;
 
   /* The commit process uses absolute paths, so we need to open the access
      baton using absolute paths, and so we really need to use absolute
      paths everywhere. */
   SVN_ERR (svn_path_get_absolute (&base_path, src_path, pool));
 
-  svn_path_split (base_path, &parent, &base_name, pool);
-  SVN_ERR (svn_wc_adm_open (&adm_access, NULL, parent, FALSE, TRUE, pool));
+  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, base_path, FALSE, TRUE,
+                                  pool));
 
   /* Split the DST_URL into an anchor and target. */
   svn_path_split (dst_url, &anchor, &target, pool);
@@ -560,26 +605,27 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
   SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, anchor, pool));
 
   /* Open an RA session for the anchor URL. */
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, anchor, parent,
+  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, anchor,
+                                        svn_wc_adm_access_path (adm_access),
                                         adm_access, NULL, TRUE, TRUE, 
                                         ctx, pool));
 
   /* Figure out the basename that will result from this operation. */
-  SVN_ERR (ra_lib->check_path (&dst_kind, session, 
-                               svn_path_uri_decode (target, pool),
-                               SVN_INVALID_REVNUM, pool));
+  SVN_ERR (ra_lib->check_path (session, svn_path_uri_decode (target, pool),
+                               SVN_INVALID_REVNUM, &dst_kind, pool));
   
   /* BASE_URL defaults to DST_URL. */
   base_url = apr_pstrdup (pool, dst_url);
   if (dst_kind == svn_node_none)
     {
-      /* DST_URL doesn't exist under it's parent URL, so the URL we
+      /* DST_URL doesn't exist under its parent URL, so the URL we
          will be creating is DST_URL. */
     }
   else if (dst_kind == svn_node_dir)
     {
       /* DST_URL is an existing directory URL.  The URL we will be
          creating, then, is DST_URL+BASENAME. */
+      svn_path_split (base_path, NULL, &base_name, pool);
       base_url = svn_path_url_add_component (base_url, base_name, pool);
     }
   else
@@ -587,9 +633,29 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
       /* DST_URL is an existing file, which can't be overwritten or
          used as a container, so error out. */
       return svn_error_createf (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-                                "file `%s' already exists.", dst_url);
+                                "File '%s' already exists", dst_url);
     }
 
+  /* Create a new commit item and add it to the array. */
+  if (ctx->log_msg_func)
+    {
+      svn_client_commit_item_t *item;
+      const char *tmp_file;
+
+      commit_items = apr_array_make (pool, 1, sizeof (item));      
+      item = apr_pcalloc (pool, sizeof (*item));
+      item->url = base_url;
+      item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
+      (*((svn_client_commit_item_t **) apr_array_push (commit_items))) = item;
+      
+      SVN_ERR ((*ctx->log_msg_func) (&message, &tmp_file, commit_items, 
+                                     ctx->log_msg_baton, pool));
+      if (! message)
+        return SVN_NO_ERROR;
+    }
+  else
+    message = "";
+
   /* Crawl the working copy for commit items. */
   SVN_ERR (svn_io_check_path (base_path, &src_kind, pool));
   if (src_kind == svn_node_dir)
@@ -621,17 +687,18 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
     goto cleanup;
 
   /* Open an RA session to BASE_URL. */
-  SVN_ERR (svn_client__default_auth_dir (&auth_dir, base_path, pool));
   if ((cmt_err = svn_client__open_ra_session (&session, ra_lib, base_url,
-                                              auth_dir, NULL, commit_items,
+                                              NULL, NULL, commit_items,
                                               FALSE, FALSE,
                                               ctx, pool)))
     goto cleanup;
 
   /* Fetch RA commit editor. */
+  SVN_ERR (svn_client__commit_get_baton (&commit_baton, commit_info, pool));
   if ((cmt_err = ra_lib->get_commit_editor (session, &editor, &edit_baton, 
-                                            &committed_rev, &committed_date, 
-                                            &committed_author, message, pool)))
+                                            message,
+                                            svn_client__commit_callback,
+                                            commit_baton, pool)))
     goto cleanup;
 
   /* Make a note that we have a commit-in-progress. */
@@ -651,9 +718,9 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
  cleanup:
   /* Abort the commit if it is still in progress. */
   if (commit_in_progress)
-    editor->abort_edit (edit_baton, pool); /* ignore return value */
+    svn_error_clear (editor->abort_edit (edit_baton, pool));
 
-  /* ### Under what conditions should we remove the locks? */
+  /* It's only a read lock, so unlocking is harmless. */
   unlock_err = svn_wc_adm_close (adm_access);
 
   /* Remove any outstanding temporary text-base files. */
@@ -662,25 +729,14 @@ wc_to_repos_copy (svn_client_commit_info_t **commit_info,
                                    ctx->cancel_func, ctx->cancel_baton,
                                    pool);
 
-  /* Fill in the commit_info structure */
-  *commit_info = svn_client__make_commit_info (committed_rev, 
-                                               committed_author, 
-                                               committed_date, pool);
-
   return reconcile_errors (cmt_err, unlock_err, cleanup_err, pool);
 }
 
 
-/*
-   ### 838 OPTIONAL_ADM_ACCESS is an access baton with a write lock for the
-   ### parent of DST_PATH. This parameter should be removed when issue 838
-   ### stops using svn_client_copy.
-*/
 static svn_error_t *
 repos_to_wc_copy (const char *src_url,
                   const svn_opt_revision_t *src_revision,
                   const char *dst_path, 
-                  svn_wc_adm_access_t *optional_adm_access,
                   svn_client_ctx_t *ctx,
                   apr_pool_t *pool)
 {
@@ -689,24 +745,18 @@ repos_to_wc_copy (const char *src_url,
   svn_node_kind_t src_kind, dst_kind;
   svn_revnum_t src_revnum;
   svn_wc_adm_access_t *adm_access;
-  apr_hash_t *props = NULL;
-  apr_hash_index_t *hi;
   const char *src_uuid = NULL, *dst_uuid = NULL;
   svn_boolean_t same_repositories;
-  const char *auth_dir;
-  svn_opt_revision_t *revision = apr_pcalloc (pool, sizeof(*revision));
+  svn_opt_revision_t revision;
 
   /* Get the RA vtable that matches URL. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
   SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, src_url, pool));
 
-  SVN_ERR (svn_client__default_auth_dir (&auth_dir, dst_path, pool));
-
   /* Open a repository session to the given URL. We do not (yet) have a
      working copy, so we don't have a corresponding path and tempfiles
-     cannot go into the admin area. We do want to store the resulting
-     auth data, though, once the WC is built. */
-  SVN_ERR (svn_client__open_ra_session (&sess, ra_lib, src_url, auth_dir,
+     cannot go into the admin area. */
+  SVN_ERR (svn_client__open_ra_session (&sess, ra_lib, src_url, NULL,
                                         NULL, NULL, FALSE, TRUE, 
                                         ctx, pool));
       
@@ -714,27 +764,27 @@ repos_to_wc_copy (const char *src_url,
      revision based on the working copy.  And additionally, we can't
      pass an 'unspecified' revnum to the update reporter;  assume HEAD
      if not specified. */
-  revision->kind = src_revision->kind;
-  revision->value = src_revision->value;
-  if (revision->kind == svn_opt_revision_unspecified)
-    revision->kind = svn_opt_revision_head;
+  revision.kind = src_revision->kind;
+  revision.value = src_revision->value;
+  if (revision.kind == svn_opt_revision_unspecified)
+    revision.kind = svn_opt_revision_head;
 
   SVN_ERR (svn_client__get_revision_number
-           (&src_revnum, ra_lib, sess, revision, NULL, pool));
+           (&src_revnum, ra_lib, sess, &revision, NULL, pool));
 
   /* Verify that SRC_URL exists in the repository. */
-  SVN_ERR (ra_lib->check_path (&src_kind, sess, "", src_revnum, pool));
+  SVN_ERR (ra_lib->check_path (sess, "", src_revnum, &src_kind, pool));
   if (src_kind == svn_node_none)
     {
       if (SVN_IS_VALID_REVNUM (src_revnum))
         return svn_error_createf
           (SVN_ERR_FS_NOT_FOUND, NULL,
-           "path `%s' not found in revision `%" SVN_REVNUM_T_FMT "'",
+           "Path '%s' not found in revision '%" SVN_REVNUM_T_FMT "'",
            src_url, src_revnum);
       else
         return svn_error_createf
           (SVN_ERR_FS_NOT_FOUND, NULL,
-           "path `%s' not found in head revision", src_url);
+           "Path '%s' not found in head revision", src_url);
     }
 
   /* There are two interfering sets of cases to watch out for here:
@@ -771,7 +821,7 @@ repos_to_wc_copy (const char *src_url,
   else if (dst_kind != svn_node_none)  /* must be a file */
     {
       return svn_error_createf (SVN_ERR_ENTRY_EXISTS, NULL,
-                                "file `%s' already exists.", dst_path);
+                                "File '%s' already exists", dst_path);
     }
 
   /* Now that dst_path has possibly been reset, check that there's
@@ -779,13 +829,24 @@ repos_to_wc_copy (const char *src_url,
   SVN_ERR (svn_io_check_path (dst_path, &dst_kind, pool));
   if (dst_kind != svn_node_none)
     return svn_error_createf (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                              "`%s' is in the way", dst_path);
+                              "'%s' is in the way", dst_path);
 
-  if (! optional_adm_access)
-    SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, dst_path, TRUE, FALSE,
-                                    pool));
-  else
-    adm_access = optional_adm_access;
+  SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, dst_path, TRUE, FALSE,
+                                  pool));
+
+  /* We've already checked for physical obstruction by a working file.
+     But there could also be logical obstruction by an entry whose
+     working file happens to be missing.*/ 
+  {
+    const svn_wc_entry_t *ent;
+
+    SVN_ERR (svn_wc_entry (&ent, dst_path, adm_access, FALSE, pool));
+    if (ent && (ent->kind != svn_node_dir))
+      return svn_error_createf
+        (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
+         "Entry for '%s' exists (though the working file is missing)",
+         dst_path);
+  }
 
   /* Decide whether the two repositories are the same or not. */
   { 
@@ -819,35 +880,10 @@ repos_to_wc_copy (const char *src_url,
 
   if (src_kind == svn_node_dir)
     {    
-      const svn_delta_editor_t *editor;
-      void *edit_baton;
-      const svn_ra_reporter_t *reporter;
-      void *report_baton;
+      SVN_ERR (svn_client__checkout_internal
+               (NULL, src_url, dst_path, &revision, TRUE, NULL, ctx, pool));
 
-      /* Get a checkout editor and wrap it. */
-      SVN_ERR (svn_wc_get_checkout_editor (dst_path, src_url, src_revnum, 1,
-                                           ctx->notify_func, ctx->notify_baton,
-                                           ctx->cancel_func, ctx->cancel_baton,
-                                           &editor, &edit_baton,
-                                           NULL, pool));
-      
-      /* Check out the new tree.  The parent dir will get no entry, so
-         it will be as if the new tree isn't really there yet. */
-      SVN_ERR (ra_lib->do_update (sess,
-                                  &reporter, &report_baton,
-                                  src_revnum,
-                                  NULL, /* no sub-target */
-                                  TRUE, /* recurse */
-                                  editor, edit_baton, pool));
-
-      SVN_ERR (reporter->set_path (report_baton, "", src_revnum,
-                                   TRUE, /* "help, my dir is empty!" */
-                                   pool));
-
-      SVN_ERR (reporter->finish_report (report_baton));               
-
-      if ((! SVN_IS_VALID_REVNUM (src_revnum))
-          && same_repositories)
+      if ((revision.kind == svn_opt_revision_head) && same_repositories)
         {
           /* If we just checked out from the "head" revision, that's fine,
              but we don't want to pass a '-1' as a copyfrom_rev to
@@ -872,64 +908,20 @@ repos_to_wc_copy (const char *src_url,
           src_revnum = d_entry->revision;
         }
 
-    } /* end directory case */
-
-  else if (src_kind == svn_node_file)
-    {
-      apr_status_t status;
-      svn_stream_t *fstream;
-      apr_file_t *fp;
-      svn_revnum_t fetched_rev = 0;
-      
-      /* Open DST_PATH for writing. */
-      SVN_ERR_W (svn_io_file_open (&fp, dst_path,
-                                   (APR_CREATE | APR_WRITE),
-                                   APR_OS_DEFAULT, pool),
-                 "failed to open file for writing.");
-
-      /* Create a generic stream that operates on this file.  */
-      fstream = svn_stream_from_aprfile (fp, pool);
-      
-      /* Have the RA layer 'push' data at this stream.  We pass a
-         relative path of "", because we opened SRC_URL, which is
-         already the full URL to the file. */         
-      SVN_ERR (ra_lib->get_file (sess, "", src_revnum, fstream, 
-                                 &fetched_rev, &props, pool));
-
-      /* Close the file. */
-      status = apr_file_close (fp);
-      if (status)
-        return svn_error_createf (status, NULL,
-                                  "failed to close file '%s'.",
-                                  dst_path);   
-     
-      /* Also, if SRC_REVNUM is invalid ('head'), then FETCHED_REV is now
-         equal to the revision that was actually retrieved.  This is
-         the value we want to use as 'copyfrom_rev' in the call to
-         svn_wc_add() below. */
-      if (! SVN_IS_VALID_REVNUM (src_revnum))
-        src_revnum = fetched_rev;
-    }
-  
-  /* Schedule the new item for addition, or addition-with-history.
-     
-     If the new item is a directory, the URLs will be recursively
-     rewritten, wcprops removed, and everything marked as 'copied',
-     assuming that the src and dst are from the same repository.
-     See comment in svn_wc_add()'s doc about whether svn_wc_add is the
-     appropriate place for this. */
-  if (same_repositories)
-    {
-      /* Schedule dst_path for addition in parent, with copy history.
-         (This function also recursively puts a 'copied' flag on every
-         entry) */
-      SVN_ERR (svn_wc_add (dst_path, adm_access, src_url, src_revnum,
-                           ctx->cancel_func, ctx->cancel_baton, 
-                           ctx->notify_func, ctx->notify_baton, pool));
-    }
-  else
-    {
-      if (src_kind == svn_node_dir)
+      /* Rewrite URLs recursively, remove wcprops, and mark everything
+         as 'copied' -- assuming that the src and dst are from the
+         same repository.  (It's kind of weird that svn_wc_add() is the
+         way to do this; see its doc for more about the controversy.) */
+      if (same_repositories)
+        {
+          /* Schedule dst_path for addition in parent, with copy history.
+             (This function also recursively puts a 'copied' flag on every
+             entry). */
+          SVN_ERR (svn_wc_add (dst_path, adm_access, src_url, src_revnum,
+                               ctx->cancel_func, ctx->cancel_baton, 
+                               ctx->notify_func, ctx->notify_baton, pool));
+        }
+      else  /* different repositories */
         {
           /* ### Someday, we would just call svn_wc_add(), as above,
              but with no copyfrom args.  I.e. in the
@@ -938,72 +930,70 @@ repos_to_wc_copy (const char *src_url,
              deleted, but WITHOUT any copied flags or copyfrom urls.
              Unfortunately, svn_wc_add() is such a mess that it chokes
              at the moment when we pass a NULL copyfromurl. */
-
+          
           return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-                                   "Source URL is from foreign repository.");
+                                   "Source URL is from foreign repository");
         }
-      
-      /* Recursively schedule unversioned tree for addition, sans history. */
-      SVN_ERR (svn_client__add (dst_path, TRUE /* recursive */,
-                                adm_access, ctx, pool));
-    }
+    } /* end directory case */
 
-  /* If any properties were fetched (in the file case), apply those
-     changes now. */
-  if (props)
+  else if (src_kind == svn_node_file)
     {
-      for (hi = apr_hash_first(pool, props); hi; hi = apr_hash_next(hi)) 
-        {
-          const void *key;
-          void *val;
-          enum svn_prop_kind kind;
-          
-          apr_hash_this (hi, &key, NULL, &val);
-          
-          /* We only want to set 'normal' props.  For now, we're
-             ignoring any wc props (they're not needed when we commit
-             an addition), and we're ignoring entry props (they're
-             written to the entries file as part of the post-commit
-             processing).  */
-          kind = svn_property_kind (NULL, key);
-          if (kind == svn_prop_regular_kind)
-            SVN_ERR (svn_wc_prop_set (key, val, dst_path, adm_access, pool));
-        }
-    }
+      apr_file_t *fp;
+      svn_stream_t *fstream;
+      svn_revnum_t real_rev;
+      const char *new_text_path;
+      apr_hash_t *new_props;
 
-  if (! optional_adm_access)
-    SVN_ERR (svn_wc_adm_close (adm_access));
+      SVN_ERR (svn_io_open_unique_file
+               (&fp, &new_text_path, dst_path, ".tmp", FALSE, pool));
+
+      fstream = svn_stream_from_aprfile (fp, pool);
+      SVN_ERR (ra_lib->get_file
+               (sess, "", src_revnum, fstream, &real_rev, &new_props, pool));
+      svn_stream_close (fstream);
+      SVN_ERR (svn_io_file_close (fp, pool));
+
+      /* If SRC_REVNUM is invalid (HEAD), then REAL_REV is now the
+         revision that was actually retrieved.  This is the value we
+         want to use as 'copyfrom_rev' below. */
+      if (! SVN_IS_VALID_REVNUM (src_revnum))
+        src_revnum = real_rev;
+
+      SVN_ERR (svn_wc_add_repos_file
+               (dst_path, adm_access,
+                new_text_path, new_props,
+                same_repositories ? src_url : NULL,
+                same_repositories ? src_revnum : SVN_INVALID_REVNUM,
+                pool));
+    }
+  
+  SVN_ERR (svn_wc_adm_close (adm_access));
 
   return SVN_NO_ERROR;
 }
 
-/*
-   ### 838 OPTIONAL_ADM_ACCESS is an access baton with a write lock for the
-   ### parent of DST_PATH. This parameter should be removed when issue 838
-   ### stops using svn_client_copy.
-*/
+
 static svn_error_t *
 setup_copy (svn_client_commit_info_t **commit_info,
             const char *src_path,
             const svn_opt_revision_t *src_revision,
             const char *dst_path,
-            svn_wc_adm_access_t *optional_adm_access,
             svn_boolean_t is_move,
             svn_boolean_t force,
             svn_client_ctx_t *ctx,
             apr_pool_t *pool)
 {
   svn_boolean_t src_is_url, dst_is_url;
-  const char *message;
 
   /* Are either of our paths URLs? */
   src_is_url = svn_path_is_url (src_path);
   dst_is_url = svn_path_is_url (dst_path);
 
-  if (svn_path_is_child (src_path, dst_path, pool))
+  if (!src_is_url && !dst_is_url
+      && svn_path_is_child (src_path, dst_path, pool))
     return svn_error_createf
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-       "cannot copy path '%s' into its own child '%s'",
+       "Cannot copy path '%s' into its own child '%s'",
        src_path, dst_path);
 
   if (is_move)
@@ -1013,7 +1003,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
           if (strcmp (src_path, dst_path) == 0)
             return svn_error_createf
               (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-               "cannot move path '%s' into itself",
+               "Cannot move path '%s' into itself",
                src_path);
         }
       else
@@ -1021,7 +1011,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
           /* Disallow moves between the working copy and the repository. */
           return svn_error_create 
             (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-             "no support for repos <--> working copy moves");
+             "No support for repos <--> working copy moves");
         }
 
       /* It doesn't make sense to specify revisions in a move. */
@@ -1036,7 +1026,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
         {
           return svn_error_create
             (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-             "cannot specify revisions with move operations");
+             "Cannot specify revisions with move operations");
         }
     }
   else
@@ -1044,7 +1034,7 @@ setup_copy (svn_client_commit_info_t **commit_info,
       if (!src_is_url)
         {
           if (src_revision->kind != svn_opt_revision_unspecified
-              && src_revision->kind != svn_opt_revision_head)
+              && src_revision->kind != svn_opt_revision_working)
             {
               /* We can convert the working copy path to a URL based on the
                  entries file. */
@@ -1055,58 +1045,41 @@ setup_copy (svn_client_commit_info_t **commit_info,
               SVN_ERR (svn_wc_entry (&entry, src_path, adm_access, FALSE,
                                      pool));
               SVN_ERR (svn_wc_adm_close (adm_access));
+
+              if (! entry)
+                return svn_error_createf
+                  (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                   "'%s' is not under version control", src_path);
+
               src_path = entry->url;
               src_is_url = TRUE;
             }
         }
     }
 
-  /* Create a new commit item and add it to the array. */
-  if (dst_is_url && ctx->log_msg_func)
-    {
-      svn_client_commit_item_t *item;
-      const char *tmp_file;
-      apr_array_header_t *commit_items 
-        = apr_array_make (pool, 1, sizeof (item));
-      
-      item = apr_pcalloc (pool, sizeof (*item));
-      item->url = apr_pstrdup (pool, dst_path);
-      item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
-      (*((svn_client_commit_item_t **) apr_array_push (commit_items))) 
-        = item;
-      
-      SVN_ERR ((*ctx->log_msg_func) (&message, &tmp_file, commit_items, 
-                                     ctx->log_msg_baton, pool));
-      if (! message)
-        return SVN_NO_ERROR;
-    }
-  else
-    message = "";
-
   /* Now, call the right handler for the operation. */
   if ((! src_is_url) && (! dst_is_url))
-    SVN_ERR (wc_to_wc_copy (src_path, dst_path, optional_adm_access,
-                            is_move, force,
-                            ctx,
-                            pool));
-
+    {
+      SVN_ERR (wc_to_wc_copy (src_path, dst_path,
+                              is_move, force,
+                              ctx,
+                              pool));
+    }
   else if ((! src_is_url) && (dst_is_url))
     {
       SVN_ERR (wc_to_repos_copy (commit_info, src_path, dst_path, 
-                                 ctx, message, 
-                                 pool));
+                                 ctx, pool));
     }
   else if ((src_is_url) && (! dst_is_url))
     {
       SVN_ERR (repos_to_wc_copy (src_path, src_revision, 
-                                 dst_path, optional_adm_access, ctx,
+                                 dst_path, ctx,
                                  pool));
     }
   else
     {
       SVN_ERR (repos_to_repos_copy (commit_info, src_path, src_revision,
-                                    dst_path, ctx, message, is_move,
-                                    pool));
+                                    dst_path, ctx, is_move, pool));
     }
 
   return SVN_NO_ERROR;
@@ -1116,22 +1089,16 @@ setup_copy (svn_client_commit_info_t **commit_info,
 
 /* Public Interfaces */
 
-/*
-   ### 838 OPTIONAL_ADM_ACCESS is an access baton with a write lock for the
-   ### parent of DST_PATH. This parameter should be removed when issue 838
-   ### stops using svn_client_copy.
-*/
 svn_error_t *
 svn_client_copy (svn_client_commit_info_t **commit_info,
                  const char *src_path,
                  const svn_opt_revision_t *src_revision,
                  const char *dst_path,
-                 svn_wc_adm_access_t *optional_adm_access,
                  svn_client_ctx_t *ctx,
                  apr_pool_t *pool)
 {
   return setup_copy (commit_info, 
-                     src_path, src_revision, dst_path, optional_adm_access,
+                     src_path, src_revision, dst_path,
                      FALSE /* is_move */,
                      TRUE /* force, set to avoid deletion check */,
                      ctx,
@@ -1149,7 +1116,7 @@ svn_client_move (svn_client_commit_info_t **commit_info,
                  apr_pool_t *pool)
 {
   return setup_copy (commit_info,
-                     src_path, src_revision, dst_path, NULL,
+                     src_path, src_revision, dst_path,
                      TRUE /* is_move */,
                      force,
                      ctx,
