@@ -2,7 +2,7 @@
  * prop_commands.c:  Implementation of propset, propget, and proplist.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,6 +22,8 @@
 
 /*** Includes. ***/
 
+#include <ctype.h>
+
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 
@@ -33,64 +35,98 @@
 
 /*** Code. ***/
 
-static svn_error_t *
-recursive_propset (const char *propname,
-                   const svn_string_t *propval,
-                   svn_wc_adm_access_t *adm_access,
-                   apr_pool_t *pool)
+/* Check whether the UTF8 NAME is a valid property name.  For now, this means
+ * the ASCII subset of an XML "Name".
+ * XML "Name" is defined at http://www.w3.org/TR/REC-xml#sec-common-syn */
+static svn_boolean_t
+is_valid_prop_name (const char *name)
 {
-  apr_hash_t *entries;
-  apr_hash_index_t *hi;
+  const char *p = name;
 
-  SVN_ERR (svn_wc_entries_read (&entries, adm_access, FALSE, pool));
-
-  for (hi = apr_hash_first (pool, entries); hi; hi = apr_hash_next (hi))
+  /* Each byte of a UTF8-encoded non-ASCII character has its high bit set and
+   * so will be rejected by this function. */
+  if (! isalpha (*p) && ! strchr ("_:", *p))
+    return FALSE;
+  p++;
+  for (; *p; p++)
     {
-      const void *key;
-      const char *keystring;
-      void * val;
-      const char *current_entry_name;
-      svn_stringbuf_t *full_entry_path
-        = svn_stringbuf_create (svn_wc_adm_access_path (adm_access), pool);
-      const svn_wc_entry_t *current_entry;
-
-      apr_hash_this (hi, &key, NULL, &val);
-      keystring = key;
-      current_entry = val;
-        
-      if (! strcmp (keystring, SVN_WC_ENTRY_THIS_DIR))
-        current_entry_name = NULL;
-      else
-        current_entry_name = keystring;
-
-      /* Compute the complete path of the entry */
-      if (current_entry_name)
-        svn_path_add_component (full_entry_path, current_entry_name);
-
-      if (current_entry->schedule != svn_wc_schedule_delete)
-        {
-          svn_error_t *err;
-          if (current_entry->kind == svn_node_dir && current_entry_name)
-            {
-              svn_wc_adm_access_t *dir_access;
-
-              SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access,
-                                            full_entry_path->data, pool));
-              err = recursive_propset (propname, propval, dir_access, pool);
-            }
-          else
-            {
-              err = svn_wc_prop_set (propname, propval,
-                                     full_entry_path->data, adm_access, pool);
-            }
-          if (err)
-            {
-              if (err->apr_err != SVN_ERR_ILLEGAL_TARGET)
-                return err;
-              svn_error_clear (err);
-            }
-        }
+      if (! isalnum (*p) && ! strchr (".-_:", *p))
+        return FALSE;
     }
+  return TRUE;
+}
+
+
+/* Check whether NAME is a revision property name.
+ * 
+ * Return TRUE if it is.
+ * Return FALSE if it is not.  
+ */ 
+static svn_boolean_t
+is_revision_prop_name (const char *name)
+{
+  apr_size_t i;
+  const char *revision_props[] = 
+    {
+      SVN_PROP_REVISION_ALL_PROPS
+    };
+
+  for (i = 0; i < sizeof (revision_props) / sizeof (revision_props[0]); i++)
+    {
+      if (strcmp (name, revision_props[i]) == 0)
+        return TRUE;
+    }
+  return FALSE;
+}
+
+/* A baton for propset_walk_cb. */
+struct propset_walk_baton
+{
+  const char *propname;  /* The name of the property to set. */
+  const svn_string_t *propval;  /* The value to set. */
+  svn_wc_adm_access_t *base_access;  /* Access for the tree being walked. */
+};
+
+/* An entries-walk callback for svn_client_propset.
+ * 
+ * For the path given by PATH and ENTRY,
+ * set the property named wb->PROPNAME to the value wb->PROPVAL,
+ * where "wb" is the WALK_BATON of type "struct propset_walk_baton *".
+ */
+static svn_error_t *
+propset_walk_cb (const char *path,
+                 const svn_wc_entry_t *entry,
+                 void *walk_baton,
+                 apr_pool_t *pool)
+{
+  struct propset_walk_baton *wb = walk_baton;
+  svn_error_t *err;
+  svn_wc_adm_access_t *adm_access;
+
+  /* We're going to receive dirents twice;  we want to ignore the
+     first one (where it's a child of a parent dir), and only use
+     the second one (where we're looking at THIS_DIR).  */
+  if ((entry->kind == svn_node_dir)
+      && (strcmp (entry->name, SVN_WC_ENTRY_THIS_DIR) != 0))
+    return SVN_NO_ERROR;
+
+  /* Ignore the entry if it does not exist at the time of interest. */
+  if (entry->schedule == svn_wc_schedule_delete)
+    return SVN_NO_ERROR;
+
+  SVN_ERR (svn_wc_adm_retrieve (&adm_access, wb->base_access,
+                                (entry->kind == svn_node_dir ? path
+                                 : svn_path_dirname (path, pool)),
+                                pool));
+  err = svn_wc_prop_set (wb->propname, wb->propval,
+                         path, adm_access, pool);
+  if (err)
+    {
+      if (err->apr_err != SVN_ERR_ILLEGAL_TARGET)
+        return err;
+      svn_error_clear (err);
+    }
+
   return SVN_NO_ERROR;
 }
 
@@ -104,26 +140,46 @@ svn_client_propset (const char *propname,
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *node;
 
+  if (is_revision_prop_name (propname))
+    {
+      return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
+                                "Revision property '%s' not allowed "
+                                "in this context", propname);
+    }
+
   if (svn_path_is_url (target))
     {
       /* ### Note that this function will need to take an auth baton
          if it's ever to support setting properties remotely. */
       return svn_error_createf
         (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
-         "Setting property on non-local target '%s' not yet supported.",
+         "Setting property on non-local target '%s' not yet supported",
          target);
     }
 
+  if (propval && ! is_valid_prop_name (propname))
+    return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
+                              "Bad property name: '%s'", propname);
+
   SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, TRUE, TRUE, pool));
   SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
   if (!node)
-    return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                              "'%s' -- not a versioned resource", 
+    return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                              "'%s' is not under version control", 
                               target);
 
   if (recurse && node->kind == svn_node_dir)
     {
-      SVN_ERR (recursive_propset (propname, propval, adm_access, pool));
+      static const svn_wc_entry_callbacks_t walk_callbacks
+        = { propset_walk_cb };
+      struct propset_walk_baton wb;
+
+      wb.base_access = adm_access;
+      wb.propname = propname;
+      wb.propval = propval;
+
+      SVN_ERR (svn_wc_walk_entries (target, adm_access,
+                                    &walk_callbacks, &wb, FALSE, pool));
     }
   else
     {
@@ -135,27 +191,35 @@ svn_client_propset (const char *propname,
 }
 
 
-
 svn_error_t *
 svn_client_revprop_set (const char *propname,
                         const svn_string_t *propval,
                         const char *URL,
                         const svn_opt_revision_t *revision,
                         svn_revnum_t *set_rev,
+                        svn_boolean_t force,
                         svn_client_ctx_t *ctx,
                         apr_pool_t *pool)
 {
   void *ra_baton, *session;
   svn_ra_plugin_t *ra_lib;
-  const char *auth_dir;
+
+  if ((strcmp (propname, SVN_PROP_REVISION_AUTHOR) == 0)
+      && propval 
+      && strchr (propval->data, '\n') != NULL 
+      && (! force))
+    return svn_error_create (SVN_ERR_CLIENT_REVISION_AUTHOR_CONTAINS_NEWLINE,
+                             NULL, "Value will not be set unless forced");
+
+  if (propval && ! is_valid_prop_name (propname))
+    return svn_error_createf (SVN_ERR_CLIENT_PROPERTY_NAME, NULL,
+                              "Bad property name: '%s'", propname);
 
   /* Open an RA session for the URL. Note that we don't have a local
-     directory, nor a place to put temp files or store the auth data,
-     although we'll try to fetch auth data from the current directory. */
+     directory, nor a place to put temp files. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
   SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, URL, pool));
-  SVN_ERR (svn_client__dir_if_wc (&auth_dir, "", pool));
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, URL, auth_dir,
+  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, URL, NULL,
                                         NULL, NULL, FALSE, TRUE,
                                         ctx, pool));
 
@@ -175,8 +239,8 @@ svn_client_revprop_set (const char *propname,
 /* Set *PROPS to the pristine (base) properties at PATH, if PRISTINE
  * is true, or else the working value if PRISTINE is false.  
  *
- * The keys of *PROPS will be `const char *' property names, and the
- * values `const svn_string_t *' property values.  Allocate *PROPS
+ * The keys of *PROPS will be 'const char *' property names, and the
+ * values 'const svn_string_t *' property values.  Allocate *PROPS
  * and its contents in POOL.
  */
 static svn_error_t *
@@ -224,77 +288,57 @@ pristine_or_working_propval (const svn_string_t **propval,
 }
 
 
-/* Helper for svn_client_propget.
+/* A baton for propget_walk_cb. */
+struct propget_walk_baton
+{
+  const char *propname;  /* The name of the property to get. */
+  svn_boolean_t pristine;  /* Select base rather than working props. */
+  svn_wc_adm_access_t *base_access;  /* Access for the tree being walked. */
+  apr_hash_t *props;  /* Out: mapping of (path:propval). */
+};
+
+/* An entries-walk callback for svn_client_propget.
  * 
- * Starting from the path associated with ADM_ACCESS, populate PROPS
- * with the values of property PROPNAME.  If PRISTINE is true, use the
- * base values, else use working values.
+ * For the path given by PATH and ENTRY,
+ * populate wb->PROPS with the values of property wb->PROPNAME,
+ * where "wb" is the WALK_BATON of type "struct propget_walk_baton *".
+ * If wb->PRISTINE is true, use the base value, else use the working value.
  *
- * The keys of PROPS will be `const char *' paths, rooted at the
+ * The keys of wb->PROPS will be 'const char *' paths, rooted at the
  * path svn_wc_adm_access_path(ADM_ACCESS), and the values are
- * `const svn_string_t *' property values.
+ * 'const svn_string_t *' property values.
  */
 static svn_error_t *
-recursive_propget (apr_hash_t *props,
-                   const char *propname,
-                   svn_boolean_t pristine,
-                   svn_wc_adm_access_t *adm_access,
-                   apr_pool_t *pool)
+propget_walk_cb (const char *path,
+                 const svn_wc_entry_t *entry,
+                 void *walk_baton,
+                 apr_pool_t *pool)
 {
-  apr_hash_t *entries;
-  apr_hash_index_t *hi;
-  SVN_ERR (svn_wc_entries_read (&entries, adm_access, FALSE, pool));
+  struct propget_walk_baton *wb = walk_baton;
+  const svn_string_t *propval;
 
-  for (hi = apr_hash_first (pool, entries); hi; hi = apr_hash_next (hi))
+  /* We're going to receive dirents twice;  we want to ignore the
+     first one (where it's a child of a parent dir), and only use
+     the second one (where we're looking at THIS_DIR).  */
+  if ((entry->kind == svn_node_dir)
+      && (strcmp (entry->name, SVN_WC_ENTRY_THIS_DIR) != 0))
+    return SVN_NO_ERROR;
+
+  /* Ignore the entry if it does not exist at the time of interest. */
+  if (entry->schedule
+      == (wb->pristine ? svn_wc_schedule_add : svn_wc_schedule_delete))
+    return SVN_NO_ERROR;
+
+  SVN_ERR (pristine_or_working_propval (&propval, wb->propname, path,
+                                        wb->base_access, wb->pristine,
+                                        apr_hash_pool_get (wb->props)));
+
+  if (propval)
     {
-      const void *key;
-      const char *keystring;
-      void * val;
-      const char *current_entry_name;
-      const char *full_entry_path;
-      const svn_wc_entry_t *current_entry;
-
-      apr_hash_this (hi, &key, NULL, &val);
-      keystring = key;
-      current_entry = val;
-    
-      if (! strcmp (keystring, SVN_WC_ENTRY_THIS_DIR))
-          current_entry_name = NULL;
-      else
-          current_entry_name = keystring;
-
-      /* Compute the complete path of the entry */
-      if (current_entry_name)
-        full_entry_path = svn_path_join (svn_wc_adm_access_path (adm_access),
-                                         current_entry_name, pool);
-      else
-        full_entry_path = apr_pstrdup (pool,
-                                       svn_wc_adm_access_path (adm_access));
-
-      if (current_entry->schedule != svn_wc_schedule_delete)
-        {
-          if (current_entry->kind == svn_node_dir && current_entry_name)
-            {
-              svn_wc_adm_access_t *dir_access;
-              SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access,
-                                            full_entry_path, pool));
-              SVN_ERR (recursive_propget (props, propname, pristine,
-                                          dir_access, pool));
-            }
-          else
-            {
-              const svn_string_t *propval;
-
-              SVN_ERR (pristine_or_working_propval (&propval, propname,
-                                                    full_entry_path,
-                                                    adm_access,
-                                                    pristine, pool));
-              if (propval)
-                apr_hash_set (props, full_entry_path,
-                              APR_HASH_KEY_STRING, propval);
-            }
-        }
+      path = apr_pstrdup (apr_hash_pool_get (wb->props), path);
+      apr_hash_set (wb->props, path, APR_HASH_KEY_STRING, propval);
     }
+
   return SVN_NO_ERROR;
 }
 
@@ -335,8 +379,8 @@ maybe_convert_to_url (const char **new_target,
       SVN_ERR (svn_wc_adm_open (&adm_access, NULL, pdir, FALSE, FALSE, pool));
       SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, pool));
       if (! entry)
-        return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                                  "'%s' is not a versioned resource", 
+        return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                                  "'%s' is not under version control", 
                                   target);
       *new_target = entry->url;
     }
@@ -350,8 +394,8 @@ maybe_convert_to_url (const char **new_target,
 /* Helper for the remote case of svn_client_propget.
  *
  * Get the value of property PROPNAME in REVNUM, using RA_LIB and
- * SESSION.  Store the value (`svn_string_t *') in PROPS, under the
- * path key "TARGET_PREFIX/TARGET_RELATIVE" (`const char *').
+ * SESSION.  Store the value ('svn_string_t *') in PROPS, under the
+ * path key "TARGET_PREFIX/TARGET_RELATIVE" ('const char *').
  *
  * If RECURSE is true and KIND is svn_node_dir, then recurse.
  *
@@ -390,7 +434,7 @@ remote_propget (apr_hash_t *props,
     {
       return svn_error_createf
         (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-         "unknown node kind for \"%s\"",
+         "Unknown node kind for '%s'",
          svn_path_join (target_prefix, target_relative, pool));
     }
   
@@ -438,6 +482,7 @@ remote_propget (apr_hash_t *props,
 }
 
 
+/* Note: this implementation is very similar to svn_client_proplist. */
 svn_error_t *
 svn_client_propget (apr_hash_t **props,
                     const char *propname,
@@ -450,9 +495,9 @@ svn_client_propget (apr_hash_t **props,
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *node;
   const char *utarget;  /* target, or the url for target */
-  svn_node_kind_t kind;
   svn_revnum_t revnum;
-  const char *auth_dir;
+
+  *props = apr_hash_make (pool);
 
   SVN_ERR (maybe_convert_to_url (&utarget, target, revision, pool));
 
@@ -462,17 +507,15 @@ svn_client_propget (apr_hash_t **props,
     {
       void *ra_baton, *session;
       svn_ra_plugin_t *ra_lib;
+      svn_node_kind_t kind;
       svn_opt_revision_t new_revision;  /* only used in one case */
 
       SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
       SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, utarget, pool));
-      SVN_ERR (svn_client__dir_if_wc (&auth_dir, "", pool));
       SVN_ERR (svn_client__open_ra_session (&session, ra_lib, utarget,
-                                            auth_dir, NULL, NULL,
+                                            NULL, NULL, NULL,
                                             FALSE, FALSE, ctx, pool));
 
-      *props = apr_hash_make (pool);
-
       /* Default to HEAD. */
       if (revision->kind == svn_opt_revision_unspecified)
         {
@@ -487,12 +530,6 @@ svn_client_propget (apr_hash_t **props,
         {
           SVN_ERR (svn_client__get_revision_number
                    (&revnum, ra_lib, session, revision, NULL, pool));
-
-          SVN_ERR (ra_lib->check_path (&kind, session, "", revnum, pool));
-
-          SVN_ERR (remote_propget (*props, propname, utarget, "",
-                                   kind, revnum, ra_lib, session,
-                                   recurse, pool));
         }
       else if (revision->kind == svn_opt_revision_previous)
         {
@@ -500,42 +537,36 @@ svn_client_propget (apr_hash_t **props,
             {
               return svn_error_createf
                 (SVN_ERR_ILLEGAL_TARGET, NULL,
-                 "\"%s\" is a url, but revision kind requires a working copy",
+                 "'%s' is a URL, but revision kind requires a working copy",
                  target);
             }
-          else  /* target is a working copy path */
-            {
-              SVN_ERR (svn_client__get_revision_number
-                       (&revnum, NULL, NULL, revision, target, pool));
-              
-              SVN_ERR (ra_lib->check_path (&kind, session, "", revnum, pool));
-              
-              SVN_ERR (remote_propget (*props, propname, utarget, "",
-                                       kind, revnum, ra_lib, session,
-                                       recurse, pool));
-            }
+
+          /* target is a working copy path */
+          SVN_ERR (svn_client__get_revision_number
+                   (&revnum, NULL, NULL, revision, target, pool));
         }
       else
         {
           return svn_error_create
-            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "unknown revision kind");
+            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "Unknown revision kind");
         }
+
+      SVN_ERR (ra_lib->check_path (session, "", revnum, &kind, pool));
+
+      SVN_ERR (remote_propget (*props, propname, utarget, "",
+                               kind, revnum, ra_lib, session,
+                               recurse, pool));
     }
   else  /* working copy path */
     {
-      apr_hash_t *prop_hash;
       svn_boolean_t pristine;
 
-      prop_hash = apr_hash_make (pool);
-      
-      SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target,
-                                      FALSE, TRUE, pool));
-      
+      SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, FALSE, TRUE,
+                                      pool));
       SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
       if (! node)
-        return svn_error_createf
-          (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-           "'%s' -- not a versioned resource", target);
+        return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                                  "'%s' is not under version control", target);
       
       SVN_ERR (svn_client__get_revision_number
                (&revnum, NULL, NULL, revision, target, pool));
@@ -553,8 +584,17 @@ svn_client_propget (apr_hash_t **props,
       /* Fetch, recursively or not. */
       if (recurse && (node->kind == svn_node_dir))
         {
-          SVN_ERR (recursive_propget (prop_hash, propname, pristine,
-                                      adm_access, pool));
+          static const svn_wc_entry_callbacks_t walk_callbacks
+            = { propget_walk_cb };
+          struct propget_walk_baton wb;
+
+          wb.base_access = adm_access;
+          wb.props = *props;
+          wb.propname = propname;
+          wb.pristine = pristine;
+
+          SVN_ERR (svn_wc_walk_entries (target, adm_access,
+                                        &walk_callbacks, &wb, FALSE, pool));
         }
       else
         {
@@ -563,12 +603,10 @@ svn_client_propget (apr_hash_t **props,
           SVN_ERR (pristine_or_working_propval (&propval, propname, target,
                                                 adm_access, pristine, pool));
 
-          apr_hash_set (prop_hash, target, APR_HASH_KEY_STRING, propval);
+          apr_hash_set (*props, target, APR_HASH_KEY_STRING, propval);
         }
       
       SVN_ERR (svn_wc_adm_close (adm_access));
-      
-      *props = prop_hash;
     }
 
   return SVN_NO_ERROR;
@@ -586,14 +624,12 @@ svn_client_revprop_get (const char *propname,
 {
   void *ra_baton, *session;
   svn_ra_plugin_t *ra_lib;
-  const char *auth_dir;
 
   /* Open an RA session for the URL. Note that we don't have a local
-     directory, nor a place to put temp files or store the auth data. */
+     directory, nor a place to put temp files. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
   SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, URL, pool));
-  SVN_ERR (svn_client__dir_if_wc (&auth_dir, "", pool));
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, URL, auth_dir,
+  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, URL, NULL,
                                         NULL, NULL, FALSE, TRUE,
                                         ctx, pool));
 
@@ -609,9 +645,9 @@ svn_client_revprop_get (const char *propname,
 }
 
 
-/* Push a new `svn_client_proplist_item_t *' item onto LIST.
+/* Push a new 'svn_client_proplist_item_t *' item onto LIST.
  * Allocate the item itself in POOL; set item->node_name to an
- * `svn_stringbuf_t *' created from PATH and also allocated in POOL,
+ * 'svn_stringbuf_t *' created from PATH and also allocated in POOL,
  * and set item->prop_hash to PROP_HASH.
  *
  * If PROP_HASH is null or has zero count, do nothing.
@@ -636,11 +672,11 @@ push_props_on_list (apr_array_header_t *list,
 
 /* Helper for the remote case of svn_client_proplist.
  *
- * Push a new `svn_client_proplist_item_t *' item onto PROPLIST,
+ * Push a new 'svn_client_proplist_item_t *' item onto PROPLIST,
  * containing the properties for "TARGET_PREFIX/TARGET_RELATIVE" in
  * REVNUM, obtained using RA_LIB and SESSION.  The item->node_name
  * will be "TARGET_PREFIX/TARGET_RELATIVE", and the value will be a
- * hash mapping `const char *' property names onto `svn_string_t *'
+ * hash mapping 'const char *' property names onto 'svn_string_t *'
  * property values.  Allocate the new item and its contents in POOL.
  *
  * KIND is the kind of the node at "TARGET_PREFIX/TARGET_RELATIVE".
@@ -677,7 +713,7 @@ remote_proplist (apr_array_header_t *proplist,
     {
       return svn_error_createf
         (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
-         "unknown node kind for \"%s\"",
+         "Unknown node kind for '%s'",
          svn_path_join (target_prefix, target_relative, pool));
     }
   
@@ -737,8 +773,8 @@ remote_proplist (apr_array_header_t *proplist,
 }
 
 
-/* Push an `svn_client_proplist_item_t *' item onto PROP_LIST, where
- * item->node_name is an `svn_stringbuf_t *' created from NODE_NAME,
+/* Push an 'svn_client_proplist_item_t *' item onto PROP_LIST, where
+ * item->node_name is an 'svn_stringbuf_t *' created from NODE_NAME,
  * and item->prop_hash is the property hash for NODE_NAME.
  *
  * If PRISTINE is true, get base props, else get working props.
@@ -761,70 +797,50 @@ add_to_proplist (apr_array_header_t *prop_list,
   return SVN_NO_ERROR;
 }
 
-/* Helper for svn_client_proplist.
+/* A baton for proplist_walk_cb. */
+struct proplist_walk_baton
+{
+  svn_boolean_t pristine;  /* Select base rather than working props. */
+  svn_wc_adm_access_t *base_access;  /* Access for the tree being walked. */
+  apr_array_header_t *props;  /* Out: array of svn_client_proplist_item_t. */
+};
+
+/* An entries-walk callback for svn_client_proplist.
  * 
- * Starting from the path associated with ADM_ACCESS, populate PROPS
- * with the values of property PROPNAME.  If PRISTINE is true, use the
- * base values, else use working values.
- *
- * The keys of PROPS will be `const char *' paths, rooted at the
- * path svn_wc_adm_access_path(ADM_ACCESS), and the values are
- * `const svn_string_t *' property values.
+ * For the path given by PATH and ENTRY,
+ * populate wb->PROPS with a svn_client_proplist_item_t for each path,
+ * where "wb" is the WALK_BATON of type "struct proplist_walk_baton *".
+ * If wb->PRISTINE is true, use the base values, else use the working values.
  */
 static svn_error_t *
-recursive_proplist (apr_array_header_t *props,
-                    svn_wc_adm_access_t *adm_access,
-                    svn_boolean_t pristine,
-                    apr_pool_t *pool)
+proplist_walk_cb (const char *path,
+                  const svn_wc_entry_t *entry,
+                  void *walk_baton,
+                  apr_pool_t *pool)
 {
-  apr_hash_t *entries;
-  apr_hash_index_t *hi;
+  struct proplist_walk_baton *wb = walk_baton;
 
-  SVN_ERR (svn_wc_entries_read (&entries, adm_access, FALSE, pool));
+  /* We're going to receive dirents twice;  we want to ignore the
+     first one (where it's a child of a parent dir), and only use
+     the second one (where we're looking at THIS_DIR).  */
+  if ((entry->kind == svn_node_dir)
+      && (strcmp (entry->name, SVN_WC_ENTRY_THIS_DIR) != 0))
+    return SVN_NO_ERROR;
 
-  for (hi = apr_hash_first (pool, entries); hi; hi = apr_hash_next (hi))
-    {
-      const void *key;
-      const char *keystring;
-      void * val;
-      const char *current_entry_name;
-      const char *full_entry_path;
-      const svn_wc_entry_t *current_entry;
+  /* Ignore the entry if it does not exist at the time of interest. */
+  if (entry->schedule
+      == (wb->pristine ? svn_wc_schedule_add : svn_wc_schedule_delete))
+    return SVN_NO_ERROR;
 
-      apr_hash_this (hi, &key, NULL, &val);
-      keystring = key;
-      current_entry = val;
-    
-      if (! strcmp (keystring, SVN_WC_ENTRY_THIS_DIR))
-          current_entry_name = NULL;
-      else
-          current_entry_name = keystring;
+  path = apr_pstrdup (wb->props->pool, path);
+  SVN_ERR (add_to_proplist (wb->props, path, wb->base_access,
+                            wb->pristine, wb->props->pool));
 
-      /* Compute the complete path of the entry */
-      if (current_entry_name)
-        full_entry_path = svn_path_join (svn_wc_adm_access_path (adm_access),
-                                         current_entry_name, pool);
-      else
-        full_entry_path = apr_pstrdup (pool,
-                                       svn_wc_adm_access_path (adm_access));
-
-      if (current_entry->schedule != svn_wc_schedule_delete)
-        {
-          if (current_entry->kind == svn_node_dir && current_entry_name)
-            {
-              svn_wc_adm_access_t *dir_access;
-              SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access,
-                                            full_entry_path, pool));
-              SVN_ERR (recursive_proplist (props, dir_access, pristine, pool));
-            }
-          else
-            SVN_ERR (add_to_proplist (props, full_entry_path, adm_access,
-                                      pristine, pool));
-        }
-    }
   return SVN_NO_ERROR;
 }
 
+
+/* Note: this implementation is very similar to svn_client_propget. */
 svn_error_t *
 svn_client_proplist (apr_array_header_t **props,
                      const char *target, 
@@ -833,15 +849,17 @@ svn_client_proplist (apr_array_header_t **props,
                      svn_client_ctx_t *ctx,
                      apr_pool_t *pool)
 {
-  apr_array_header_t *prop_list
-      = apr_array_make (pool, 5, sizeof (svn_client_proplist_item_t *));
   svn_wc_adm_access_t *adm_access;
-  const svn_wc_entry_t *entry;
+  const svn_wc_entry_t *node;
   const char *utarget;  /* target, or the url for target */
   svn_revnum_t revnum;
 
+  *props = apr_array_make (pool, 5, sizeof (svn_client_proplist_item_t *));
+
   SVN_ERR (maybe_convert_to_url (&utarget, target, revision, pool));
 
+  /* Iff utarget is a url, that means we must use it, that is, the
+     requested property information is not available locally. */
   if (svn_path_is_url (utarget))
     {
       void *ra_baton, *session;
@@ -869,12 +887,6 @@ svn_client_proplist (apr_array_header_t **props,
         {
           SVN_ERR (svn_client__get_revision_number
                    (&revnum, ra_lib, session, revision, NULL, pool));
-
-          SVN_ERR (ra_lib->check_path (&kind, session, "", revnum, pool));
-
-          SVN_ERR (remote_proplist (prop_list, utarget, "",
-                                    kind, revnum, ra_lib, session,
-                                    recurse, pool));
         }
       else if (revision->kind == svn_opt_revision_previous)
         {
@@ -882,26 +894,25 @@ svn_client_proplist (apr_array_header_t **props,
             {
               return svn_error_createf
                 (SVN_ERR_ILLEGAL_TARGET, NULL,
-                 "\"%s\" is a url, but revision kind requires a working copy",
+                 "'%s' is a URL, but revision kind requires a working copy",
                  target);
             }
-          else  /* it's a working copy path */
-            {
-              SVN_ERR (svn_client__get_revision_number
-                       (&revnum, NULL, NULL, revision, target, pool));
-              
-              SVN_ERR (ra_lib->check_path (&kind, session, "", revnum, pool));
-              
-              SVN_ERR (remote_proplist (prop_list, utarget, "",
-                                        kind, revnum, ra_lib, session,
-                                        recurse, pool));
-            }
+
+          /* target is a working copy path */
+          SVN_ERR (svn_client__get_revision_number
+                   (&revnum, NULL, NULL, revision, target, pool));
         }
       else
         {
           return svn_error_create
-            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "unknown revision kind");
+            (SVN_ERR_CLIENT_BAD_REVISION, NULL, "Unknown revision kind");
         }
+
+      SVN_ERR (ra_lib->check_path (session, "", revnum, &kind, pool));
+
+      SVN_ERR (remote_proplist (*props, utarget, "",
+                                kind, revnum, ra_lib, session,
+                                recurse, pool));
     }
   else  /* working copy path */
     {
@@ -909,15 +920,14 @@ svn_client_proplist (apr_array_header_t **props,
 
       SVN_ERR (svn_wc_adm_probe_open (&adm_access, NULL, target, FALSE, TRUE,
                                       pool));
-      SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, pool));
-      if (! entry)
-        return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                                  "'%s' -- not a versioned resource", 
-                                  target);
+      SVN_ERR (svn_wc_entry (&node, target, adm_access, FALSE, pool));
+      if (! node)
+        return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                                  "'%s' is not under version control", target);
       
       SVN_ERR (svn_client__get_revision_number
                (&revnum, NULL, NULL, revision, target, pool));
-      
+
       if ((revision->kind == svn_opt_revision_committed)
           || (revision->kind == svn_opt_revision_base))
         {
@@ -928,17 +938,26 @@ svn_client_proplist (apr_array_header_t **props,
           pristine = FALSE;
         }
 
-      if (recurse && entry->kind == svn_node_dir)
-        SVN_ERR (recursive_proplist (prop_list, adm_access, pristine, pool));
+      /* Fetch, recursively or not. */
+      if (recurse && (node->kind == svn_node_dir))
+        {
+          static const svn_wc_entry_callbacks_t walk_callbacks
+            = { proplist_walk_cb };
+          struct proplist_walk_baton wb;
+
+          wb.base_access = adm_access;
+          wb.props = *props;
+          wb.pristine = pristine;
+
+          SVN_ERR (svn_wc_walk_entries (target, adm_access,
+                                        &walk_callbacks, &wb, FALSE, pool));
+        }
       else 
-        SVN_ERR (add_to_proplist (prop_list, target, adm_access, pristine,
-                                  pool));
+        SVN_ERR (add_to_proplist (*props, target, adm_access, pristine, pool));
       
       SVN_ERR (svn_wc_adm_close (adm_access));
     }
 
-  *props = prop_list;
-
   return SVN_NO_ERROR;
 }
 
@@ -954,10 +973,9 @@ svn_client_revprop_list (apr_hash_t **props,
   void *ra_baton, *session;
   svn_ra_plugin_t *ra_lib;
   apr_hash_t *proplist;
-  apr_hash_index_t *hi;
 
   /* Open an RA session for the URL. Note that we don't have a local
-     directory, nor a place to put temp files or store the auth data. */
+     directory, nor a place to put temp files. */
   SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
   SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, URL, pool));
   SVN_ERR (svn_client__open_ra_session (&session, ra_lib, URL, NULL,
@@ -972,16 +990,6 @@ svn_client_revprop_list (apr_hash_t **props,
   /* The actual RA call. */
   SVN_ERR (ra_lib->rev_proplist (session, *set_rev, &proplist, pool));
 
-  for (hi = apr_hash_first (pool, proplist); hi; hi = apr_hash_next (hi))
-    {
-      const void *key;
-      void *val;
-      apr_ssize_t klen;
-      
-      apr_hash_this (hi, &key, &klen, &val);
-      apr_hash_set (proplist, key, klen, val);
-    } 
-  
   *props = proplist;
   return SVN_NO_ERROR;
 }
