@@ -2,7 +2,7 @@
  * diff_file.c :  routines for doing diffs on files
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -32,12 +32,15 @@
 #include "svn_string.h"
 #include "svn_io.h"
 #include "svn_pools.h"
+#include "diff.h"
 
 
 typedef struct svn_diff__file_token_t
 {
-  apr_off_t   length;
-  const char *line;
+  struct svn_diff__file_token_t *next;
+  svn_diff_datasource_e datasource;
+  apr_off_t offset;
+  apr_off_t length;
 } svn_diff__file_token_t;
 
 
@@ -45,12 +48,15 @@ typedef struct svn_diff__file_baton_t
 {
   const char *path[4];
 
+  apr_file_t *file[4];
+  apr_off_t size[4];
+
+  int chunk[4];
   char *buffer[4];
   char *curp[4];
   char *endp[4];
 
-  svn_diff__file_token_t *token;
-  svn_boolean_t reuse_token;
+  svn_diff__file_token_t *tokens;
 
   apr_pool_t *pool;
 } svn_diff__file_baton_t;
@@ -78,6 +84,44 @@ svn_diff__file_datasource_to_index(svn_diff_datasource_e datasource)
   return -1;
 }
 
+/* Files are read in chunks of 128k.  There is no support for this number
+ * whatsoever.  If there is a number someone comes up with that has some
+ * argumentation, let's use that.
+ */
+#define CHUNK_SHIFT 17
+#define CHUNK_SIZE (1 << CHUNK_SHIFT)
+
+#define chunk_to_offset(chunk) ((chunk) << CHUNK_SHIFT)
+#define offset_to_chunk(offset) ((offset) >> CHUNK_SHIFT)
+#define offset_in_chunk(offset) ((offset) & (CHUNK_SIZE - 1))
+
+
+/* Read a chunk from a FILE into BUFFER, starting from OFFSET, going for
+ * *LENGTH.  The actual bytes read are stored in *LENGTH on return.
+ */
+static APR_INLINE
+svn_error_t *
+read_chunk(apr_file_t *file, const char *path,
+           char *buffer, apr_size_t length,
+           apr_off_t offset)
+{
+  apr_status_t apr_err;
+
+  /* XXX: The final offset may not be the one we asked for.
+   * XXX: Check.
+   */
+  apr_err = apr_file_seek(file, APR_SET, &offset);
+  if (apr_err != APR_SUCCESS)
+    return svn_error_wrap_apr(apr_err, "Can't seek in file '%s'", path);
+
+  apr_err = apr_file_read_full(file, buffer, length, NULL);
+  if (apr_err != APR_SUCCESS)
+    return svn_error_wrap_apr(apr_err, "Failed to read file '%s'", path);
+
+  return SVN_NO_ERROR;
+}
+
+
 /* Map or read a file at PATH. *BUFFER will point to the file
  * contents; if the file was mapped, *FILE and *MM will contain the
  * mmap context; otherwise they will be NULL.  SIZE will contain the
@@ -107,10 +151,7 @@ map_or_read_file(apr_file_t **file,
 
   rv = apr_file_info_get(&finfo, APR_FINFO_SIZE, *file);
   if (rv != APR_SUCCESS)
-    {
-      return svn_error_createf(rv, NULL, "Failed to get file info '%s'.",
-                               path);
-    }
+    return svn_error_wrap_apr(rv, "Can't get file info for '%s'", path);
 
 #if APR_HAS_MMAP
   if (finfo.size > APR_MMAP_THRESHOLD)
@@ -131,22 +172,12 @@ map_or_read_file(apr_file_t **file,
     {
       *buffer = apr_palloc(pool, finfo.size);
 
-      rv = apr_file_read_full(*file, *buffer, finfo.size, NULL);
-      if (rv != APR_SUCCESS)
-        {
-          return svn_error_createf(rv, NULL, "Failed to read file '%s'.",
-                                   path);
-        }
+      SVN_ERR (svn_io_file_read_full(*file, *buffer, finfo.size, NULL, pool));
 
       /* Since we have the entire contents of the file we can
        * close it now.
        */
-      rv = apr_file_close(*file);
-      if (rv != APR_SUCCESS)
-        {
-          return svn_error_createf(rv, NULL, "Failed to close file '%s'.",
-                                   path);
-        }
+      SVN_ERR (svn_io_file_close(*file, pool));
 
       *file = NULL;
     }
@@ -156,45 +187,68 @@ map_or_read_file(apr_file_t **file,
   return SVN_NO_ERROR;
 }
 
+
 static
 svn_error_t *
 svn_diff__file_datasource_open(void *baton,
                                svn_diff_datasource_e datasource)
 {
   svn_diff__file_baton_t *file_baton = baton;
-  apr_file_t *file;
-#if APR_HAS_MMAP
-  apr_mmap_t *mm;
-#endif /* APR_HAS_MMAP */
-  apr_off_t size;
   int idx;
+  apr_status_t apr_err;
+  apr_finfo_t finfo;
+  apr_size_t length;
+  char *curp;
+  char *endp;
 
   idx = svn_diff__file_datasource_to_index(datasource);
-  SVN_ERR(map_or_read_file(&file,
-                           MMAP_T_ARG(mm)
-                           &file_baton->buffer[idx], &size,
-                           file_baton->path[idx], file_baton->pool));
 
-  file_baton->curp[idx] = file_baton->buffer[idx];
-  file_baton->endp[idx] = file_baton->buffer[idx];
+  SVN_ERR(svn_io_file_open(&file_baton->file[idx], file_baton->path[idx],
+                           APR_READ, APR_OS_DEFAULT, file_baton->pool));
 
-  if (file_baton->endp[idx])
-    file_baton->endp[idx] += size;
+  apr_err = apr_file_info_get(&finfo, APR_FINFO_SIZE, file_baton->file[idx]);
+  if (apr_err != APR_SUCCESS)
+    {
+      return svn_error_wrap_apr(apr_err, "Can't get file info for '%s'",
+                                file_baton->path[idx]);
+    }
+
+  file_baton->size[idx] = finfo.size;
+  length = finfo.size > CHUNK_SIZE ? CHUNK_SIZE : finfo.size;
+
+  if (length == 0)
+    return SVN_NO_ERROR;
+
+  endp = curp = apr_palloc(file_baton->pool, length);
+  endp += length;
+
+  file_baton->buffer[idx] = file_baton->curp[idx] = curp;
+  file_baton->endp[idx] = endp;
+
+  SVN_ERR(read_chunk(file_baton->file[idx], file_baton->path[idx],
+                     curp, length, 0));
 
   return SVN_NO_ERROR;
 }
 
+
 static
 svn_error_t *
 svn_diff__file_datasource_close(void *baton,
                                 svn_diff_datasource_e datasource)
 {
+  /* Do nothing.  The compare_token function needs previous datasources
+   * to stay available until all datasources are processed.
+   */
+
   return SVN_NO_ERROR;
 }
 
+
 static
 svn_error_t *
-svn_diff__file_datasource_get_next_token(void **token, void *baton,
+svn_diff__file_datasource_get_next_token(apr_uint32_t *hash, void **token,
+                                         void *baton,
                                          svn_diff_datasource_e datasource)
 {
   svn_diff__file_baton_t *file_baton = baton;
@@ -203,6 +257,9 @@ svn_diff__file_datasource_get_next_token(void **token, void *baton,
   char *endp;
   char *curp;
   char *eol;
+  int last_chunk;
+  apr_size_t length;
+  apr_uint32_t h = 0;
 
   *token = NULL;
 
@@ -211,32 +268,65 @@ svn_diff__file_datasource_get_next_token(void **token, void *baton,
   curp = file_baton->curp[idx];
   endp = file_baton->endp[idx];
 
-  if (curp == endp)
+  last_chunk = offset_to_chunk(file_baton->size[idx]);
+
+  if (curp == endp
+      && last_chunk == file_baton->chunk[idx])
     {
       return SVN_NO_ERROR;
     }
 
-  if (!file_baton->reuse_token)
+  /* Get a new token */
+  file_token = file_baton->tokens;
+  if (file_token)
+    {
+      file_baton->tokens = file_token->next;
+    }
+  else
     {
       file_token = apr_palloc(file_baton->pool, sizeof(*file_token));
-      file_baton->token = file_token;
-    }
-  else
-    {
-      file_token = file_baton->token;
-      file_baton->reuse_token = FALSE;
     }
 
+  file_token->datasource = datasource;
+  file_token->offset = chunk_to_offset(file_baton->chunk[idx])
+                       + (curp - file_baton->buffer[idx]);
   file_token->length = 0;
 
-  eol = memchr(curp, '\n', endp - curp);
-  if (!eol)
-    eol = endp;
-  else
-    eol++;
+  while (1)
+    {
+      /* XXX: '\n' doesn't really cut it.  We need to be able to detect
+       * XXX: '\n', '\r' and '\r\n'.
+       */
+      eol = memchr(curp, '\n', endp - curp);
+      if (eol)
+        {
+          eol++;
+          break;
+        }
 
-  file_token->line = curp;
-  file_token->length = eol - curp;
+      if (file_baton->chunk[idx] == last_chunk)
+        {
+          eol = endp;
+          break;
+        }
+
+      length = endp - curp;
+      file_token->length += length;
+      h = svn_diff__adler32(h, curp, length);
+
+      curp = endp = file_baton->buffer[idx];
+      file_baton->chunk[idx]++;
+      length = file_baton->chunk[idx] == last_chunk ? offset_in_chunk(file_baton->size[idx]) : CHUNK_SIZE;
+      endp += length;
+      file_baton->endp[idx] = endp;
+
+      SVN_ERR(read_chunk(file_baton->file[idx], file_baton->path[idx],
+                         curp, length, chunk_to_offset(file_baton->chunk[idx])));
+    }
+
+  length = eol - curp;
+  file_token->length += length;
+  *hash = svn_diff__adler32(h, curp, length);
 
   file_baton->curp[idx] = eol;
   *token = file_token;
@@ -244,44 +334,124 @@ svn_diff__file_datasource_get_next_token(void **token, void *baton,
   return SVN_NO_ERROR;
 }
 
+#define COMPARE_CHUNK_SIZE 4096
+
 static
-int
+svn_error_t *
 svn_diff__file_token_compare(void *baton,
                              void *token1,
-                             void *token2)
+                             void *token2,
+                             int *compare)
 {
+  svn_diff__file_baton_t *file_baton = baton;
   svn_diff__file_token_t *file_token1 = token1;
   svn_diff__file_token_t *file_token2 = token2;
+  char buffer[2][COMPARE_CHUNK_SIZE];
+  char *bufp[2];
+  apr_off_t offset[2];
+  int idx[2];
+  apr_off_t length[2];
+  apr_off_t total_length;
+  apr_off_t len;
+  int i;
+  int chunk[2];
 
   if (file_token1->length < file_token2->length)
-    return -1;
+    {
+      *compare = -1;
+      return SVN_NO_ERROR;
+    }
 
   if (file_token1->length > file_token2->length)
-    return 1;
+    {
+      *compare = 1;
+      return SVN_NO_ERROR;
+    }
 
-  return memcmp(file_token1->line, file_token2->line, file_token1->length);
+  total_length = file_token1->length;
+  if (total_length == 0)
+    {
+      *compare = 0;
+      return SVN_NO_ERROR;
+    }
+
+  idx[0] = svn_diff__file_datasource_to_index(file_token1->datasource);
+  idx[1] = svn_diff__file_datasource_to_index(file_token2->datasource);
+  offset[0] = file_token1->offset;
+  offset[1] = file_token2->offset;
+  chunk[0] = file_baton->chunk[idx[0]];
+  chunk[1] = file_baton->chunk[idx[1]];
+
+  do
+    {
+      for (i = 0; i < 2; i++)
+        {
+          if (offset_to_chunk(offset[i]) == chunk[i])
+            {
+              /* If the start of the token is in memory, the entire token is
+               * in memory.
+               */
+              bufp[i] = file_baton->buffer[idx[i]];
+              bufp[i] += offset_in_chunk(offset[i]);
+
+              length[i] = total_length;
+            }
+          else
+            {
+              /* Read a chunk from disk into a buffer */
+              bufp[i] = buffer[i];
+              length[i] = total_length > COMPARE_CHUNK_SIZE ? COMPARE_CHUNK_SIZE : total_length;
+
+              SVN_ERR(read_chunk(file_baton->file[idx[i]],
+                                 file_baton->path[idx[i]],
+                                 bufp[i], length[i], offset[i]));
+            }
+        }
+
+      len = length[0] > length[1] ? length[1] : length[0];
+      offset[0] += len;
+      offset[1] += len;
+
+      /* Compare two chunks (that could be entire tokens if they both reside
+       * in memory).
+       */
+      *compare = memcmp(bufp[0], bufp[1], len);
+      if (*compare != 0)
+        return SVN_NO_ERROR;
+
+      total_length -= len;
+    }
+  while(total_length > 0);
+
+  *compare = 0;
+  return SVN_NO_ERROR;
 }
 
+
 static
 void
 svn_diff__file_token_discard(void *baton,
                              void *token)
 {
   svn_diff__file_baton_t *file_baton = baton;
+  svn_diff__file_token_t *file_token = token;
 
-  file_baton->reuse_token = file_baton->token == token;
+  file_token->next = file_baton->tokens;
+  file_baton->tokens = file_token;
 }
 
+
 static
 void
 svn_diff__file_token_discard_all(void *baton)
 {
   svn_diff__file_baton_t *file_baton = baton;
 
-  /* This will also close any open files and destroy mmap segments */
+  /* Discard all memory in use by the tokens, and close all open files. */
   svn_pool_clear(file_baton->pool);
 }
 
+
 static const svn_diff_fns_t svn_diff__file_vtable =
 {
   svn_diff__file_datasource_open,
@@ -362,7 +532,7 @@ svn_diff_file_diff4(svn_diff_t **diff,
 
 typedef struct svn_diff__file_output_baton_t
 {
-  apr_file_t *output_file;
+  svn_stream_t *output_stream;
 
   const char *path[2];
   apr_file_t *file[2];
@@ -440,6 +610,9 @@ svn_diff__file_output_unified_line(svn_diff__file_output_baton_t *baton,
                 }
             }
 
+          /* XXX: '\n' doesn't really cut it.  We need to be able to detect
+           * XXX: '\n', '\r' and '\r\n'.
+           */
           eol = memchr(curp, '\n', length);
 
           if (eol != NULL)
@@ -479,20 +652,16 @@ svn_diff__file_output_unified_line(svn_diff__file_output_baton_t *baton,
   while (rv == APR_SUCCESS);
 
   if (rv != APR_SUCCESS && ! APR_STATUS_IS_EOF(rv))
-    {
-      return svn_error_createf(rv, NULL, "Error reading from '%s'.",
-                               baton->path[idx]);
-    }
+    return svn_error_wrap_apr(rv, "Can't read from '%s'", baton->path[idx]);
 
   if (APR_STATUS_IS_EOF(rv))
     {
       /* Special case if we reach the end of file AND the last line is in the
          changed range AND the file doesn't end with a newline */
-      if (bytes_processed && (type == svn_diff__file_output_unified_delete
-                              || type == svn_diff__file_output_unified_insert))
+      if (bytes_processed && (type != svn_diff__file_output_unified_skip))
         {
           svn_stringbuf_appendcstr(baton->hunk,
-                                   "\n\\ No newline at end of file\n");
+            APR_EOL_STR "\\ No newline at end of file" APR_EOL_STR);
         }
 
       baton->length[idx] = 0;
@@ -507,7 +676,6 @@ svn_diff__file_output_unified_flush_hunk(svn_diff__file_output_baton_t *baton)
 {
   apr_off_t target_line;
   apr_size_t hunk_len;
-  apr_status_t rv;
   int i;
 
   if (svn_stringbuf_isempty(baton->hunk))
@@ -536,33 +704,35 @@ svn_diff__file_output_unified_flush_hunk(svn_diff__file_output_baton_t *baton)
 
   /* Output the hunk header.  If the hunk length is 1, the file is a one line
      file.  In this case, surpress the number of lines in the hunk (it is
-     1 implicitly) */
-  apr_file_printf(baton->output_file, "@@ -%" APR_OFF_T_FMT,
-                  baton->hunk_start[0]);
+     1 implicitly) 
+   */
+  SVN_ERR(svn_stream_printf(baton->output_stream, baton->pool,
+                            "@@ -%" APR_OFF_T_FMT,
+                            baton->hunk_start[0]));
   if (baton->hunk_length[0] != 1)
     {
-      apr_file_printf(baton->output_file, ",%" APR_OFF_T_FMT,
-                      baton->hunk_length[0]);
+      SVN_ERR(svn_stream_printf(baton->output_stream, baton->pool,
+                                ",%" APR_OFF_T_FMT,
+                                baton->hunk_length[0]));
     }
 
-  apr_file_printf(baton->output_file, " +%" APR_OFF_T_FMT,
-                  baton->hunk_start[1]);
+  SVN_ERR(svn_stream_printf(baton->output_stream, baton->pool,
+                            " +%" APR_OFF_T_FMT,
+                            baton->hunk_start[1]));
   if (baton->hunk_length[1] != 1)
     {
-      apr_file_printf(baton->output_file, ",%" APR_OFF_T_FMT,
-                      baton->hunk_length[1]);
+      SVN_ERR(svn_stream_printf(baton->output_stream, baton->pool,
+                                ",%" APR_OFF_T_FMT,
+                                baton->hunk_length[1]));
     }
 
-  apr_file_printf(baton->output_file, " @@\n");
+  SVN_ERR(svn_stream_printf(baton->output_stream, baton->pool,
+                            " @@" APR_EOL_STR));
 
   /* Output the hunk content */
   hunk_len = baton->hunk->len;
-  rv = apr_file_write(baton->output_file, baton->hunk->data, &hunk_len);
-  if (rv != APR_SUCCESS)
-    {
-      return svn_error_create(rv, NULL,
-               "svn_diff_file_output_unified: error writing hunk.");
-    }
+  SVN_ERR (svn_stream_write(baton->output_stream, baton->hunk->data,
+                            &hunk_len));
 
   /* Prepare for the next hunk */
   baton->hunk_length[0] = 0;
@@ -671,7 +841,7 @@ static const svn_diff_output_fns_t svn_diff__file_output_unified_vtable =
 };
 
 svn_error_t *
-svn_diff_file_output_unified(apr_file_t *output_file,
+svn_diff_file_output_unified(svn_stream_t *output_stream,
                              svn_diff_t *diff,
                              const char *original_path,
                              const char *modified_path,
@@ -685,7 +855,7 @@ svn_diff_file_output_unified(apr_file_t *output_file,
   if (svn_diff_contains_diffs(diff))
     {
       memset(&baton, 0, sizeof(baton));
-      baton.output_file = output_file;
+      baton.output_stream = output_stream;
       baton.pool = pool;
       baton.path[0] = original_path;
       baton.path[1] = modified_path;
@@ -709,10 +879,10 @@ svn_diff_file_output_unified(apr_file_t *output_file,
             svn_diff__file_output_unified_default_hdr(pool, modified_path);
         }
 
-      SVN_ERR( svn_io_file_printf(output_file,
-                                  "--- %s\n"
-                                  "+++ %s\n",
-                                  original_header, modified_header) );
+      SVN_ERR(svn_stream_printf(output_stream, pool,
+                                "--- %s" APR_EOL_STR
+                                "+++ %s" APR_EOL_STR,
+                                original_header, modified_header));
 
       SVN_ERR(svn_diff_output(diff, &baton,
                               &svn_diff__file_output_unified_vtable));
@@ -720,13 +890,7 @@ svn_diff_file_output_unified(apr_file_t *output_file,
 
       for (i = 0; i < 2; i++)
         {
-          apr_status_t rv = apr_file_close(baton.file[i]);
-          if (rv != APR_SUCCESS)
-            {
-              return svn_error_createf(rv, NULL,
-                                       "Failed to close file '%s'.",
-                                       baton.path[i]);
-            }
+	  SVN_ERR (svn_io_file_close(baton.file[i], pool));
         }
     }
 
@@ -738,7 +902,7 @@ svn_diff_file_output_unified(apr_file_t *output_file,
 
 typedef struct svn_diff3__file_output_baton_t
 {
-  apr_file_t *output_file;
+  svn_stream_t *output_stream;
 
   const char *path[3];
 
@@ -776,7 +940,6 @@ svn_diff3__file_output_line(svn_diff3__file_output_baton_t *baton,
   char *endp;
   char *eol;
   apr_size_t len;
-  apr_status_t rv;
 
   curp = baton->curp[idx];
   endp = baton->endp[idx];
@@ -788,6 +951,9 @@ svn_diff3__file_output_line(svn_diff3__file_output_baton_t *baton,
   if (curp == endp)
     return SVN_NO_ERROR;
 
+  /* XXX: '\n' doesn't really cut it.  We need to be able to detect
+   * XXX: '\n', '\r' and '\r\n'.
+   */
   eol = memchr(curp, '\n', endp - curp);
   if (!eol)
     eol = endp;
@@ -797,12 +963,7 @@ svn_diff3__file_output_line(svn_diff3__file_output_baton_t *baton,
   if (type != svn_diff3__file_output_skip)
     {
       len = eol - curp;
-      rv = apr_file_write(baton->output_file, curp, &len);
-      if (rv != APR_SUCCESS)
-        {
-          return svn_error_create(rv, NULL,
-            "svn_diff3_file_output: error writing file.");
-        }
+      SVN_ERR(svn_stream_write(baton->output_stream, curp, &len));
     }
 
   baton->curp[idx] = eol;
@@ -894,8 +1055,8 @@ svn_diff3__file_output_conflict(void *baton,
   apr_off_t latest_start, apr_off_t latest_length,
   svn_diff_t *diff)
 {
-  apr_status_t rv;
   svn_diff3__file_output_baton_t *file_baton = baton;
+  apr_size_t len;
 
   if (diff && file_baton->display_resolved_conflicts)
     {
@@ -903,59 +1064,52 @@ svn_diff3__file_output_conflict(void *baton,
                              &svn_diff3__file_output_vtable);
     }
 
-  rv = apr_file_puts(file_baton->conflict_modified, file_baton->output_file);
-  if (rv != APR_SUCCESS)
-    {
-      return svn_error_create(rv, NULL,
-        "svn_diff3_file_output: error writing file.");
-    }
-
-  apr_file_putc('\n', file_baton->output_file);
+  len = strlen(file_baton->conflict_modified);
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           file_baton->conflict_modified,
+                           &len));
+  len = sizeof(APR_EOL_STR) - 1;
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           APR_EOL_STR, &len));
 
   SVN_ERR(svn_diff3__file_output_hunk(baton, 1,
             modified_start, modified_length));
 
   if (file_baton->display_original_in_conflict)
     {
-      rv = apr_file_puts(file_baton->conflict_original, file_baton->output_file);
-      if (rv != APR_SUCCESS)
-        {
-          return svn_error_create(rv, NULL,
-            "svn_diff3_file_output: error writing file.");
-        }
-
-      apr_file_putc('\n', file_baton->output_file);
+      len = strlen(file_baton->conflict_original);
+      SVN_ERR(svn_stream_write(file_baton->output_stream,
+                               file_baton->conflict_original, &len));
+      len = sizeof(APR_EOL_STR) - 1;
+      SVN_ERR(svn_stream_write(file_baton->output_stream,
+                               APR_EOL_STR, &len));
 
       SVN_ERR(svn_diff3__file_output_hunk(baton, 0,
               original_start, original_length));
     }
 
-  rv = apr_file_puts(file_baton->conflict_separator, file_baton->output_file);
-  if (rv != APR_SUCCESS)
-    {
-      return svn_error_create(rv, NULL,
-        "svn_diff3_file_output: error writing file.");
-    }
-
-  apr_file_putc('\n', file_baton->output_file);
+  len = strlen(file_baton->conflict_separator);
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           file_baton->conflict_separator, &len));
+  len = sizeof(APR_EOL_STR) - 1;
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           APR_EOL_STR, &len));
 
   SVN_ERR(svn_diff3__file_output_hunk(baton, 2,
             latest_start, latest_length));
 
-  rv = apr_file_puts(file_baton->conflict_latest, file_baton->output_file);
-  if (rv != APR_SUCCESS)
-    {
-      return svn_error_create(rv, NULL,
-        "svn_diff3_file_output: error writing file.");
-    }
-
-  apr_file_putc('\n', file_baton->output_file);
+  len = strlen(file_baton->conflict_latest);
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           file_baton->conflict_latest, &len));
+  len = sizeof(APR_EOL_STR) - 1;
+  SVN_ERR(svn_stream_write(file_baton->output_stream,
+                           APR_EOL_STR, &len));
 
   return SVN_NO_ERROR;
 }
 
 svn_error_t *
-svn_diff_file_output_merge(apr_file_t *output_file,
+svn_diff_file_output_merge(svn_stream_t *output_stream,
                            svn_diff_t *diff,
                            const char *original_path,
                            const char *modified_path,
@@ -971,14 +1125,13 @@ svn_diff_file_output_merge(apr_file_t *output_file,
   svn_diff3__file_output_baton_t baton;
   apr_file_t *file[3];
   apr_off_t size;
-  apr_status_t rv;
   int idx;
 #if APR_HAS_MMAP
   apr_mmap_t *mm[3] = { 0 };
 #endif /* APR_HAS_MMAP */
 
   memset(&baton, 0, sizeof(baton));
-  baton.output_file = output_file;
+  baton.output_stream = output_stream;
   baton.pool = pool;
   baton.path[0] = original_path;
   baton.path[1] = modified_path;
@@ -1018,23 +1171,18 @@ svn_diff_file_output_merge(apr_file_t *output_file,
 #if APR_HAS_MMAP
       if (mm[idx])
         {
-          rv = apr_mmap_delete(mm[idx]);
+          apr_status_t rv = apr_mmap_delete(mm[idx]);
           if (rv != APR_SUCCESS)
             {
-              return svn_error_createf(rv, NULL, "Failed to delete mmap '%s'.",
-                                       baton.path[idx]);
+              return svn_error_wrap_apr(rv, "Failed to delete mmap '%s'",
+                                        baton.path[idx]);
             }
         }
 #endif /* APR_HAS_MMAP */
 
       if (file[idx])
         {
-          rv = apr_file_close(file[idx]);
-          if (rv != APR_SUCCESS)
-            {
-              return svn_error_createf(rv, NULL, "Failed to close file '%s'.",
-                                       baton.path[idx]);
-            }
+          SVN_ERR (svn_io_file_close(file[idx], pool));
         }
     }
 
