@@ -1,7 +1,7 @@
 /* dag.c : DAG-like interface filesystem, private to libsvn_fs
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -76,6 +76,9 @@ struct dag_node_t
      `get_node_revision' and `set_node_revision', which take care of
      things for you.  */
   svn_fs__node_revision_t *node_revision;
+
+  /* the path at which this node was created. */
+  const char *created_path;
 };
 
 
@@ -87,20 +90,6 @@ svn_node_kind_t svn_fs__dag_node_kind (dag_node_t *node)
 }
 
 
-int 
-svn_fs__dag_is_file (dag_node_t *node)
-{
-  return (node->kind == svn_node_file);
-}
-
-
-int 
-svn_fs__dag_is_directory (dag_node_t *node)
-{
-  return (node->kind == svn_node_dir);
-}
-
-
 const svn_fs_id_t *
 svn_fs__dag_get_id (dag_node_t *node)
 {
@@ -108,6 +97,13 @@ svn_fs__dag_get_id (dag_node_t *node)
 }
 
 
+const char *
+svn_fs__dag_get_created_path (dag_node_t *node)
+{
+  return node->created_path;
+}
+
+
 svn_fs_t *
 svn_fs__dag_get_fs (dag_node_t *node)
 {
@@ -140,7 +136,8 @@ copy_node_revision (svn_fs__node_revision_t *noderev,
     nr->data_key = apr_pstrdup (pool, noderev->data_key);
   if (noderev->edit_key)
     nr->edit_key = apr_pstrdup (pool, noderev->edit_key);
-
+  if (noderev->created_path)
+    nr->created_path = apr_pstrdup (pool, noderev->created_path);
   return nr;
 }
 
@@ -232,10 +229,10 @@ set_node_revision (dag_node_t *node,
 }
 
 
-int svn_fs__dag_check_mutable (dag_node_t *node, 
-                               const char *txn_id)
+svn_boolean_t svn_fs__dag_check_mutable (dag_node_t *node, 
+                                         const char *txn_id)
 {
-  return (! strcmp (svn_fs__id_txn_id (svn_fs__dag_get_id (node)), txn_id));
+  return (strcmp (svn_fs__id_txn_id (svn_fs__dag_get_id (node)), txn_id) == 0);
 }
 
 
@@ -254,12 +251,13 @@ svn_fs__dag_get_node (dag_node_t **node,
   new_node->id = svn_fs__id_copy (id, trail->pool); 
   new_node->pool = trail->pool;
 
-  /* Grab the contents so we can inspect the node's kind. */
+  /* Grab the contents so we can inspect the node's kind and created path. */
   SVN_ERR (get_node_revision (&noderev, new_node, trail));
 
-  /* Initialize the KIND attribute */
+  /* Initialize the KIND and CREATED_PATH attributes */
   new_node->kind = noderev->kind;
-  
+  new_node->created_path = apr_pstrdup (trail->pool, noderev->created_path);
+
   /* Return a fresh new node */
   *node = new_node;
   return SVN_NO_ERROR;
@@ -271,18 +269,11 @@ svn_fs__dag_get_revision (svn_revnum_t *rev,
                           dag_node_t *node,
                           trail_t *trail)
 {
-  svn_fs__transaction_t *txn;
-
-  /* Get the txn ID from the node revision ID. */
-  const char *txn_id = svn_fs__id_txn_id (svn_fs__dag_get_id (node));
-  
-  /* Use the txn ID to look up the transaction.  */
-  SVN_ERR (svn_fs__bdb_get_txn (&txn, svn_fs__dag_get_fs (node), txn_id, trail));
-
-  /* If the transaction has been committed, we can return the revision
-     number, else we return the invalid revision number. */
-  *rev = txn->revision;
-  return SVN_NO_ERROR;
+  /* Use the txn ID from the NODE's id to look up the transaction and
+     get its revision number.  */
+  return svn_fs__txn_get_revision 
+    (rev, svn_fs__dag_get_fs (node), 
+     svn_fs__id_txn_id (svn_fs__dag_get_id (node)), trail);
 }
 
 
@@ -320,7 +311,7 @@ svn_fs__dag_walk_predecessors (dag_node_t *node,
 {
   svn_fs_t *fs = svn_fs__dag_get_fs (node);
   dag_node_t *this_node;
-  int done = 0;
+  svn_boolean_t done = FALSE;
 
   this_node = node;
   while ((! done) && this_node)
@@ -351,12 +342,12 @@ svn_fs__dag_walk_predecessors (dag_node_t *node,
 
 /* Trail body for svn_fs__dag_init_fs. */
 static svn_error_t *
-txn_body_dag_init_fs (void *fs_baton, trail_t *trail)
+txn_body_dag_init_fs (void *baton, trail_t *trail)
 {
   svn_fs__node_revision_t noderev;
   svn_fs__revision_t revision;
   svn_revnum_t rev = SVN_INVALID_REVNUM;
-  svn_fs_t *fs = fs_baton;
+  svn_fs_t *fs = trail->fs;
   svn_string_t date;
   const char *txn_id;
   const char *copy_id;
@@ -365,6 +356,7 @@ txn_body_dag_init_fs (void *fs_baton, trail_t *trail)
   /* Create empty root directory with node revision 0.0.0. */
   memset (&noderev, 0, sizeof (noderev));
   noderev.kind = svn_node_dir;
+  noderev.created_path = "/";
   SVN_ERR (svn_fs__bdb_put_node_revision (fs, root_id, &noderev, trail));
 
   /* Create a new transaction (better have an id of "0") */
@@ -372,23 +364,25 @@ txn_body_dag_init_fs (void *fs_baton, trail_t *trail)
   if (strcmp (txn_id, "0"))
     return svn_error_createf 
       (SVN_ERR_FS_CORRUPT, 0,
-       "initial transaction id not `0' in filesystem `%s'", fs->path);
+       "Corrupt DB: initial transaction id not '0' in filesystem '%s'",
+       fs->path);
 
   /* Create a default copy (better have an id of "0") */
   SVN_ERR (svn_fs__bdb_reserve_copy_id (&copy_id, fs, trail));
   if (strcmp (copy_id, "0"))
     return svn_error_createf 
       (SVN_ERR_FS_CORRUPT, 0,
-       "initial copy id not `0' in filesystem `%s'", fs->path);
-  SVN_ERR (svn_fs__bdb_create_copy (copy_id, fs, NULL, NULL, root_id, trail));
+       "Corrupt DB: initial copy id not '0' in filesystem '%s'", fs->path);
+  SVN_ERR (svn_fs__bdb_create_copy (fs, copy_id, NULL, NULL, root_id, 
+                                    svn_fs__copy_kind_real, trail));
 
   /* Link it into filesystem revision 0. */
   revision.txn_id = txn_id;
   SVN_ERR (svn_fs__bdb_put_rev (&rev, fs, &revision, trail));
   if (rev != 0)
     return svn_error_createf (SVN_ERR_FS_CORRUPT, 0,
-                              "initial revision number is not `0'"
-                              " in filesystem `%s'", fs->path);
+                              "Corrupt DB: initial revision number is not '0'"
+                              " in filesystem '%s'", fs->path);
 
   /* Promote our transaction to a "committed" transaction. */
   SVN_ERR (svn_fs__txn_make_committed (fs, txn_id, rev, trail));
@@ -403,7 +397,7 @@ txn_body_dag_init_fs (void *fs_baton, trail_t *trail)
 svn_error_t *
 svn_fs__dag_init_fs (svn_fs_t *fs)
 {
-  return svn_fs__retry_txn (fs, txn_body_dag_init_fs, fs, fs->pool);
+  return svn_fs__retry_txn (fs, txn_body_dag_init_fs, NULL, fs->pool);
 }
 
 
@@ -413,9 +407,10 @@ svn_fs__dag_init_fs (svn_fs_t *fs)
 /* Some of these are helpers for functions outside this section. */
 
 /* Given directory NODEREV in FS, set *ENTRIES_P to its entries list
-   hash, as part of TRAIL, or to NULL if NODEREV has no entries.
-   The entries list will be allocated in TRAIL->pool.  If NODEREV is
-   not a directory, return the error SVN_ERR_FS_NOT_DIRECTORY. */
+   hash, as part of TRAIL, or to NULL if NODEREV has no entries.  The
+   entries list will be allocated in TRAIL->pool, and the entries in
+   that list will not have interesting value in their 'kind' fields.
+   If NODEREV is not a directory, return the error SVN_ERR_FS_NOT_DIRECTORY. */
 static svn_error_t *
 get_dir_entries (apr_hash_t **entries_p,
                  svn_fs_t *fs,
@@ -466,6 +461,7 @@ get_dir_entries (apr_hash_t **entries_p,
       apr_hash_this (hi, &key, &klen, &val);
       dirent->name = (char *) key;
       dirent->id = val;
+      dirent->kind = svn_node_unknown;
       apr_hash_set (*entries_p, key, klen, (void *) dirent);
     }
 
@@ -582,6 +578,7 @@ set_entry (dag_node_t *parent,
 static svn_error_t *
 make_entry (dag_node_t **child_p,
             dag_node_t *parent,
+            const char *parent_path,
             const char *name,
             svn_boolean_t is_dir,
             const char *txn_id,
@@ -594,10 +591,10 @@ make_entry (dag_node_t **child_p,
   if (! svn_path_is_single_path_component (name))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to create a node with an illegal name `%s'", name);
+       "Attempted to create a node with an illegal name '%s'", name);
 
   /* Make sure that parent is a directory */
-  if (! svn_fs__dag_is_directory (parent))
+  if (parent->kind != svn_node_dir)
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
        "Attempted to create entry in non-directory parent");
@@ -618,6 +615,7 @@ make_entry (dag_node_t **child_p,
   /* Create the new node's NODE-REVISION */
   memset (&new_noderev, 0, sizeof (new_noderev));
   new_noderev.kind = is_dir ? svn_node_dir : svn_node_file;
+  new_noderev.created_path = svn_path_join (parent_path, name, trail->pool);
   SVN_ERR (svn_fs__create_node 
            (&new_node_id, svn_fs__dag_get_fs (parent),
             &new_noderev, svn_fs__id_copy_id (svn_fs__dag_get_id (parent)),
@@ -657,16 +655,16 @@ svn_fs__dag_set_entry (dag_node_t *node,
                        trail_t *trail)
 {
   /* Check it's a directory. */
-  if (! svn_fs__dag_is_directory (node))
+  if (node->kind != svn_node_dir)
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to set entry in non-directory node.");
+       "Attempted to set entry in non-directory node");
   
   /* Check it's mutable. */
   if (! svn_fs__dag_check_mutable (node, txn_id))
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to set entry in immutable node.");
+       "Attempted to set entry in immutable node");
 
   return set_entry (node, entry_name, id, txn_id, trail);
 }
@@ -726,7 +724,7 @@ svn_fs__dag_set_proplist (dag_node_t *node,
       svn_string_t *idstr = svn_fs_unparse_id (node->id, node->pool);
       return svn_error_createf 
         (SVN_ERR_FS_NOT_MUTABLE, NULL,
-         "Can't set_proplist on *immutable* node-revision %s", idstr->data);
+         "Can't set proplist on *immutable* node-revision %s", idstr->data);
     }
 
   /* Go get a fresh NODE-REVISION for this node. */
@@ -810,6 +808,7 @@ svn_fs__dag_txn_base_root (dag_node_t **node_p,
 svn_error_t *
 svn_fs__dag_clone_child (dag_node_t **child_p,
                          dag_node_t *parent,
+                         const char *parent_path,
                          const char *name,
                          const char *copy_id,
                          const char *txn_id,
@@ -829,7 +828,7 @@ svn_fs__dag_clone_child (dag_node_t **child_p,
   if (! svn_path_is_single_path_component (name))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to make a child clone with an illegal name `%s'", name);
+       "Attempted to make a child clone with an illegal name '%s'", name);
 
   /* Find the node named NAME in PARENT's entries list if it exists. */
   SVN_ERR (svn_fs__dag_open (&cur_entry, parent, name, trail));
@@ -852,6 +851,7 @@ svn_fs__dag_clone_child (dag_node_t **child_p,
       noderev->predecessor_id = svn_fs__id_copy (cur_entry->id, trail->pool);
       if (noderev->predecessor_count != -1)
         noderev->predecessor_count++;
+      noderev->created_path = svn_path_join (parent_path, name, trail->pool);
       SVN_ERR (svn_fs__create_successor (&new_node_id, fs, cur_entry->id, 
                                          noderev, copy_id, txn_id, trail));
       
@@ -887,7 +887,8 @@ svn_fs__dag_clone_root (dag_node_t **root_p,
       /* Of my own flesh and bone...
          (Get the NODE-REVISION for the base node, and then write
          it back out as the clone.) */
-      SVN_ERR (svn_fs__bdb_get_node_revision (&noderev, fs, base_root_id, trail));
+      SVN_ERR (svn_fs__bdb_get_node_revision (&noderev, fs, 
+                                              base_root_id, trail));
 
       /* Store it. */
       /* ### todo: Does it even makes sense to have a different copy id for
@@ -927,12 +928,11 @@ svn_fs__dag_clone_root (dag_node_t **root_p,
 
    If return SVN_ERR_FS_NO_SUCH_ENTRY, then there is no entry NAME in
    PARENT.  */
-static svn_error_t *
-delete_entry (dag_node_t *parent,
-              const char *name,
-              svn_boolean_t require_empty,
-              const char *txn_id,
-              trail_t *trail)
+svn_error_t *
+svn_fs__dag_delete (dag_node_t *parent,
+                    const char *name,
+                    const char *txn_id,
+                    trail_t *trail)
 {
   svn_fs__node_revision_t *parent_noderev;
   const char *rep_key, *mutable_rep_key;
@@ -944,22 +944,22 @@ delete_entry (dag_node_t *parent,
   dag_node_t *node; 
 
   /* Make sure parent is a directory. */
-  if (! svn_fs__dag_is_directory (parent))
+  if (parent->kind != svn_node_dir)
     return svn_error_createf
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to delete entry `%s' from *non*-directory node.", name);    
+       "Attempted to delete entry '%s' from *non*-directory node", name);    
 
   /* Make sure parent is mutable. */
   if (! svn_fs__dag_check_mutable (parent, txn_id))
     return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Attempted to delete entry `%s' from immutable directory node.", name);
+       "Attempted to delete entry '%s' from immutable directory node", name);
 
   /* Make sure that NAME is a single path component. */
   if (! svn_path_is_single_path_component (name))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to delete a node with an illegal name `%s'", name);
+       "Attempted to delete a node with an illegal name '%s'", name);
 
   /* Get a fresh NODE-REVISION for the parent node. */
   SVN_ERR (get_node_revision (&parent_noderev, parent, trail));
@@ -973,7 +973,7 @@ delete_entry (dag_node_t *parent,
   if (! rep_key)
     return svn_error_createf 
       (SVN_ERR_FS_NO_SUCH_ENTRY, NULL,
-       "Delete failed--directory has no entry `%s'", name);
+       "Delete failed--directory has no entry '%s'", name);
 
   /* Ensure we have a key to a mutable representation of the entries
      list.  We'll have to update the NODE-REVISION if it points to an
@@ -1007,22 +1007,11 @@ delete_entry (dag_node_t *parent,
   if (! id)
     return svn_error_createf 
       (SVN_ERR_FS_NO_SUCH_ENTRY, NULL,
-       "Delete failed--directory has no entry `%s'", name);
+       "Delete failed--directory has no entry '%s'", name);
 
-  /* Use the ID of this ENTRY to get the entry's node.  If the node we
-     get is a directory, make sure it meets up to our emptiness
-     standards (as determined by REQUIRE_EMPTY).  */
+  /* Use the ID of this ENTRY to get the entry's node.  */
   SVN_ERR (svn_fs__dag_get_node (&node, svn_fs__dag_get_fs (parent), 
                                  id, trail));
-  if (svn_fs__dag_is_directory (node))
-    {
-      apr_hash_t *entries_here;
-      SVN_ERR (svn_fs__dag_dir_entries (&entries_here, node, trail));
-      if (require_empty && entries_here && apr_hash_count (entries_here))
-        return svn_error_createf
-          (SVN_ERR_DIR_NOT_EMPTY, NULL,
-           "Attempt to delete non-empty directory `%s'.", name);
-    }
 
   /* If mutable, remove it and any mutable children from db. */
   SVN_ERR (svn_fs__dag_delete_if_mutable (parent->fs, id, txn_id, trail));
@@ -1052,68 +1041,21 @@ delete_entry (dag_node_t *parent,
 
 
 svn_error_t *
-svn_fs__dag_delete (dag_node_t *parent,
-                    const char *name,
-                    const char *txn_id,
-                    trail_t *trail)
-{
-  return delete_entry (parent, name, TRUE, txn_id, trail);
-}
-
-
-svn_error_t *
-svn_fs__dag_delete_tree (dag_node_t *parent,
-                         const char *name,
+svn_fs__dag_remove_node (svn_fs_t *fs,
+                         const svn_fs_id_t *id,
                          const char *txn_id,
                          trail_t *trail)
-{
-  return delete_entry (parent, name, FALSE, txn_id, trail);
-}
-
-
-svn_error_t *
-svn_fs__dag_delete_if_mutable (svn_fs_t *fs,
-                               const svn_fs_id_t *id,
-                               const char *txn_id,
-                               trail_t *trail)
 {
   dag_node_t *node;
   svn_fs__node_revision_t *noderev;
 
+  /* Fetch the node. */
   SVN_ERR (svn_fs__dag_get_node (&node, fs, id, trail));
 
   /* If immutable, do nothing and return immediately. */
   if (! svn_fs__dag_check_mutable (node, txn_id))
-    return SVN_NO_ERROR;
-
-  /* Else it's mutable.  Recurse on directories... */
-  if (svn_fs__dag_is_directory (node))
-    {
-      apr_hash_t *entries;
-      apr_hash_index_t *hi;
-      apr_pool_t *pool = trail->pool;
-
-      /* Loop over hash entries */
-      SVN_ERR (svn_fs__dag_dir_entries (&entries, node, trail));
-      if (entries)
-        {
-          for (hi = apr_hash_first (pool, entries); 
-               hi; 
-               hi = apr_hash_next (hi))
-            {
-              void *val;
-              svn_fs_dirent_t *dirent;
-              
-              apr_hash_this (hi, NULL, NULL, &val);
-              dirent = val;
-              SVN_ERR (svn_fs__dag_delete_if_mutable (fs, dirent->id, 
-                                                      txn_id, trail));
-            }
-        }
-    }
-
-  /* ... then delete the node itself, after deleting any mutable
-     representations and strings it points to. */
+    return svn_error_createf (SVN_ERR_FS_NOT_MUTABLE, NULL, 
+                              "Attempted removal of immutable node");
 
   /* Get a fresh node-revision. */
   SVN_ERR (svn_fs__bdb_get_node_revision (&noderev, fs, id, trail));
@@ -1140,78 +1082,78 @@ svn_fs__dag_delete_if_mutable (svn_fs_t *fs,
 }
 
 
+svn_error_t *
+svn_fs__dag_delete_if_mutable (svn_fs_t *fs,
+                               const svn_fs_id_t *id,
+                               const char *txn_id,
+                               trail_t *trail)
+{
+  dag_node_t *node;
+
+  /* Get the node. */
+  SVN_ERR (svn_fs__dag_get_node (&node, fs, id, trail));
+
+  /* If immutable, do nothing and return immediately. */
+  if (! svn_fs__dag_check_mutable (node, txn_id))
+    return SVN_NO_ERROR;
+
+  /* Else it's mutable.  Recurse on directories... */
+  if (node->kind == svn_node_dir)
+    {
+      apr_hash_t *entries;
+      apr_hash_index_t *hi;
+      apr_pool_t *pool = trail->pool;
+
+      /* Loop over hash entries */
+      SVN_ERR (svn_fs__dag_dir_entries (&entries, node, trail));
+      if (entries)
+        {
+          for (hi = apr_hash_first (pool, entries); 
+               hi; 
+               hi = apr_hash_next (hi))
+            {
+              void *val;
+              svn_fs_dirent_t *dirent;
+              
+              apr_hash_this (hi, NULL, NULL, &val);
+              dirent = val;
+              SVN_ERR (svn_fs__dag_delete_if_mutable (fs, dirent->id, 
+                                                      txn_id, trail));
+            }
+        }
+    }
+
+  /* ... then delete the node itself, after deleting any mutable
+     representations and strings it points to. */
+  SVN_ERR (svn_fs__dag_remove_node (fs, id, txn_id, trail));
+  
+  return SVN_NO_ERROR;
+}
+
+
 svn_error_t *
 svn_fs__dag_make_file (dag_node_t **child_p,
                        dag_node_t *parent,
+                       const char *parent_path,
                        const char *name,
                        const char *txn_id, 
                        trail_t *trail)
 {
   /* Call our little helper function */
-  return make_entry (child_p, parent, name, FALSE, txn_id, trail);
+  return make_entry (child_p, parent, parent_path, name, FALSE, txn_id, trail);
 }
 
 
 svn_error_t *
 svn_fs__dag_make_dir (dag_node_t **child_p,
                       dag_node_t *parent,
+                      const char *parent_path,
                       const char *name,
                       const char *txn_id, 
                       trail_t *trail)
 {
   /* Call our little helper function */
-  return make_entry (child_p, parent, name, TRUE, txn_id, trail);
-}
-
-
-/* ### somebody todo: figure out why this *reaaaaaaally* exists.  It
-   has no callers (though kfogel has some speculation about a possible
-   use (see tree.c:merge) */
-svn_error_t *
-svn_fs__dag_link (dag_node_t *parent,
-                  dag_node_t *child,
-                  const char *name,
-                  const char *txn_id, 
-                  trail_t *trail)
-{
-  const svn_fs_id_t *entry_id;
-
-  /* Make sure that parent is a directory */
-  if (! svn_fs__dag_is_directory (parent))
-    return svn_error_createf 
-      (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to create entry in non-directory parent");
-    
-  /* Make sure parent is mutable */
-  if (! svn_fs__dag_check_mutable (parent, txn_id))
-    return svn_error_createf 
-      (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Can't add a link from an immutable parent");
-
-  /* Make sure child is IMmutable */
-  if (svn_fs__dag_check_mutable (child, txn_id))
-    return svn_error_createf 
-      (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Can't add a link to a mutable child");
-
-  /* Make sure that NAME is a single path component. */
-  if (! svn_path_is_single_path_component (name))
-    return svn_error_createf 
-      (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to link to a node with an illegal name `%s'", name);
-
-  /* Verify that this parent node does not already have an entry named NAME. */
-  SVN_ERR (dir_entry_id_from_node (&entry_id, parent, name, trail));
-  if (entry_id)
-    return svn_error_createf 
-      (SVN_ERR_FS_ALREADY_EXISTS, NULL,
-       "Attempted to create entry that already exists");
-
-  /* We can safely call set_entry because we already know that PARENT
-     is mutable, and we know that CHILD is immutable (since every
-     parent of a mutable node is mutable itself, we know that CHILD
-     can't be equal to, or a parent of, PARENT).  */
-  return set_entry (parent, name, svn_fs__dag_get_id (child), txn_id, trail);
+  return make_entry (child_p, parent, parent_path, name, TRUE, txn_id, trail);
 }
 
 
@@ -1224,10 +1166,10 @@ svn_fs__dag_get_contents (svn_stream_t **contents,
   svn_fs__node_revision_t *noderev;
 
   /* Make sure our node is a file. */
-  if (! svn_fs__dag_is_file (file))
+  if (file->kind != svn_node_file)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FILE, NULL,
-       "Attempted to get textual contents of a *non*-file node.");
+       "Attempted to get textual contents of a *non*-file node");
   
   /* Go get a fresh node-revision for FILE. */
   SVN_ERR (get_node_revision (&noderev, file, trail));
@@ -1257,10 +1199,10 @@ svn_fs__dag_file_length (svn_filesize_t *length,
   svn_fs__node_revision_t *noderev;
 
   /* Make sure our node is a file. */
-  if (! svn_fs__dag_is_file (file))
+  if (file->kind != svn_node_file)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FILE, NULL,
-       "Attempted to get length of a *non*-file node.");
+       "Attempted to get length of a *non*-file node");
 
   /* Go get a fresh node-revision for FILE, and . */
   SVN_ERR (get_node_revision (&noderev, file, trail));
@@ -1281,17 +1223,17 @@ svn_fs__dag_file_checksum (unsigned char digest[],
 { 
   svn_fs__node_revision_t *noderev;
 
-  if (! svn_fs__dag_is_file (file))
+  if (file->kind != svn_node_file)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FILE, NULL,
-       "Attempted to get checksum of a *non*-file node.");
+       "Attempted to get checksum of a *non*-file node");
 
   SVN_ERR (get_node_revision (&noderev, file, trail));
   if (noderev->data_key)
     SVN_ERR (svn_fs__rep_contents_checksum (digest, file->fs,
                                             noderev->data_key, trail));
   else
-    memset (digest, 0, MD5_DIGESTSIZE);
+    memset (digest, 0, APR_MD5_DIGESTSIZE);
 
   return SVN_NO_ERROR;
 }
@@ -1310,16 +1252,16 @@ svn_fs__dag_get_edit_stream (svn_stream_t **contents,
   svn_stream_t *ws;
 
   /* Make sure our node is a file. */
-  if (! svn_fs__dag_is_file (file))
+  if (file->kind != svn_node_file)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FILE, NULL,
-       "Attempted to set textual contents of a *non*-file node.");
+       "Attempted to set textual contents of a *non*-file node");
   
   /* Make sure our node is mutable. */
   if (! svn_fs__dag_check_mutable (file, txn_id))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Attempted to set textual contents of an immutable node.");
+       "Attempted to set textual contents of an immutable node");
 
   /* Get the node revision. */
   SVN_ERR (get_node_revision (&noderev, file, trail));
@@ -1360,16 +1302,16 @@ svn_fs__dag_finalize_edits (dag_node_t *file,
   const char *old_data_key;
   
   /* Make sure our node is a file. */
-  if (! svn_fs__dag_is_file (file))
+  if (file->kind != svn_node_file)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FILE, NULL,
-       "Attempted to set textual contents of a *non*-file node.");
+       "Attempted to set textual contents of a *non*-file node");
   
   /* Make sure our node is mutable. */
   if (! svn_fs__dag_check_mutable (file, txn_id))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Attempted to set textual contents of an immutable node.");
+       "Attempted to set textual contents of an immutable node");
 
   /* Get the node revision. */
   SVN_ERR (get_node_revision (&noderev, file, trail));
@@ -1380,7 +1322,7 @@ svn_fs__dag_finalize_edits (dag_node_t *file,
 
   if (checksum)
     {
-      unsigned char digest[MD5_DIGESTSIZE];
+      unsigned char digest[APR_MD5_DIGESTSIZE];
       const char *hex;
 
       SVN_ERR (svn_fs__rep_contents_checksum
@@ -1391,7 +1333,7 @@ svn_fs__dag_finalize_edits (dag_node_t *file,
         return svn_error_createf
           (SVN_ERR_CHECKSUM_MISMATCH, 
            NULL,
-           "svn_fs__dag_finalize_edits: checksum mismatch, rep \"%s\":\n"
+           "Checksum mismatch, rep '%s':\n"
            "   expected:  %s\n"
            "     actual:  %s\n",
            noderev->edit_key, checksum, hex);
@@ -1418,15 +1360,16 @@ svn_fs__dag_finalize_edits (dag_node_t *file,
 
 dag_node_t *
 svn_fs__dag_dup (dag_node_t *node,
-                 trail_t *trail)
+                 apr_pool_t *pool)
 {
   /* Allocate our new node. */
-  dag_node_t *new_node = apr_pcalloc (trail->pool, sizeof (*new_node));
+  dag_node_t *new_node = apr_pcalloc (pool, sizeof (*new_node));
 
   new_node->fs = node->fs;
-  new_node->pool = trail->pool;
-  new_node->id = svn_fs__id_copy (node->id, node->pool);
+  new_node->pool = pool;
+  new_node->id = svn_fs__id_copy (node->id, pool);
   new_node->kind = node->kind;
+  new_node->created_path = apr_pstrdup (pool, node->created_path);
 
   /* Leave new_node->node_revision zero for now, so it'll get read in.
      We can get fancy and duplicate node's cache later.  */
@@ -1448,13 +1391,13 @@ svn_fs__dag_open (dag_node_t **child_p,
   if (! node_id)
     return svn_error_createf 
       (SVN_ERR_FS_NOT_FOUND, NULL,
-       "Attempted to open non-existant child node `%s'", name);
+       "Attempted to open non-existant child node '%s'", name);
   
   /* Make sure that NAME is a single path component. */
   if (! svn_path_is_single_path_component (name))
     return svn_error_createf 
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to open node with an illegal name `%s'", name);
+       "Attempted to open node with an illegal name '%s'", name);
 
   /* Now get the node that was requested. */
   return svn_fs__dag_get_node (child_p, svn_fs__dag_get_fs (parent),
@@ -1494,6 +1437,9 @@ svn_fs__dag_copy (dag_node_t *to_node,
       to_noderev->predecessor_id = svn_fs__id_copy (src_id, trail->pool);
       if (to_noderev->predecessor_count != -1)
         to_noderev->predecessor_count++;
+      to_noderev->created_path = 
+        svn_path_join (svn_fs__dag_get_created_path (to_node), entry, 
+                       trail->pool);
       SVN_ERR (svn_fs__create_successor (&id, fs, src_id, to_noderev,
                                          copy_id, txn_id, trail));
 
@@ -1504,9 +1450,9 @@ svn_fs__dag_copy (dag_node_t *to_node,
          about the copy to the `copies' table, using the COPY_ID we
          reserved above.  */
       SVN_ERR (svn_fs__bdb_create_copy 
-               (copy_id, fs, 
+               (fs, copy_id, 
                 svn_fs__canonicalize_abspath (from_path, trail->pool), 
-                from_txn_id, id, trail));
+                from_txn_id, id, svn_fs__copy_kind_real, trail));
 
       /* Finally, add the COPY_ID to the transaction's list of copies
          so that, if this transaction is aborted, the `copies' table
@@ -1525,59 +1471,13 @@ svn_fs__dag_copy (dag_node_t *to_node,
 }
 
 
-svn_error_t *
-svn_fs__dag_copied_from (svn_revnum_t *rev_p,
-                         const char **path_p,
-                         dag_node_t *node,
-                         trail_t *trail)
-{
-  svn_fs__node_revision_t *noderev;
-  const svn_fs_id_t *id = svn_fs__dag_get_id (node), *pred_id;
-  
-  /* Initialize the return values to mean "not a copy". */
-  *rev_p = SVN_INVALID_REVNUM;
-  *path_p = NULL;
-  
-  SVN_ERR (get_node_revision (&noderev, node, trail));
-  if ((pred_id = noderev->predecessor_id))
-    {
-      const char *id_copy_id = svn_fs__id_copy_id (id);
-      const char *pred_copy_id = svn_fs__id_copy_id (pred_id);
-      
-      /* If NODE's copy id differs from that of its predecessor... */
-      if (strcmp (id_copy_id, pred_copy_id))
-        {
-          /* ... then NODE was either the target of a copy operation,
-             a copied subtree item.  We examine the actual copy record
-             to determine which is the case.  */
-          svn_fs__copy_t *copy;
-          SVN_ERR (svn_fs__bdb_get_copy (&copy, svn_fs__dag_get_fs (node),
-                                         id_copy_id, trail));
-          if (svn_fs__id_eq (copy->dst_noderev_id, id))
-            {
-              /* We need to translate the COPY's transaction ID into a
-                 revision.  So we lookup the transaction, and pull the
-                 revision from it.  It's really not that complicated.  */
-              svn_fs__transaction_t *txn;
-              SVN_ERR (svn_fs__bdb_get_txn (&txn, svn_fs__dag_get_fs (node),
-                                            copy->src_txn_id, trail));
-              *rev_p = txn->revision;
-              *path_p = copy->src_path;
-            }
-        }
-    }
-  
-  return SVN_NO_ERROR;
-}
-
-
 
 /*** Deltification ***/
 
 svn_error_t *
 svn_fs__dag_deltify (dag_node_t *target,
                      dag_node_t *source,
-                     int props_only,
+                     svn_boolean_t props_only,
                      trail_t *trail)
 {
   svn_fs__node_revision_t *source_nr, *target_nr;
@@ -1619,16 +1519,10 @@ svn_fs__dag_commit_txn (svn_revnum_t *new_rev,
                         const char *txn_id,
                         trail_t *trail)
 {
-  dag_node_t *root;
   svn_fs__revision_t revision;
-  svn_fs__transaction_t *transaction;
   svn_string_t date;
 
-  SVN_ERR (svn_fs__dag_txn_root (&root, fs, txn_id, trail));
-
-  /* Add new revision entry to `revisions' table, copying the
-     transaction's property list.  */
-  SVN_ERR (svn_fs__bdb_get_txn (&transaction, fs, txn_id, trail));
+  /* Add new revision entry to `revisions' table. */
   revision.txn_id = txn_id;
   if (new_rev)
     *new_rev = SVN_INVALID_REVNUM;
@@ -1652,8 +1546,8 @@ svn_fs__dag_commit_txn (svn_revnum_t *new_rev,
 /*** Comparison. ***/
 
 svn_error_t *
-svn_fs__things_different (int *props_changed,
-                          int *contents_changed,
+svn_fs__things_different (svn_boolean_t *props_changed,
+                          svn_boolean_t *contents_changed,
                           dag_node_t *node1,
                           dag_node_t *node2,
                           trail_t *trail)
@@ -1687,16 +1581,16 @@ svn_fs__things_different (int *props_changed,
 struct is_ancestor_baton
 {
   const svn_fs_id_t *node1_id;
-  int is_ancestor;
-  int need_parent; /* non-zero if we only care about parenthood, not
-                      full ancestry */
+  svn_boolean_t is_ancestor;
+  svn_boolean_t need_parent; /* TRUE if we only care about parenthood, not
+                                full ancestry */
 };
 
 
 static svn_error_t *
 is_ancestor_callback (void *baton,
                       dag_node_t *node,
-                      int *done,
+                      svn_boolean_t *done,
                       trail_t *trail)
 {
   struct is_ancestor_baton *b = baton;
@@ -1707,19 +1601,19 @@ is_ancestor_callback (void *baton,
     {
       /* ... compare NODE's ID with the ID we're looking for. */
       if (svn_fs__id_eq (b->node1_id, svn_fs__dag_get_id (node)))
-        b->is_ancestor = 1;
+        b->is_ancestor = TRUE;
 
       /* Now, if we only are interested in parenthood, we don't care
          to look any further than this. */
       if (b->need_parent)
-        *done = 1;
+        *done = TRUE;
     }
 
   return SVN_NO_ERROR;
 }
 
 svn_error_t *
-svn_fs__dag_is_ancestor (int *is_ancestor,
+svn_fs__dag_is_ancestor (svn_boolean_t *is_ancestor,
                          dag_node_t *node1,
                          dag_node_t *node2,
                          trail_t *trail)
@@ -1730,27 +1624,27 @@ svn_fs__dag_is_ancestor (int *is_ancestor,
     *id2 = svn_fs__dag_get_id (node2);
 
   /* Pessimism. */
-  *is_ancestor = 0;
+  *is_ancestor = FALSE;
 
   /* Ancestry holds relatedness as a prerequisite. */
   if (! svn_fs_check_related (id1, id2))
     return SVN_NO_ERROR;
 
-  baton.is_ancestor = 0;
-  baton.need_parent = 0;
+  baton.is_ancestor = FALSE;
+  baton.need_parent = FALSE;
   baton.node1_id = id1;
 
   SVN_ERR (svn_fs__dag_walk_predecessors (node2, is_ancestor_callback,
                                           &baton, trail));
   if (baton.is_ancestor)
-    *is_ancestor = 1;
+    *is_ancestor = TRUE;
 
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
-svn_fs__dag_is_parent (int *is_parent,
+svn_fs__dag_is_parent (svn_boolean_t *is_parent,
                        dag_node_t *node1,
                        dag_node_t *node2,
                        trail_t *trail)
@@ -1761,20 +1655,20 @@ svn_fs__dag_is_parent (int *is_parent,
     *id2 = svn_fs__dag_get_id (node2);
 
   /* Pessimism. */
-  *is_parent = 0;
+  *is_parent = FALSE;
 
   /* Parentry holds relatedness as a prerequisite. */
   if (! svn_fs_check_related (id1, id2))
     return SVN_NO_ERROR;
 
-  baton.is_ancestor = 0;
-  baton.need_parent = 1;
+  baton.is_ancestor = FALSE;
+  baton.need_parent = TRUE;
   baton.node1_id = id1;
 
   SVN_ERR (svn_fs__dag_walk_predecessors (node2, is_ancestor_callback,
                                           &baton, trail));
   if (baton.is_ancestor)
-    *is_parent = 1;
+    *is_parent = TRUE;
 
   return SVN_NO_ERROR;
 }
