@@ -1,7 +1,7 @@
 /* dag.h : DAG-like interface filesystem, private to libsvn_fs
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -76,12 +76,12 @@ svn_fs__dag_get_node (dag_node_t **node,
 
 
 /* Return a new dag_node_t object referring to the same node as NODE,
-   allocated in TRAIL->pool.  If you're trying to build a structure in
-   TRAIL->pool that wants to refer to dag nodes that may have been
-   allocated elsewhere, you can call this function, and avoid
-   inter-pool pointers.  */
+   allocated in POOL.  If you're trying to build a structure in a
+   trail that wants to refer to dag nodes that may have been allocated
+   elsewhere, you can call this function, passing "trail->pool", and
+   avoid inter-pool pointers.  */
 dag_node_t *svn_fs__dag_dup (dag_node_t *node,
-                             trail_t *trail);
+                             apr_pool_t *pool);
 
 
 /* Return the filesystem containing NODE.  */
@@ -101,6 +101,11 @@ svn_error_t *svn_fs__dag_get_revision (svn_revnum_t *rev,
 const svn_fs_id_t *svn_fs__dag_get_id (dag_node_t *node);
 
 
+/* Return the created path of NODE.  The value returned is shared
+   with NODE, and will be deallocated when NODE is.  */
+const char *svn_fs__dag_get_created_path (dag_node_t *node);
+
+
 /* Set *ID_P to the node revision ID of NODE's immediate predecessor,
    or NULL if NODE has no predecessor, as part of TRAIL.  The returned
    ID will be allocated in TRAIL->pool.  */
@@ -119,7 +124,7 @@ svn_error_t *svn_fs__dag_get_predecessor_count (int *count,
 /* Callback function type for svn_fs__dag_walk_predecessors() */
 typedef svn_error_t *(*svn_fs__dag_pred_func_t) (void *baton,
                                                  dag_node_t *node,
-                                                 int *done,
+                                                 svn_boolean_t *done,
                                                  trail_t *trail);
 
 /* Walk over NODE's predecessor list, calling CALLBACK (with its
@@ -138,17 +143,12 @@ svn_error_t *svn_fs__dag_walk_predecessors (dag_node_t *node,
 
 /* Return non-zero IFF NODE is currently mutable under Subversion
    transaction TXN_ID.  */
-int svn_fs__dag_check_mutable (dag_node_t *node,
-                               const char *txn_id);
+svn_boolean_t svn_fs__dag_check_mutable (dag_node_t *node,
+                                         const char *txn_id);
 
 /* Return the node kind of NODE. */
 svn_node_kind_t svn_fs__dag_node_kind (dag_node_t *node);
 
-/* Return true iff NODE is a file/directory. */
-int svn_fs__dag_is_file (dag_node_t *node);
-int svn_fs__dag_is_directory (dag_node_t *node);
-
-
 /* Set *PROPLIST_P to a PROPLIST hash representing the entire property
    list of NODE, as part of TRAIL.  The hash has const char * names
    (the property names) and svn_string_t * values (the property values).
@@ -254,7 +254,12 @@ svn_error_t *svn_fs__dag_open (dag_node_t **child_p,
    The returned table is allocated in *either* TRAIL->pool or the pool
    NODE was allocated in, at this function's discretion; the caller
    must finish using it while both of those remain live.  If the
-   caller needs the table to live longer, it should copy the hash.  */
+   caller needs the table to live longer, it should copy the hash. 
+
+   NOTE: the 'kind' field of the svn_fs_dirent_t's is set to
+   svn_node_unknown by this function -- callers that need in
+   interesting value in these slots should fill them in using a new
+   TRAIL, since the list of entries can be arbitrarily large.  */
 svn_error_t *svn_fs__dag_dir_entries (apr_hash_t **entries_p,
                                       dag_node_t *node,
                                       trail_t *trail);
@@ -276,52 +281,27 @@ svn_error_t *svn_fs__dag_set_entry (dag_node_t *node,
    unless NAME in PARENT already refers to a mutable node.  In either
    case, set *CHILD_P to a reference to the new node, allocated in
    TRAIL->pool.  PARENT must be mutable.  NAME must be a single path
-   component; it cannot be a slash-separated directory path.  
+   component; it cannot be a slash-separated directory path.
+   PARENT_PATH must be the canonicalized absolute path of the parent
+   directory.
 
    COPY_ID, if non-NULL, is a key into the `copies' table, and
    indicates that this new node is being created as the result of a
    copy operation, and specifically which operation that was.  
 
+   PATH is the canonicalized absolute path at which this node is being
+   created.
+
    TXN_ID is the Subversion transaction under which this occurs.  */
 svn_error_t *svn_fs__dag_clone_child (dag_node_t **child_p,
                                       dag_node_t *parent,
+                                      const char *parent_path,
                                       const char *name,
                                       const char *copy_id,
                                       const char *txn_id, 
                                       trail_t *trail);
 
 
-/* Create a link to CHILD in PARENT named NAME, as part of TRAIL.
-   PARENT must be mutable.  CHILD must be immutable.  NAME must be a
-   single path component; it cannot be a slash-separated directory
-   path.  TXN_ID is the Subversion transaction under which this
-   occurs.
-
-   Note that it is impossible to use this function to create cyclic
-   directory structures.  Since PARENT is mutable, and every parent of
-   a mutable node is mutable itself, and CHILD is immutable, we know
-   that CHILD can't be equal to, or a parent of, PARENT.  */
-svn_error_t *svn_fs__dag_link (dag_node_t *parent,
-                               dag_node_t *child,
-                               const char *name,
-                               const char *txn_id, 
-                               trail_t *trail);
-
-
-/* Delete the directory entry named NAME from PARENT, as part of
-   TRAIL.  PARENT must be mutable.  NAME must be a single path
-   component; it cannot be a slash-separated directory path.  If the
-   node being deleted is a directory, it must be empty.  TXN_ID is the
-   Subversion transaction under which this occurs.
-
-   If return SVN_ERR_FS_NO_SUCH_ENTRY, then there is no entry NAME in
-   PARENT.  */
-svn_error_t *svn_fs__dag_delete (dag_node_t *parent,
-                                 const char *name,
-                                 const char *txn_id, 
-                                 trail_t *trail);
-
-
 /* Delete the directory entry named NAME from PARENT, as part of
    TRAIL.  PARENT must be mutable.  NAME must be a single path
    component; it cannot be a slash-separated directory path.  If the
@@ -331,16 +311,34 @@ svn_error_t *svn_fs__dag_delete (dag_node_t *parent,
 
    If return SVN_ERR_FS_NO_SUCH_ENTRY, then there is no entry NAME in
    PARENT.  */
-svn_error_t *svn_fs__dag_delete_tree (dag_node_t *parent,
-                                      const char *name,
+svn_error_t *svn_fs__dag_delete (dag_node_t *parent,
+                                 const char *name,
+                                 const char *txn_id,
+                                 trail_t *trail);
+
+
+/* Delete the node revision assigned to node ID from FS's `nodes'
+   table, as part of TRAIL.  Also delete any mutable representations
+   and strings associated with that node revision.  ID may refer to a
+   file or directory, which must be mutable.  TXN_ID is the Subversion
+   transaction under which this occurs.
+
+   NOTE: If ID represents a directory, and that directory has mutable
+   children, you risk orphaning those children by leaving them
+   danging, disconnected from all DAG trees.  It is assumed that
+   callers of this interface know what in the world they are doing.  */
+svn_error_t *svn_fs__dag_remove_node (svn_fs_t *fs,
+                                      const svn_fs_id_t *id,
                                       const char *txn_id,
                                       trail_t *trail);
 
 
 /* Delete all mutable node revisions reachable from node ID, including
-   ID itself, from FS's `nodes' table, as part of TRAIL.  ID may refer
-   to a file or directory, which may be mutable or immutable.  TXN_ID
-   is the Subversion transaction under which this occurs.  */
+   ID itself, from FS's `nodes' table, as part of TRAIL.  Also delete
+   any mutable representations and strings associated with that node
+   revision.  ID may refer to a file or directory, which may be
+   mutable or immutable.  TXN_ID is the Subversion transaction under
+   which this occurs.  */
 svn_error_t *svn_fs__dag_delete_if_mutable (svn_fs_t *fs,
                                             const svn_fs_id_t *id,
                                             const char *txn_id,
@@ -351,12 +349,14 @@ svn_error_t *svn_fs__dag_delete_if_mutable (svn_fs_t *fs,
    TRAIL.  Set *CHILD_P to a reference to the new node, allocated in
    TRAIL->pool.  The new directory has no contents, and no properties.
    PARENT must be mutable.  NAME must be a single path component; it
-   cannot be a slash-separated directory path.  PARENT must not
-   currently have an entry named NAME.  Do any temporary allocation in
-   TRAIL->pool.  TXN_ID is the Subversion transaction under which this
-   occurs.  */
+   cannot be a slash-separated directory path.  PARENT_PATH must be
+   the canonicalized absolute path of the parent directory.  PARENT
+   must not currently have an entry named NAME.  Do any temporary
+   allocation in TRAIL->pool.  TXN_ID is the Subversion transaction
+   under which this occurs.  */
 svn_error_t *svn_fs__dag_make_dir (dag_node_t **child_p,
                                    dag_node_t *parent,
+                                   const char *parent_path,
                                    const char *name,
                                    const char *txn_id,
                                    trail_t *trail);
@@ -412,7 +412,7 @@ svn_error_t *svn_fs__dag_file_length (svn_filesize_t *length,
                                       trail_t *trail);
 
 /* Put the recorded MD5 checksum of FILE into DIGEST, as part of
- * TRAIL.  DIGEST must point to MD5_DIGESTSIZE bytes of storage.
+ * TRAIL.  DIGEST must point to APR_MD5_DIGESTSIZE bytes of storage.
  *
  * If no stored checksum is available, do not calculate the checksum,
  * just put all 0's into DIGEST.
@@ -427,9 +427,12 @@ svn_fs__dag_file_checksum (unsigned char digest[],
    TRAIL->pool.  The new file's contents are the empty string, and it
    has no properties.  PARENT must be mutable.  NAME must be a single
    path component; it cannot be a slash-separated directory path.
-   TXN_ID is the Subversion transaction under which this occurs.  */
+   PARENT_PATH must be the canonicalized absolute path of the parent
+   directory.  TXN_ID is the Subversion transaction under which this
+   occurs.  */
 svn_error_t *svn_fs__dag_make_file (dag_node_t **child_p,
                                     dag_node_t *parent,
+                                    const char *parent_path,
                                     const char *name,
                                     const char *txn_id,
                                     trail_t *trail);
@@ -458,18 +461,6 @@ svn_error_t *svn_fs__dag_copy (dag_node_t *to_node,
                                trail_t *trail);
 
 
-/* If NODE was copied from some other node, set *REV_P and *PATH_P to
-   the revision and path of the other node, as part of TRAIL.
-   Allocate *PATH_P in TRAIL->pool.
-
-   Else if NODE is not a copy, set *REV_P to SVN_INVALID_REVNUM and
-   *PATH_P to null.  */
-svn_error_t *svn_fs__dag_copied_from (svn_revnum_t *rev_p,
-                                      const char **path_p,
-                                      dag_node_t *node,
-                                      trail_t *trail);
-
-
 
 /* Deltification */
 
@@ -484,7 +475,7 @@ svn_error_t *svn_fs__dag_copied_from (svn_revnum_t *rev_p,
    future.  */
 svn_error_t *svn_fs__dag_deltify (dag_node_t *target,
                                   dag_node_t *source,
-                                  int props_only,
+                                  svn_boolean_t props_only,
                                   trail_t *trail);
 
 
@@ -506,8 +497,8 @@ svn_error_t *svn_fs__dag_deltify (dag_node_t *target,
    may leave us with a slight chance of a false positive, though I
    don't really see how that would happen in practice.  Nevertheless,
    it should probably be fixed.  */
-svn_error_t *svn_fs__things_different (int *props_changed,
-                                       int *contents_changed,
+svn_error_t *svn_fs__things_different (svn_boolean_t *props_changed,
+                                       svn_boolean_t *contents_changed,
                                        dag_node_t *node1,
                                        dag_node_t *node2,
                                        trail_t *trail);
@@ -515,7 +506,7 @@ svn_error_t *svn_fs__things_different (int *props_changed,
 
 /* Set *IS_ANCESTOR to non-zero IFF NODE1 is an ancestor of NODE2.
    Perform this test under TRAIL.  */
-svn_error_t *svn_fs__dag_is_ancestor (int *is_ancestor,
+svn_error_t *svn_fs__dag_is_ancestor (svn_boolean_t *is_ancestor,
                                       dag_node_t *node1,
                                       dag_node_t *node2,
                                       trail_t *trail);
@@ -523,7 +514,7 @@ svn_error_t *svn_fs__dag_is_ancestor (int *is_ancestor,
 
 /* Set *IS_PARENT to non-zero IFF NODE1 is the parent of NODE2.
    Perform this test under TRAIL.  */
-svn_error_t *svn_fs__dag_is_parent (int *is_ancestor,
+svn_error_t *svn_fs__dag_is_parent (svn_boolean_t *is_ancestor,
                                     dag_node_t *node1,
                                     dag_node_t *node2,
                                     trail_t *trail);
