@@ -2,7 +2,7 @@
  * editor.c :  Driving and consuming an editor across an svn connection
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -308,15 +308,23 @@ void svn_ra_svn_get_editor(const svn_delta_editor_t **editor,
                            apr_pool_t *pool, svn_ra_svn_edit_callback callback,
                            void *callback_baton)
 {
-  svn_delta_editor_t *ra_svn_editor = svn_delta_default_editor(pool);
+  svn_delta_editor_t *ra_svn_editor;
   ra_svn_edit_baton_t *eb;
 
+  if (svn_ra_svn_has_capability(conn, SVN_RA_SVN_CAP_EDIT_PIPELINE))
+    {
+      svn_ra_svn__get_editorp(editor, edit_baton, conn, pool, callback,
+                              callback_baton);
+      return;
+    }
+
   eb = apr_palloc(pool, sizeof(*eb));
   eb->conn = conn;
   eb->callback = callback;
   eb->callback_baton = callback_baton;
   eb->next_token = 0;
 
+  ra_svn_editor = svn_delta_default_editor(pool);
   ra_svn_editor->set_target_revision = ra_svn_target_rev;
   ra_svn_editor->open_root = ra_svn_open_root;
   ra_svn_editor->delete_entry = ra_svn_delete_entry;
@@ -661,10 +669,12 @@ static svn_error_t *ra_svn_handle_close_edit(svn_ra_svn_conn_t *conn,
                                              void *baton)
 {
   ra_svn_driver_state_t *ds = baton;
+  svn_error_t *err;
 
+  err = ds->editor->close_edit(ds->edit_baton, pool);
   if (ds->aborted)
-    *ds->aborted = FALSE;
-  SVN_CMD_ERR(ds->editor->close_edit(ds->edit_baton, pool));
+    *ds->aborted = (err != SVN_NO_ERROR);
+  SVN_CMD_ERR(err);
   return svn_ra_svn_write_cmd_response(conn, pool, "");
 }
 
@@ -706,6 +716,9 @@ svn_error_t *svn_ra_svn_drive_editor(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
 {
   ra_svn_driver_state_t state;
 
+  if (svn_ra_svn_has_capability(conn, SVN_RA_SVN_CAP_EDIT_PIPELINE))
+    return svn_ra_svn__drive_editorp(conn, pool, editor, edit_baton, aborted);
+
   state.editor = editor;
   state.edit_baton = edit_baton;
   state.tokens = apr_hash_make(pool);
