@@ -2,7 +2,7 @@
  * config_impl.h :  private header for the config file implementation.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -21,6 +21,8 @@
 #ifndef SVN_LIBSVN_SUBR_CONFIG_IMPL_H
 #define SVN_LIBSVN_SUBR_CONFIG_IMPL_H
 
+#define APR_WANT_STDIO
+#include <apr_want.h>
 
 #include <apr_hash.h>
 #include "svn_types.h"
@@ -69,7 +71,7 @@ svn_error_t *svn_config__parse_file (svn_config_t *cfg,
 #define SVN_CONFIG__DEFAULT_SECTION "DEFAULT"
 
 
-#ifdef SVN_WIN32
+#ifdef WIN32
 /* Get the common or user-specific AppData folder */
 svn_error_t *svn_config__win_config_path (const char **folder,
                                           int system_path,
@@ -99,17 +101,17 @@ svn_error_t *svn_config__parse_registry (svn_config_t *cfg,
                                SVN_REGISTRY_PREFIX     \
                                SVN_REGISTRY_HKCU       \
                                SVN_REGISTRY_PATH
-#endif /* SVN_WIN32 */
+#endif /* WIN32 */
 
 /* System-wide and configuration subdirectory names.
    NOTE: Don't use these directly; call svn_config__sys_config_path()
    or svn_config__user_config_path() instead. */
-#ifdef SVN_WIN32
+#ifdef WIN32
 #  define SVN_CONFIG__SUBDIRECTORY    "Subversion"
-#else  /* ! SVN_WIN32 */
+#else  /* ! WIN32 */
 #  define SVN_CONFIG__SYS_DIRECTORY   "/etc/subversion"
 #  define SVN_CONFIG__USR_DIRECTORY   ".subversion"
-#endif /* SVN_WIN32 */
+#endif /* WIN32 */
 
 /* The description/instructions file in the config directory. */
 #define SVN_CONFIG__USR_README_FILE    "README.txt"
@@ -135,13 +137,48 @@ svn_config__sys_config_path (const char **path_p,
    name of the user's config area.  Allocate *PATH_P in POOL.
 
    If the user's personal configuration area cannot be located (most
-   likely under Win32), set *PATH_P to NULL regardless of FNAME.  */
+   likely under Win32), set *PATH_P to NULL regardless of FNAME.  
+   
+   CONFIG_DIR overrides this and if set *PATH_P is set to it.
+   */
 svn_error_t *
-svn_config__user_config_path (const char **path_p,
+svn_config__user_config_path (const char *config_dir,
+                              const char **path_p,
                               const char *fname,
                               apr_pool_t *pool);
 
 
+/* Open a config file FILENAME with mode MODE. FILENAME is encoded in
+   UTF-8, so use POOL for any temporary storage needed for
+   conversions. */
+svn_error_t *
+svn_config__open_file (FILE **pfile,
+                       const char *filename,
+                       const char *mode,
+                       apr_pool_t *pool);
+
+/** A callback function used in enumerating config sections.
+ *
+ * See @c svn_config_enumerate_sections for the details of this type.
+ */
+typedef svn_boolean_t (*svn_config__section_enumerator_t)
+       (const char *name, void *baton);
+
+/** Enumerate the sections, passing @a baton and the current section's name to
+ * @a callback.  Continue the enumeration if @a callback returns @c TRUE.
+ * Return the number of times @a callback was called.
+ *
+ * ### See kff's comment to @c svn_config_enumerate.  It applies to this
+ * function, too. ###
+ *
+ * @a callback's @a name and @a name parameters are only valid for the
+ * duration of the call.          
+ */
+int svn_config__enumerate_sections (svn_config_t *cfg,
+                                   svn_config__section_enumerator_t callback,
+                                   void *baton);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
