@@ -2,7 +2,7 @@
  * paths.c:   a path manipulation library using svn_stringbuf_t
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -134,6 +134,7 @@ svn_path_canonicalize (const char *path, apr_pool_t *pool)
 }
 
 
+#ifndef NDEBUG
 static svn_boolean_t
 is_canonical (const char *path,
               apr_size_t len)
@@ -141,6 +142,7 @@ is_canonical (const char *path,
   return (! SVN_PATH_IS_PLATFORM_EMPTY (path, len)
           && (len <= 1 || path[len-1] != '/'));
 }
+#endif
 
 
 char *svn_path_join (const char *base,
@@ -695,9 +697,12 @@ svn_path_is_single_path_component (const char *name)
 
 /*** URI Stuff ***/
 
-
-svn_boolean_t 
-svn_path_is_url (const char *path)
+/* Examine PATH as a potential URI, and return a substring of PATH
+   that immediately follows the (scheme):// portion of the URI, or
+   NULL if PATH doesn't appear to be a valid URI.  The returned value
+   is not alloced -- it shares memory with PATH. */
+static const char *
+skip_uri_schema (const char *path)
 {
   apr_size_t j;
   apr_size_t len = strlen (path);
@@ -705,6 +710,41 @@ svn_path_is_url (const char *path)
   /* ### Taking strlen() initially is inefficient.  It's a holdover
      from svn_stringbuf_t days. */
 
+  /* Make sure we have enough characters to even compare. */
+  if (len < 4)
+    return NULL;
+
+  /* Look for the sequence '://' */
+  for (j = 0; j < len - 3; j++)
+    {
+      /* We hit a '/' before finding the sequence. */
+      if (path[j] == '/')
+        return NULL;
+
+      /* Skip stuff up to the first ':'. */
+      if (path[j] != ':')
+        continue;
+
+      /* Current character is a ':' now.  It better not be the first
+         character. */
+      if (j == 0)
+        return NULL;
+
+      /* Expecting the next two chars to be '/' */
+
+      if ((path[j + 1] == '/')
+          && (path[j + 2] == '/'))
+        return path + j + 3;
+      
+      return NULL;
+    }
+     
+  return NULL;
+}
+
+svn_boolean_t 
+svn_path_is_url (const char *path)
+{
   /* ### This function is reaaaaaaaaaaaaaally stupid right now.
      We're just going to look for:
  
@@ -714,37 +754,7 @@ svn_path_is_url (const char *path)
 
      Someday it might be nice to have an actual URI parser here.
   */
-
-  /* Make sure we have enough characters to even compare. */
-  if (len < 4)
-    return FALSE;
-
-  /* Look for the sequence '://' */
-  for (j = 0; j < len - 3; j++)
-    {
-      /* We hit a '/' before finding the sequence. */
-      if (path[j] == '/')
-        return FALSE;
-
-      /* Skip stuff up to the first ':'. */
-      if (path[j] != ':')
-        continue;
-
-      /* Current character is a ':' now.  It better not be the first
-         character. */
-      if (j == 0)
-        return FALSE;
-
-      /* Expecting the next two chars to be '/' */
-
-      if ((path[j + 1] == '/')
-          && (path[j + 2] == '/'))
-        return TRUE;
-      
-      return FALSE;
-    }
-     
-  return FALSE;
+  return skip_uri_schema (path) ? TRUE : FALSE;
 }
 
 
@@ -792,6 +802,22 @@ svn_path_is_uri_safe (const char *path)
 {
   apr_size_t i;
 
+  /* Skip the schema. */
+  path = skip_uri_schema (path);
+
+  /* No schema?  Get outta here. */
+  if (! path)
+    return FALSE;
+
+  /* Skip to the first slash that's after the schema. */
+  path = strchr (path, '/');
+
+  /* If there's no first slash, then there's only a host portion;
+     therefore there couldn't be any uri-unsafe characters after the
+     host... so return true. */
+  if (path == NULL)
+    return TRUE;
+
   for (i = 0; path[i]; i++)
     {
       /* Allow '%XX' (where each X is a hex digit) */
@@ -826,9 +852,6 @@ svn_path_uri_encode (const char *path, apr_pool_t *pool)
   apr_size_t i, copied = 0;
   int c;
 
-  if (! path)
-    return NULL;
-
   retstr = svn_stringbuf_create ("", pool);
   for (i = 0; path[i]; i++)
     {
@@ -879,9 +902,6 @@ svn_path_uri_decode (const char *path, apr_pool_t *pool)
   apr_size_t i;
   svn_boolean_t query_start = FALSE;
 
-  if (! path)
-    return NULL;
-
   retstr = svn_stringbuf_create ("", pool);
 
   /* avoid repeated realloc */
@@ -965,7 +985,7 @@ svn_path_get_absolute(const char **pabsolute,
 
       if (apr_err)
         return svn_error_createf(SVN_ERR_BAD_FILENAME, NULL,
-                                 "Couldn't determine absolute path of '%s'.", 
+                                 "Couldn't determine absolute path of '%s'", 
                                  relative);
     }
 
@@ -1004,7 +1024,7 @@ svn_path_split_if_file(const char *path,
   else 
     {
       return svn_error_createf(SVN_ERR_BAD_FILENAME, NULL,
-                               "'%s' is neither a file nor a directory name.",
+                               "'%s' is neither a file nor a directory name",
                                path);
     }
 
@@ -1020,17 +1040,15 @@ get_path_encoding (svn_boolean_t *path_is_utf8, apr_pool_t *pool)
   int encoding_style;
 
   apr_err = apr_filepath_encoding (&encoding_style, pool);
-  if (!apr_err)
-    {
-      /* ### What to do about APR_FILEPATH_ENCODING_UNKNOWN?
-         Well, for now we'll just punt to the svn_utf_ functions;
-         those will at least do the ASCII-subset check. */
-      *path_is_utf8 = (encoding_style == APR_FILEPATH_ENCODING_UTF8);
-      return SVN_NO_ERROR;
-    }
-  else
-    return svn_error_create (apr_err, NULL,
-                             "Can't determine the native path encoding");
+  if (apr_err)
+    return svn_error_wrap_apr (apr_err,
+                               "Can't determine the native path encoding");
+
+  /* ### What to do about APR_FILEPATH_ENCODING_UNKNOWN?
+     Well, for now we'll just punt to the svn_utf_ functions;
+     those will at least do the ASCII-subset check. */
+  *path_is_utf8 = (encoding_style == APR_FILEPATH_ENCODING_UTF8);
+  return SVN_NO_ERROR;
 }
 
 
@@ -1064,5 +1082,5 @@ svn_path_cstring_to_utf8 (const char **path_utf8,
       return SVN_NO_ERROR;
     }
   else
-    return svn_utf_cstring_to_utf8 (path_utf8, path_apr, NULL, pool);
+    return svn_utf_cstring_to_utf8 (path_utf8, path_apr, pool);
 }
