@@ -2,7 +2,7 @@
  * adm_crawler.c:  report local WC mods to an Editor.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -42,24 +42,35 @@
 #include "adm_files.h"
 #include "props.h"
 #include "translate.h"
+#include "entries.h"
 
 
 /* Helper for report_revisions().
    
    Perform an atomic restoration of the file FILE_PATH; that is, copy
    the file's text-base to the administrative tmp area, and then move
-   that file to FILE_PATH with possible translations/expansions.  */
+   that file to FILE_PATH with possible translations/expansions.  If
+   USE_COMMIT_TIMES is set, then set working file's timestamp to
+   last-commit-time.  Either way, set entry-timestamp to match that of
+   the working file when all is finished. */
 static svn_error_t *
 restore_file (const char *file_path,
               svn_wc_adm_access_t *adm_access,
+              svn_boolean_t use_commit_times,
               apr_pool_t *pool)
 {
   const char *text_base_path, *tmp_text_base_path;
   svn_subst_keywords_t *keywords;
   const char *eol;
+  const svn_wc_entry_t *entry;
+  svn_wc_entry_t newentry;
+  apr_time_t tstamp;
+  const char *bname;
+  apr_uint32_t modify_flags = 0;
 
   text_base_path = svn_wc__text_base_path (file_path, FALSE, pool);
   tmp_text_base_path = svn_wc__text_base_path (file_path, TRUE, pool);
+  bname = svn_path_basename (file_path, pool);
 
   SVN_ERR (svn_io_copy_file (text_base_path, tmp_text_base_path,
                              FALSE, pool));
@@ -85,13 +96,30 @@ restore_file (const char *file_path,
   SVN_ERR (svn_wc__maybe_set_executable (NULL, file_path, adm_access, pool));
 
   /* Remove any text conflict */
-  SVN_ERR (svn_wc_resolve_conflict (file_path, adm_access, TRUE, FALSE, FALSE,
-                                    NULL, NULL, pool));
+  SVN_ERR (svn_wc_resolved_conflict (file_path, adm_access, TRUE, FALSE, FALSE,
+                                     NULL, NULL, pool));
 
-  /* ### hey guys, shouldn't we recording the 'restored'
-     working-file's timestamp in its entry?  Right now, every time we
-     restore a file, the front-line-timestamp-check-for-modifiedness
-     is being destroyed. */
+  SVN_ERR (svn_wc_entry (&entry, file_path, adm_access, FALSE, pool));
+  assert(entry != NULL);
+
+  /* Possibly set timestamp to last-commit-time. */
+  if (use_commit_times)
+    {
+      SVN_ERR (svn_io_set_file_affected_time (entry->cmt_date,
+                                              file_path, pool));
+      tstamp = entry->cmt_date;
+    }
+  else
+    {
+      SVN_ERR (svn_io_file_affected_time (&tstamp, file_path, pool));
+    }
+  
+  /* Modify our entry's text-timestamp to match the working file. */
+  modify_flags |= SVN_WC__ENTRY_MODIFY_TEXT_TIME;
+  newentry.text_time = tstamp;
+  SVN_ERR (svn_wc__entry_modify (adm_access, bname,
+                                 &newentry, modify_flags,
+                                 TRUE /* do_sync now */, pool));
 
   return SVN_NO_ERROR;
 }
@@ -118,7 +146,8 @@ restore_file (const char *file_path,
 
    If RESTORE_FILES is set, then unexpectedly missing working files
    will be restored from text-base and NOTIFY_FUNC/NOTIFY_BATON
-   will be called to report the restoration. */
+   will be called to report the restoration.  USE_COMMIT_TIMES is
+   passed to restore_file() helper. */
 static svn_error_t *
 report_revisions (svn_wc_adm_access_t *adm_access,
                   const char *dir_path,
@@ -130,6 +159,7 @@ report_revisions (svn_wc_adm_access_t *adm_access,
                   svn_boolean_t restore_files,
                   svn_boolean_t recurse,
                   svn_boolean_t report_everything,
+                  svn_boolean_t use_commit_times,
                   svn_wc_traversal_info_t *traversal_info,
                   apr_pool_t *pool)
 {
@@ -141,7 +171,7 @@ report_revisions (svn_wc_adm_access_t *adm_access,
   svn_wc_adm_access_t *dir_access;
 
   /* Get both the SVN Entries and the actual on-disk entries.   Also
-     notice that we're picking up 'deleted' entries too. */
+     notice that we're picking up hidden entries too. */
   full_path = svn_path_join (svn_wc_adm_access_path (adm_access), 
                              dir_path, subpool);
   SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access, full_path, subpool));
@@ -160,7 +190,7 @@ report_revisions (svn_wc_adm_access_t *adm_access,
     {
       const svn_string_t *val;
       SVN_ERR (svn_wc_prop_get (&val, SVN_PROP_EXTERNALS, full_path, adm_access,
-                                pool));
+                                subpool));
       if (val)
         {
           apr_pool_t *dup_pool = traversal_info->pool;
@@ -182,9 +212,13 @@ report_revisions (svn_wc_adm_access_t *adm_access,
       apr_ssize_t klen;
       void *val;
       const svn_wc_entry_t *current_entry; 
-      svn_node_kind_t *dirent_kind;
+      svn_node_kind_t *dirent_kind, dirent_kind_storage;
       svn_boolean_t missing = FALSE;
 
+      /* Clear the iteration subpool here because the loop has a bunch
+         of 'continue' jump statements. */
+      svn_pool_clear (iterpool);
+
       /* Get the next entry */
       apr_hash_this (hi, &key, &klen, &val);
       current_entry = val;
@@ -201,10 +235,11 @@ report_revisions (svn_wc_adm_access_t *adm_access,
 
       /*** The Big Tests: ***/
 
-      /* If the entry is 'deleted', make sure the server knows its
-         missing... unless we're reporting everything, in which case
-         it's already missing on the server.  */
-      if (current_entry->deleted && (! report_everything))
+      /* If the entry is 'deleted' or 'absent', make sure the server
+         knows it's gone... unless we're reporting everything, in
+         which case it's already missing on the server.  */
+      if ((current_entry->deleted || current_entry->absent)
+          && (! report_everything))
         {
           SVN_ERR (reporter->delete_path (report_baton, this_path, iterpool));
           continue;
@@ -213,7 +248,17 @@ report_revisions (svn_wc_adm_access_t *adm_access,
       /* Is the entry on disk?  Set a flag if not. */
       dirent_kind = (svn_node_kind_t *) apr_hash_get (dirents, key, klen);
       if (! dirent_kind)
-        missing = TRUE;
+        {
+          /* It is possible on a case insensitive system that the
+             entry is not really missing, so we call our trusty but
+             expensive friend svn_io_check_path to be sure. */
+          SVN_ERR (svn_io_check_path (this_full_path, &dirent_kind_storage,
+                                      iterpool));
+          if (dirent_kind_storage == svn_node_none)
+            missing = TRUE;
+          else
+            dirent_kind = &dirent_kind_storage;
+        }
       
       /* From here on out, ignore any entry scheduled for addition */
       if (current_entry->schedule == svn_wc_schedule_add)
@@ -244,7 +289,7 @@ report_revisions (svn_wc_adm_access_t *adm_access,
             {
               /* ... recreate file from text-base, and ... */
               SVN_ERR (restore_file (this_full_path, dir_access,
-                                     iterpool));
+                                     use_commit_times, iterpool));
               
               /* ... report the restoration to the caller.  */
               if (notify_func != NULL)
@@ -302,10 +347,11 @@ report_revisions (svn_wc_adm_access_t *adm_access,
              recreate it locally, so report as missing and move
              along.  Again, don't bother if we're reporting
              everything, because the dir is already missing on the server. */
-          if (missing && (! report_everything))
+          if (missing)
             {
-              SVN_ERR (reporter->delete_path (report_baton, this_path,
-                                              iterpool));
+              if (! report_everything)
+                SVN_ERR (reporter->delete_path (report_baton, this_path,
+                                                iterpool));
               continue;
             }
           
@@ -317,9 +363,8 @@ report_revisions (svn_wc_adm_access_t *adm_access,
             {
               return svn_error_createf
                 (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                 "The entry '%s' is no longer a directory,\n"
-                 "which prevents proper updates.\n"
-                 "Please remove this entry and try updating again.",
+                 "The entry '%s' is no longer a directory; "
+                 "remove the entry before updating",
                  this_path);
             }
 
@@ -369,12 +414,10 @@ report_revisions (svn_wc_adm_access_t *adm_access,
                                      notify_func, notify_baton,
                                      restore_files, recurse,
                                      subdir_entry->incomplete,
+                                     use_commit_times,
                                      traversal_info,
                                      iterpool));
         } /* end directory case */
-
-      svn_pool_clear (iterpool);
-
     } /* end main entries loop */
 
   /* We're done examining this dir's entries, so free everything. */
@@ -397,6 +440,7 @@ svn_wc_crawl_revisions (const char *path,
                         void *report_baton,
                         svn_boolean_t restore_files,
                         svn_boolean_t recurse,
+                        svn_boolean_t use_commit_times,
                         svn_wc_notify_func_t notify_func,
                         void *notify_baton,
                         svn_wc_traversal_info_t *traversal_info,
@@ -462,6 +506,7 @@ svn_wc_crawl_revisions (const char *path,
                                   notify_func, notify_baton,
                                   restore_files, recurse,
                                   entry->incomplete,
+                                  use_commit_times,
                                   traversal_info,
                                   pool);
           if (err)
@@ -476,7 +521,7 @@ svn_wc_crawl_revisions (const char *path,
       if (missing && restore_files)
         {
           /* Recreate file from text-base. */
-          err = restore_file (path, adm_access, pool);
+          err = restore_file (path, adm_access, use_commit_times, pool);
           if (err)
             goto abort_report;
 
@@ -528,18 +573,19 @@ svn_wc_crawl_revisions (const char *path,
     }
 
   /* Finish the report, which causes the update editor to be driven. */
-  SVN_ERR (reporter->finish_report (report_baton));
+  SVN_ERR (reporter->finish_report (report_baton, pool));
 
  abort_report:
   if (err)
     {
       /* Clean up the fs transaction. */
       svn_error_t *fserr;
-      fserr = reporter->abort_report (report_baton);
-      if (fserr)
-        return svn_error_quick_wrap (fserr, "Error aborting report.");
-      else
-        return err;
+      if ((fserr = reporter->abort_report (report_baton, pool)))
+        {
+          fserr = svn_error_quick_wrap (fserr, "Error aborting report");
+          svn_error_compose (err, fserr);
+        }
+      return err;
     }
 
   return SVN_NO_ERROR;
@@ -556,17 +602,16 @@ svn_wc_transmit_text_deltas (const char *path,
                              apr_pool_t *pool)
 {
   const char *tmpf, *tmp_base;
-  apr_status_t status;
   svn_txdelta_window_handler_t handler;
   void *wh_baton;
   svn_txdelta_stream_t *txdelta_stream;
   apr_file_t *localfile = NULL;
   apr_file_t *basefile = NULL;
   const char *base_digest_hex = NULL;
-  unsigned char digest[MD5_DIGESTSIZE];
+  unsigned char digest[APR_MD5_DIGESTSIZE];
   
   /* Make an untranslated copy of the working file in the
-     adminstrative tmp area because a) we want this to work even if
+     administrative tmp area because a) we want this to work even if
      someone changes the working file while we're generating the
      txdelta, b) we need to detranslate eol and keywords anyway, and
      c) after the commit, we're going to copy the tmp file to become
@@ -595,7 +640,7 @@ svn_wc_transmit_text_deltas (const char *path,
       if (ent->checksum)
         {
           const char *tb = svn_wc__text_base_path (path, FALSE, pool);
-          unsigned char tb_digest[MD5_DIGESTSIZE];
+          unsigned char tb_digest[APR_MD5_DIGESTSIZE];
 
           SVN_ERR (svn_io_file_checksum (tb_digest, tb, pool));
           base_digest_hex = svn_md5_digest_to_cstring (tb_digest, pool);
@@ -634,12 +679,9 @@ svn_wc_transmit_text_deltas (const char *path,
                   
                   return svn_error_createf
                     (SVN_ERR_WC_CORRUPT_TEXT_BASE, NULL,
-                     "svn_wc_transmit_text_deltas: "
-                     "checksum mismatch for '%s':\n"
-                     "   recorded checksum:        %s\n"
-                     "   actual checksum (hex):    %s\n"
-                     "   actual checksum (base64): %s\n",
-                     tb, ent->checksum, base_digest_hex, digest_base64);
+                     "Checksum mismatch for '%s': "
+                     "expected '%s', actual: '%s'",
+                     tb, ent->checksum, base_digest_hex);
                 }
             }
         }
@@ -662,7 +704,7 @@ svn_wc_transmit_text_deltas (const char *path,
   /* Open a filehandle for tmp text-base. */
   SVN_ERR_W (svn_io_file_open (&localfile, tmp_base,
                                APR_READ, APR_OS_DEFAULT, pool),
-             "svn_wc_transmit_text_deltas: error opening local file");
+             "Error opening local file");
 
   /* Create a text-delta stream object that pulls data out of the two
      files. */
@@ -676,9 +718,7 @@ svn_wc_transmit_text_deltas (const char *path,
                                       wh_baton, pool));
     
   /* Close the two files */
-  if ((status = apr_file_close (localfile)))
-    return svn_error_create (status, NULL,
-                             "error closing local file");
+  SVN_ERR (svn_io_file_close (localfile, pool));
   
   if (basefile)
     SVN_ERR (svn_wc__close_text_base (basefile, path, 0, pool));
@@ -712,12 +752,14 @@ svn_wc_transmit_prop_deltas (const char *path,
   apr_hash_t *localprops = apr_hash_make (pool);
   apr_hash_t *baseprops = apr_hash_make (pool);
   
+  /* Get the right access baton for the job. */
+  SVN_ERR (svn_wc_adm_probe_retrieve (&adm_access, adm_access, path, pool));
+
   /* First, get the prop_path from the original path */
   SVN_ERR (svn_wc__prop_path (&props, path, adm_access, FALSE, pool));
   
   /* Get the full path of the prop-base `pristine' file */
-  if ((entry->schedule == svn_wc_schedule_replace)
-      || (entry->schedule == svn_wc_schedule_add))
+  if (entry->schedule == svn_wc_schedule_replace)
     {
       /* do nothing: baseprop hash should be -empty- for comparison
          purposes.  if they already exist on disk, they're "leftover"
@@ -744,8 +786,7 @@ svn_wc_transmit_prop_deltas (const char *path,
     SVN_ERR (svn_wc__load_prop_file (props_base, baseprops, pool));
   
   /* Get an array of local changes by comparing the hashes. */
-  SVN_ERR (svn_wc_get_local_propchanges (&propmods, localprops, 
-                                         baseprops, pool));
+  SVN_ERR (svn_prop_diffs (&propmods, localprops, baseprops, pool));
 
   /* Apply each local change to the baton */
   for (i = 0; i < propmods->nelts; i++)
