@@ -6,7 +6,7 @@
  *              information is kept.  
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -168,10 +168,9 @@ svn_wc__make_adm_thing (svn_wc_adm_access_t *adm_access,
 {
   svn_error_t *err = SVN_NO_ERROR;
   apr_file_t *f = NULL;
-  apr_status_t apr_err = 0;
   const char *path;
 
-  SVN_ERR (svn_wc_adm_write_check (adm_access));
+  SVN_ERR (svn_wc__adm_write_check (adm_access));
 
   path = extend_with_adm_name (svn_wc_adm_access_path (adm_access),
                                NULL, tmp, pool, thing, NULL);
@@ -184,9 +183,7 @@ svn_wc__make_adm_thing (svn_wc_adm_access_t *adm_access,
                                  pool));
 
       /* Creation succeeded, so close immediately. */
-      apr_err = apr_file_close (f);
-      if (apr_err)
-        err = svn_error_create (apr_err, NULL, path);
+      SVN_ERR (svn_io_file_close (f, pool));
     }
   else if (type == svn_node_dir)
     {
@@ -198,47 +195,13 @@ svn_wc__make_adm_thing (svn_wc_adm_access_t *adm_access,
          segfault or other obvious indicator that something went
          wrong.  Even so, not sure if it's appropriate.  Thoughts? */
       err = svn_error_create 
-        (0, NULL, "svn_wc__make_admin_thing: bad type indicator");
+        (0, NULL, "Bad type indicator");
     }
 
   return err;
 }
 
 
-
-/* Copy SRC to DST if SRC exists, else create DST empty. */
-static svn_error_t *
-maybe_copy_file (const char *src, const char *dst, apr_pool_t *pool)
-{
-  svn_node_kind_t kind;
-  apr_status_t apr_err;
-
-  /* First test if SRC exists. */
-  SVN_ERR (svn_io_check_path (src, &kind, pool));
-  if (kind == svn_node_none)
-    {
-      /* SRC doesn't exist, create DST empty. */
-      apr_file_t *f = NULL;
-      SVN_ERR (svn_io_file_open (&f,
-                                 dst,
-                                 (APR_WRITE | APR_CREATE),
-                                 APR_OS_DEFAULT,
-                                 pool));
-      apr_err = apr_file_close (f);
-      if (apr_err)
-        return svn_error_create (apr_err, NULL, dst);
-      else
-        return SVN_NO_ERROR;
-    }
-  else /* SRC exists, so copy it to DST. */
-    {    
-      SVN_ERR (svn_io_copy_file (src, dst, FALSE, pool));
-    }
-
-  return SVN_NO_ERROR;
-}
-
-
 
 /*** Syncing files in the adm area. ***/
 
@@ -264,10 +227,8 @@ sync_adm_file (const char *path,
   path = v_extend_with_adm_name (path, extension, 0, pool, ap);
   va_end (ap);
   
-  /* Remove read-only flag on destination. */
-  SVN_ERR (svn_io_set_file_read_write (path, TRUE, pool));
- 
   /* Rename. */
+  SVN_ERR (svn_wc__prep_file_for_replacement (path, TRUE, pool));
   SVN_ERR (svn_io_file_rename (tmp_path, path, pool));
   SVN_ERR (svn_io_set_file_read_only (path, FALSE, pool));
 
@@ -438,20 +399,15 @@ open_adm_file (apr_file_t **handle,
     {
       if (flags & APR_APPEND)
         {
-          const char *opath, *tmp_path;
-
-          va_start (ap, pool);
-          opath = v_extend_with_adm_name (path, extension, 0, pool, ap);
-          va_end (ap);
-
-          va_start (ap, pool);
-          tmp_path = v_extend_with_adm_name (path, extension, 1, pool, ap);
-          va_end (ap);
-
-          /* Copy the original thing to the tmp location. */
-          SVN_ERR (maybe_copy_file (opath, tmp_path, pool));
+          /* We don't handle append.  To do so we would need to copy the
+             contents into the apr_file_t once it has been opened. */
+          return svn_error_create (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
+                                   "APR_APPEND not supported for adm files");
         }
 
+      /* Need to own the temporary file, so don't reuse an existing one. */
+      flags |= APR_EXCL | APR_CREATE;
+
       /* Extend with tmp name. */
       va_start (ap, pool);
       path = v_extend_with_adm_name (path, extension, 1, pool, ap);
@@ -466,6 +422,14 @@ open_adm_file (apr_file_t **handle,
     }
 
   err = svn_io_file_open (handle, path, flags, protection, pool);
+  if ((flags & APR_WRITE) && err && APR_STATUS_IS_EEXIST(err->apr_err))
+    {
+      /* Exclusive open failed, delete and retry */
+      svn_error_clear (err);
+      SVN_ERR (svn_io_remove_file (path, pool));
+      err = svn_io_file_open (handle, path, flags, protection, pool);
+    }
+
   if (err)
     {
       /* Oddly enough, APR will set *HANDLE even if the open failed.
@@ -482,9 +446,7 @@ open_adm_file (apr_file_t **handle,
         {
           err = svn_error_quick_wrap(err,
                                "Your .svn/tmp directory may be missing or "
-                               "corrupt. "
-                               "Please run 'svn cleanup' and try your "
-                               "operation again.");
+                               "corrupt; run 'svn cleanup' and try again");
         }
     }
 
@@ -505,7 +467,6 @@ close_adm_file (apr_file_t *fp,
                 apr_pool_t *pool,
                 ...)
 {
-  apr_status_t apr_err = 0;
   const char *tmp_path;
   va_list ap;
 
@@ -514,10 +475,7 @@ close_adm_file (apr_file_t *fp,
   tmp_path = v_extend_with_adm_name (path, extension, sync, pool, ap);
   va_end (ap);
 
-  apr_err = apr_file_close (fp);
-
-  if (apr_err)
-    return svn_error_create (apr_err, NULL, tmp_path);
+  SVN_ERR (svn_io_file_close (fp, pool));
 
   /* If we're syncing a tmp file, it needs to be renamed after closing. */
   if (sync)
@@ -530,10 +488,8 @@ close_adm_file (apr_file_t *fp,
       path = v_extend_with_adm_name (path, extension, 0, pool, ap);
       va_end (ap);
       
-      /* Temporarily remove read-only flag on destination. */
-      SVN_ERR (svn_io_set_file_read_write (path, TRUE, pool));
-      
       /* Rename. */
+      SVN_ERR (svn_wc__prep_file_for_replacement (path, TRUE, pool));
       SVN_ERR (svn_io_file_rename (tmp_path, path, pool));
       SVN_ERR (svn_io_set_file_read_only (path, FALSE, pool));
       
@@ -576,8 +532,6 @@ svn_wc__remove_adm_file (const char *path, apr_pool_t *pool, ...)
   path = v_extend_with_adm_name (path, NULL, 0, pool, ap);
   va_end (ap);
       
-  /* Remove read-only flag on path. */
-  SVN_ERR(svn_io_set_file_read_write (path, FALSE, pool));
   SVN_ERR(svn_io_remove_file (path, pool));
 
   return SVN_NO_ERROR;
@@ -666,13 +620,14 @@ svn_wc__open_props (apr_file_t **handle,
   if (wc_format_version == 0)
     return svn_error_createf
       (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-       "svn_wc__open_props: '%s' is not a working copy directory", parent_dir);
+       "'%s' is not a working copy", parent_dir);
 
   /* Then examine the flags to know -which- kind of prop file to get. */
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "open_props: no such thing as 'base' wcprops!");
+                             "No such thing as 'base' "
+                             "working copy properties!");
 
   else if (base)
     {
@@ -740,13 +695,14 @@ svn_wc__close_props (apr_file_t *fp,
   if (wc_format_version == 0)
     return svn_error_createf
       (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-       "svn_wc__close_props: '%s' is not a working copy directory", parent_dir);
+       "'%s' is not a working copy", parent_dir);
 
   /* Then examine the flags to know -which- kind of prop file to get. */
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "close_props: no such thing as 'base' wcprops!");
+                             "No such thing as 'base' "
+                             "working copy properties!");
 
   else if (base)
     {
@@ -809,7 +765,8 @@ svn_wc__sync_props (const char *path,
 
   if (base && wcprops)
     return svn_error_create (SVN_ERR_WC_PATH_NOT_FOUND, NULL,
-                             "close_props: no such thing as 'base' wcprops!");
+                             "No such thing as 'base' "
+                             "working copy properties!");
 
   else if (base)
     {
@@ -876,7 +833,8 @@ check_adm_exists (svn_boolean_t *exists,
     {
       /* If got an error other than dir non-existence, then
          something's weird and we should return a genuine error. */
-      return svn_error_create (APR_ENOTDIR, NULL, tmp_path);
+      return svn_error_createf (APR_ENOTDIR, NULL,
+                                "'%s' is not a directory", tmp_path);
     }
   else if (kind == svn_node_none)
     {
@@ -909,8 +867,6 @@ check_adm_exists (svn_boolean_t *exists,
         svn_error_clear (err);
         wc_exists = FALSE;
       }
-    else if (wc_format > SVN_WC__VERSION)
-      wc_exists = FALSE;
     else
       wc_exists = TRUE;
   }
@@ -930,7 +886,7 @@ check_adm_exists (svn_boolean_t *exists,
       SVN_ERR (svn_wc_adm_close (adm_access));
       if (!entry)
         return svn_error_createf (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-                                  "no entry for '%s'", path);
+                                  "No entry for '%s'", path);
 
       /* The revisions must match except when adding a directory with a
          name that matches a directory scheduled for deletion. That's
@@ -940,7 +896,7 @@ check_adm_exists (svn_boolean_t *exists,
           && !(entry->schedule == svn_wc_schedule_delete && revision == 0))
         return
           svn_error_createf (SVN_ERR_WC_OBSTRUCTED_UPDATE, NULL,
-                             "revision %" SVN_REVNUM_T_FMT
+                             "Revision %" SVN_REVNUM_T_FMT
                              " doesn't match existing revision %"
                              SVN_REVNUM_T_FMT " in '%s'",
                              revision, entry->revision, path);
@@ -964,9 +920,7 @@ static svn_error_t *
 make_empty_adm (const char *path, apr_pool_t *pool)
 {
   path = extend_with_adm_name (path, NULL, 0, pool, NULL);
-
   SVN_ERR (svn_io_dir_make_hidden (path, APR_OS_DEFAULT, pool));
-
   return SVN_NO_ERROR;
 }
 
@@ -979,21 +933,15 @@ init_adm_file (const char *path,
                const char *contents,
                apr_pool_t *pool)
 {
-  svn_error_t *err = SVN_NO_ERROR;
-  apr_status_t apr_err;
   apr_file_t *f = NULL;
 
-  SVN_ERR (svn_wc__open_adm_file (&f, path, thing, APR_WRITE | APR_CREATE,
-                                  pool));
-
-  apr_err = apr_file_write_full (f, contents, strlen (contents), NULL);
-
+  SVN_ERR (svn_wc__open_adm_file (&f, path, thing, 
+                                  APR_WRITE | APR_CREATE, pool));
+  SVN_ERR (svn_io_file_write_full (f, contents, 
+                                   strlen (contents), NULL, pool));
   SVN_ERR (svn_wc__close_adm_file (f, path, thing, 1, pool));
   
-  if (apr_err)
-    err = svn_error_create (apr_err, NULL, path);
-
-  return err;
+  return SVN_NO_ERROR;
 }
 
 
@@ -1034,6 +982,7 @@ init_adm_tmp_area (svn_wc_adm_access_t *adm_access,
    unlock it when done. */
 static svn_error_t *
 init_adm (const char *path,
+          const char *uuid,
           const char *url,
           svn_revnum_t initial_rev,
           apr_pool_t *pool)
@@ -1082,7 +1031,7 @@ init_adm (const char *path,
   /** Initialize each administrative file. */
 
   /* SVN_WC__ADM_ENTRIES */
-  SVN_ERR (svn_wc__entries_init (path, url, initial_rev, pool));
+  SVN_ERR (svn_wc__entries_init (path, uuid, url, initial_rev, pool));
 
   /* SVN_WC__ADM_EMPTY_FILE exists because sometimes an readable, empty
      file is required (in the repository diff for example). Creating such a
@@ -1090,7 +1039,9 @@ init_adm (const char *path,
      efficient than just having one around. It doesn't take up much space
      after all. */
   SVN_ERR (svn_wc__make_adm_thing (adm_access, SVN_WC__ADM_EMPTY_FILE,
-                                   svn_node_file, APR_UREAD, 0, pool));
+                                   svn_node_file,
+                                   APR_UREAD | APR_GREAD | APR_WREAD,
+                                   0, pool));
 
   /* SVN_WC__ADM_README */
   SVN_ERR (init_adm_file (path, SVN_WC__ADM_README, readme_contents, pool));
@@ -1098,9 +1049,8 @@ init_adm (const char *path,
   /* THIS FILE MUST BE CREATED LAST: 
      After this exists, the dir is considered complete. */
   SVN_ERR (svn_io_write_version_file 
-           (svn_path_join_many (pool, 
-                                path, SVN_WC_ADM_DIR_NAME, SVN_WC__ADM_FORMAT, 
-                                NULL),
+           (extend_with_adm_name (path, NULL, FALSE, pool,
+                                  SVN_WC__ADM_FORMAT, NULL),
             SVN_WC__VERSION, pool));
 
   /* Now unlock it.  It's now a valid working copy directory, that
@@ -1114,6 +1064,7 @@ init_adm (const char *path,
 
 svn_error_t *
 svn_wc_ensure_adm (const char *path,
+                   const char *uuid,
                    const char *url,
                    svn_revnum_t revision,
                    apr_pool_t *pool)
@@ -1122,21 +1073,22 @@ svn_wc_ensure_adm (const char *path,
 
   SVN_ERR (check_adm_exists (&exists_already, path, url, revision, pool));
   return (exists_already ? SVN_NO_ERROR :
-          init_adm (path, url, revision, pool));
+          init_adm (path, uuid, url, revision, pool));
 }
 
 
 svn_error_t *
-svn_wc__adm_destroy (svn_wc_adm_access_t *adm_access, apr_pool_t *pool)
+svn_wc__adm_destroy (svn_wc_adm_access_t *adm_access, 
+                     apr_pool_t *pool)
 {
   const char *path;
 
-  SVN_ERR (svn_wc_adm_write_check (adm_access));
+  SVN_ERR (svn_wc__adm_write_check (adm_access));
 
   /* Well, the coast is clear for blowing away the administrative
      directory, which also removes the lock file */
-  path = svn_path_join (svn_wc_adm_access_path(adm_access),
-                        SVN_WC_ADM_DIR_NAME, pool);
+  path = extend_with_adm_name (svn_wc_adm_access_path (adm_access),
+                               NULL, FALSE, pool, NULL);
   SVN_ERR (svn_io_remove_dir (path, pool));
   SVN_ERR (svn_wc_adm_close (adm_access));
 
@@ -1145,14 +1097,15 @@ svn_wc__adm_destroy (svn_wc_adm_access_t *adm_access, apr_pool_t *pool)
 
 
 svn_error_t *
-svn_wc__adm_cleanup_tmp_area (svn_wc_adm_access_t *adm_access, apr_pool_t *pool)
+svn_wc__adm_cleanup_tmp_area (svn_wc_adm_access_t *adm_access, 
+                              apr_pool_t *pool)
 {
   const char *tmp_path;
 
-  SVN_ERR (svn_wc_adm_write_check (adm_access));
+  SVN_ERR (svn_wc__adm_write_check (adm_access));
 
   /* Get the path to the tmp area, and blow it away. */
-  tmp_path = extend_with_adm_name (svn_wc_adm_access_path(adm_access),
+  tmp_path = extend_with_adm_name (svn_wc_adm_access_path (adm_access),
                                    NULL, 0, pool, SVN_WC__ADM_TMP, NULL);
   SVN_ERR (svn_io_remove_dir (tmp_path, pool));
 
@@ -1181,3 +1134,27 @@ svn_wc_create_tmp_file (apr_file_t **fp,
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_wc__prep_file_for_replacement (const char *path,
+                                   svn_boolean_t ignore_enoent,
+                                   apr_pool_t *pool)
+{
+   /* On Unix a read-only file can still be removed or replaced because
+      this is really an edit of the parent directory, not of the file
+      itself.  Windows apparently has different semantics, and so
+      when the svn_io_set_file_read_write() call was temporarily
+      removed in revision 5663, Subversion stopped working on Windows. 
+      
+      However, the svn_io_set_file_read_write() call sets all write
+      permissions on Unix, which is undesireable.  Since it is unnecessary
+      to make the file writeable, do nothing to prep the file for replacement
+      on Unix. */
+
+#ifdef WIN32
+  return svn_io_set_file_read_write (path, ignore_enoent, pool);
+#endif /* WIN32 */
+
+  return SVN_NO_ERROR;
+}
