@@ -2,7 +2,7 @@
  * deadprops.c: mod_dav_svn dead property provider functions for Subversion
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -23,12 +23,14 @@
 
 #include <apr_hash.h>
 
-#include "mod_dav_svn.h"
+#include "dav_svn.h"
+
 #include "svn_xml.h"
 #include "svn_pools.h"
 #include "svn_dav.h"
 #include "svn_base64.h"
 
+
 struct dav_db {
   const dav_resource *resource;
   apr_pool_t *p;
@@ -109,7 +111,8 @@ static dav_error *get_value(dav_db *db, const dav_prop_name *name,
                             propname, db->p);
   if (serr != NULL)
     return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               "could not fetch a property");
+                               "could not fetch a property",
+                               db->resource->pool);
 
   return NULL;
 }
@@ -155,7 +158,8 @@ static dav_error *save_value(dav_db *db, const dav_prop_name *name,
                                          propname, value, db->resource->pool);
   if (serr != NULL)
     return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               "could not change a property");
+                               NULL,
+                               db->resource->pool);
 
   /* a change to the props was made; make sure our cached copy is gone */
   db->props = NULL;
@@ -370,7 +374,8 @@ static dav_error *dav_svn_db_remove(dav_db *db, const dav_prop_name *name)
                                          propname, NULL, db->resource->pool);
   if (serr != NULL)
     return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               "could not remove a property");
+                               "could not remove a property",
+                               db->resource->pool);
 
   /* a change to the props was made; make sure our cached copy is gone */
   db->props = NULL;
@@ -467,7 +472,8 @@ static dav_error *dav_svn_db_first_name(dav_db *db, dav_prop_name *pname)
       if (serr != NULL)
         return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
                                    "could not begin sequencing through "
-                                   "properties");
+                                   "properties",
+                                   db->resource->pool);
     }
 
   /* begin the iteration over the hash */
