@@ -2,7 +2,7 @@
  * main.c: Subversion dump stream filtering tool.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,6 +19,7 @@
 
 #include <apr_file_io.h>
 
+#include "svn_cmdline.h"
 #include "svn_error.h"
 #include "svn_opt.h"
 #include "svn_utf.h"
@@ -58,8 +59,7 @@ create_stdio_stream (svn_stream_t **stream,
 
   apr_status_t apr_err = open_fn (&stdio_file, pool);
   if (apr_err)
-    return svn_error_create (apr_err, NULL,
-                             "error opening stdio file");
+    return svn_error_wrap_apr (apr_err, "Can't open stdio file");
 
   *stream = svn_stream_from_aprfile (stdio_file, pool);
   return SVN_NO_ERROR;
@@ -198,6 +198,7 @@ struct parse_baton_t {
   svn_boolean_t       do_exclude;
   svn_boolean_t       drop_empty_revs;
   svn_boolean_t       do_renumber_revs;
+  svn_boolean_t       preserve_revprops;
   svn_stream_t       *in_stream;
   svn_stream_t       *out_stream;
   apr_int32_t         rev_drop_count;
@@ -216,7 +217,7 @@ struct revision_baton_t {
   svn_revnum_t     rev_orig;
   svn_revnum_t     rev_actual;
   svn_stringbuf_t *header;
-  svn_stringbuf_t *props;
+  apr_hash_t      *props;
   svn_stringbuf_t *body;
   svn_stream_t    *body_stream;
 };
@@ -263,8 +264,8 @@ new_revision_record (void **revision_baton,
   rb->had_dropped_nodes = FALSE;
   rb->pb     = parse_baton;
   rb->header = svn_stringbuf_create ("", pool);
-  rb->props  = svn_stringbuf_create ("", pool);
   rb->body   = svn_stringbuf_create ("", pool);
+  rb->props  = apr_hash_make (pool);
   rb->body_stream = svn_stream_from_stringbuf (rb->body, pool);
 
   header_stream = svn_stream_from_stringbuf (rb->header, pool);
@@ -361,10 +362,35 @@ new_node_record (void **node_baton,
                                 SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH,
                                 APR_HASH_KEY_STRING);
 
-  /* See if this node was copied from dropped source (drop it too, then.) */
-  if ((copyfrom_path != NULL) && (!nb->do_skip))
+  /* See if this node was copied from dropped source.  If it was,
+     we have to drop this node, too.  
+
+     However, there is one special case we'll handle.  If the node is
+     a file, and this was a copy-and-modify operation, then the
+     dumpfile should contain the new contents of the file.  In this
+     scenario, we'll just do an add without history using the new
+     contents.  */
+   if ((copyfrom_path != NULL) && (! nb->do_skip))
     {
-      nb->do_skip = ary_exact_match (pb->dropped_nodes, copyfrom_path);
+      const char *kind, *tcl;
+      kind = apr_hash_get (headers, SVN_REPOS_DUMPFILE_NODE_KIND,
+                           APR_HASH_KEY_STRING);
+      tcl = apr_hash_get (headers, SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH,
+                          APR_HASH_KEY_STRING);
+      /* If there is a Text-content-length header, and the kind is
+         "file", we just fallback to an add without history. */
+      if (tcl && (strcmp (kind, "file") == 0))
+        {
+          apr_hash_set (headers, SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH,
+                        APR_HASH_KEY_STRING, NULL);
+          apr_hash_set (headers, SVN_REPOS_DUMPFILE_NODE_COPYFROM_REV,
+                        APR_HASH_KEY_STRING, NULL);
+          copyfrom_path = NULL;
+        }
+      else
+        {
+          nb->do_skip = ary_exact_match (pb->dropped_nodes, copyfrom_path);
+        }
     }
 
   /* if we're skipping the node, take note of path, discarding the rest
@@ -399,7 +425,7 @@ new_node_record (void **node_baton,
 
           /* Rewrite Node-Copyfrom-Rev if we are renumbering revisions.
              The number points to some revision in the past. We keep track
-             of revision renumbering in a apr_hash, which maps original
+             of revision renumbering in an apr_hash, which maps original
              revisions to new ones. Dropped revision are mapped to -1.
              This should never happen here.
           */
@@ -415,7 +441,7 @@ new_node_record (void **node_baton,
                   /* bail out with an error */
                   return svn_error_createf
                     (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
-                     "Node with dropped parent sneaked in.");
+                     "Node with dropped parent sneaked in");
                 }
               SVN_ERR (svn_stream_printf
                        (header_stream, pool,
@@ -443,10 +469,11 @@ set_revision_property (void *revision_baton,
                        const svn_string_t *value)
 {
   struct revision_baton_t *rb = revision_baton;
+  apr_pool_t *hash_pool = apr_hash_pool_get (rb->props);
 
-  write_prop_to_stringbuf (&(rb->props), name, value);
   rb->has_props = TRUE;
-
+  apr_hash_set (rb->props, apr_pstrdup (hash_pool, name),
+                APR_HASH_KEY_STRING, svn_string_dup (value, hash_pool));
   return SVN_NO_ERROR;
 }
 
@@ -560,21 +587,55 @@ close_revision (void *revision_baton)
   struct revision_baton_t *rb = revision_baton;
   int bytes_used;
   char buf[SVN_KEYLINE_MAXLEN];
+  apr_hash_index_t *hi;
+  apr_pool_t *hash_pool = apr_hash_pool_get (rb->props);
+  svn_stringbuf_t *props = svn_stringbuf_create ("", hash_pool);
 
-  if (rb->has_props)
-    svn_stringbuf_appendcstr (rb->props, "PROPS-END\n");
+  /* If this revision has no nodes left because the ones it had were
+     dropped, and we are not dropping empty revisions, and we were not
+     told to preserve revision props, then we want to fixup the
+     revision props to only contain:
+       - the date
+       - a log message that reports that this revision is just stuffing. */
+  if ((! rb->pb->preserve_revprops)
+      && (! rb->has_nodes) 
+      && rb->had_dropped_nodes 
+      && (! rb->pb->drop_empty_revs))
+    {
+      apr_hash_t *old_props = rb->props;
+      rb->has_props = TRUE;
+      rb->props = apr_hash_make (hash_pool);
+      apr_hash_set (rb->props, SVN_PROP_REVISION_DATE, APR_HASH_KEY_STRING,
+                    apr_hash_get (old_props, SVN_PROP_REVISION_DATE, 
+                                  APR_HASH_KEY_STRING));
+      apr_hash_set (rb->props, SVN_PROP_REVISION_LOG, APR_HASH_KEY_STRING,
+                    svn_string_create ("This is an empty revision for "
+                                       "padding.", hash_pool));
+    }
 
+  /* Now, "rasterize" the props to a string, and append the property
+     information to the header string.  */
   if (rb->has_props)
     {
+      for (hi = apr_hash_first (hash_pool, rb->props); 
+           hi; 
+           hi = apr_hash_next (hi))
+        {
+          const void *key;
+          void *val;
+          apr_hash_this (hi, &key, NULL, &val);
+          write_prop_to_stringbuf (&props, key, val);
+        }
+      svn_stringbuf_appendcstr (props, "PROPS-END\n");
       svn_stringbuf_appendcstr (rb->header,
                                 SVN_REPOS_DUMPFILE_PROP_CONTENT_LENGTH);
-      sprintf (buf, ": %" APR_SIZE_T_FMT "%n", rb->props->len, &bytes_used);
+      sprintf (buf, ": %" APR_SIZE_T_FMT "%n", props->len, &bytes_used);
       svn_stringbuf_appendbytes (rb->header, buf, bytes_used);
       svn_stringbuf_appendbytes (rb->header, "\n", 1);
     }
 
   svn_stringbuf_appendcstr (rb->header, SVN_REPOS_DUMPFILE_CONTENT_LENGTH);
-  sprintf (buf, ": %" APR_SIZE_T_FMT "%n", rb->props->len, &bytes_used);
+  sprintf (buf, ": %" APR_SIZE_T_FMT "%n", props->len, &bytes_used);
   svn_stringbuf_appendbytes (rb->header, buf, bytes_used);
   svn_stringbuf_appendbytes (rb->header, "\n", 1);
 
@@ -582,7 +643,7 @@ close_revision (void *revision_baton)
   svn_stringbuf_appendbytes (rb->header, "\n", 1);
 
   /* put an end to revision */
-  svn_stringbuf_appendbytes (rb->props,  "\n", 1);
+  svn_stringbuf_appendbytes (props,  "\n", 1);
 
   /* write out the revision */
   /* Revision is written out in the following cases:
@@ -598,7 +659,7 @@ close_revision (void *revision_baton)
       SVN_ERR (svn_stream_write (rb->pb->out_stream,
                                  rb->header->data , &(rb->header->len)));
       SVN_ERR (svn_stream_write (rb->pb->out_stream,
-                                 rb->props->data  , &(rb->props->len)));
+                                 props->data      , &(props->len)));
       SVN_ERR (svn_stream_write (rb->pb->out_stream,
                                  rb->body->data   , &(rb->body->len)));
       fprintf (stderr, "Revision %" SVN_REVNUM_T_FMT " committed as %"
@@ -642,7 +703,8 @@ subcommand_help,
 enum
   {
     svndumpfilter__drop_empty_revs = SVN_OPT_FIRST_LONGOPT_ID,
-    svndumpfilter__renumber_revs
+    svndumpfilter__renumber_revs,
+    svndumpfilter__preserve_revprops
   };
 
 /* Option codes and descriptions.
@@ -664,7 +726,8 @@ static const apr_getopt_option_t options_table[] =
      "Remove revisions emptied by filtering."},
     {"renumber-revs",     svndumpfilter__renumber_revs, 0,
      "Renumber revisions left after filtering." },
-
+    {"preserve-revprops",  svndumpfilter__preserve_revprops, 0,
+     "Don't filter revision properties." },
     {NULL}
   };
 
@@ -675,18 +738,20 @@ static const apr_getopt_option_t options_table[] =
 static const svn_opt_subcommand_desc_t cmd_table[] =
   {
     {"exclude", subcommand_exclude, {0},
-     "usage: svndumpfilter exclude PATH_PREFIX [PATH_PREFIX ...]\n\n"
+     "usage: svndumpfilter exclude PATH_PREFIX...\n\n"
      "Filter out nodes with given prefixes from dumpstream.\n",
-     {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs} },
+     {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
+      svndumpfilter__preserve_revprops} },
 
     {"include", subcommand_include, {0},
-     "usage: svndumpfilter include PATH_PREFIX [PATH_PREFIX ...]\n\n"
+     "usage: svndumpfilter include PATH_PREFIX...\n\n"
      "Filter out nodes without given prefixes from dumpstream.\n",
-     {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs} },
+     {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
+      svndumpfilter__preserve_revprops} },
 
     {"help", subcommand_help, {"?", "h"},
-     "usage: svn help [SUBCOMMAND1 [SUBCOMMAND2] ...]\n\n"
-     "Display this usage message.\n",
+     "usage: svndumpfilter help [SUBCOMMAND...]\n\n"
+     "Describe the usage of this program or its subcommands.\n",
      {0} },
 
     { NULL, NULL, {0}, NULL, {0} }
@@ -696,12 +761,13 @@ static const svn_opt_subcommand_desc_t cmd_table[] =
 /* Baton for passing option/argument state to a subcommand function. */
 struct svndumpfilter_opt_state
 {
-  svn_opt_revision_t start_revision;     /* -r X[:Y] is       */
-  svn_opt_revision_t end_revision;       /* not implemented.  */
-  svn_boolean_t drop_empty_revs;         /* --drop-empty-revs */
-  svn_boolean_t help;                    /* --help or -?      */
-  svn_boolean_t renumber_revs;           /* --renumber_revs   */
-  apr_array_header_t *prefixes;          /* mainargs.         */
+  svn_opt_revision_t start_revision;     /* -r X[:Y] is         */
+  svn_opt_revision_t end_revision;       /* not implemented.    */
+  svn_boolean_t drop_empty_revs;         /* --drop-empty-revs   */
+  svn_boolean_t help;                    /* --help or -?        */
+  svn_boolean_t renumber_revs;           /* --renumber-revs     */
+  svn_boolean_t preserve_revprops;       /* --preserve-revprops */
+  apr_array_header_t *prefixes;          /* mainargs.           */
 };
 
 
@@ -743,6 +809,7 @@ parse_baton_initialize (struct parse_baton_t **pb,
   (*pb)->do_exclude = do_exclude;
   (*pb)->do_renumber_revs = opt_state->renumber_revs;
   (*pb)->drop_empty_revs = opt_state->drop_empty_revs;
+  (*pb)->preserve_revprops = opt_state->preserve_revprops;
   (*pb)->prefixes = opt_state->prefixes;
   /* this may be used to shift revision numbers while filtering */
   (*pb)->rev_drop_count = 0;
@@ -765,7 +832,7 @@ subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
   const char *header =
     "general usage: svndumpfilter SUBCOMMAND [ARGS & OPTIONS ...]\n"
-    "Type \"svndumfilter help <subcommand>\" for help on a "
+    "Type \"svndumpfilter help <subcommand>\" for help on a "
     "specific subcommand.\n"
     "\n"
     "Available subcommands:\n";
@@ -787,7 +854,8 @@ subcommand_exclude (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   SVN_ERR (parse_baton_initialize (&pb, opt_state, TRUE, pool));
 
   SVN_ERR (svn_repos_parse_dumpstream (pb->in_stream,
-                                       &filtering_vtable, pb, pool));
+                                       &filtering_vtable, pb,
+                                       NULL, NULL, pool));
 
   fprintf (stderr, "\n Dropped %d revisions, %d nodes\n",
            pb->rev_drop_count, pb->node_drop_count);
@@ -821,7 +889,8 @@ subcommand_include (apr_getopt_t *os, void *baton, apr_pool_t *pool)
   SVN_ERR (parse_baton_initialize (&pb, opt_state, FALSE, pool));
 
   SVN_ERR (svn_repos_parse_dumpstream (pb->in_stream,
-                                       &filtering_vtable, pb, pool));
+                                       &filtering_vtable, pb,
+                                       NULL, NULL, pool));
 
   fprintf (stderr, "\n Dropped %d revisions, %d nodes\n",
            pb->rev_drop_count, pb->node_drop_count);
@@ -854,6 +923,7 @@ main (int argc, const char * const *argv)
 {
   svn_error_t *err;
   apr_status_t apr_err;
+  apr_allocator_t *allocator;
   apr_pool_t *pool;
 
   const svn_opt_subcommand_desc_t *subcommand = NULL;
@@ -868,9 +938,17 @@ main (int argc, const char * const *argv)
   if (svn_cmdline_init ("svndumpfilter", stderr) != EXIT_SUCCESS)
     return EXIT_FAILURE;
 
-  /* Create our top-level pool. */
-  pool = svn_pool_create (NULL);
+  /* Create our top-level pool.  Use a seperate mutexless allocator,
+   * given this application is single threaded.
+   */
+ if (apr_allocator_create (&allocator))
+   return EXIT_FAILURE;
 
+  apr_allocator_max_free_set (allocator, SVN_ALLOCATOR_RECOMMENDED_MAX_FREE);
+
+  pool = svn_pool_create_ex (NULL, allocator);
+  apr_allocator_owner_set (allocator, pool);
+		  
   if (argc <= 1)
     {
       subcommand_help (NULL, NULL, pool);
@@ -917,6 +995,9 @@ main (int argc, const char * const *argv)
         case svndumpfilter__renumber_revs:
           opt_state.renumber_revs = TRUE;
           break;
+        case svndumpfilter__preserve_revprops:
+          opt_state.preserve_revprops = TRUE;
+          break;
         default:
           {
             subcommand_help (NULL, NULL, pool);
@@ -950,7 +1031,7 @@ main (int argc, const char * const *argv)
           subcommand = svn_opt_get_canonical_subcommand (cmd_table, first_arg);
           if (subcommand == NULL)
             {
-              fprintf (stderr, "unknown command: %s\n", first_arg);
+              fprintf (stderr, "unknown command: '%s'\n", first_arg);
               subcommand_help (NULL, NULL, pool);
               svn_pool_destroy (pool);
               return EXIT_FAILURE;
@@ -974,7 +1055,7 @@ main (int argc, const char * const *argv)
               const char *prefix_utf8;
               SVN_INT_ERR (svn_utf_cstring_to_utf8 (&prefix_utf8,
                                                     os->argv[i],
-                                                    NULL,  pool));
+                                                    pool));
               *(const char **)apr_array_push (opt_state.prefixes)
                 = svn_path_internal_style (prefix_utf8, pool);
             }
@@ -993,7 +1074,7 @@ main (int argc, const char * const *argv)
     }
 
 
-  /* Check that the subcommand wasn't passed any inappropriate options.*/
+  /* Check that the subcommand wasn't passed any inappropriate options. */
   for (i = 0; i < num_opts; i++)
     {
       opt_id = received_opts[i];
@@ -1012,12 +1093,9 @@ main (int argc, const char * const *argv)
             svn_opt_get_option_from_code (opt_id, options_table);
           svn_opt_format_option (&optstr, badopt, FALSE, pool);
           fprintf (stderr,
-                   "\nError: subcommand '%s' doesn't accept option '%s'\n\n",
-                   subcommand->name, optstr);
-          svn_opt_subcommand_help (subcommand->name,
-                                   cmd_table,
-                                   options_table,
-                                   pool);
+                   "subcommand '%s' doesn't accept option '%s'\n"
+                   "Type 'svndumpfilter help %s' for usage.\n",
+                   subcommand->name, optstr, subcommand->name);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
