@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -17,7 +17,7 @@
 ######################################################################
 
 # General modules
-import string, sys, os.path, re
+import string, sys, os.path, re, time
 
 # Our testing module
 import svntest
@@ -29,19 +29,18 @@ XFail = svntest.testcase.XFail
 Item = svntest.wc.StateItem
 
 
+
 ######################################################################
 # Tests
 #
-#   Each test must return 0 on success or non-zero on failure.
+#   Each test must return on success or raise on failure.
 
 #----------------------------------------------------------------------
 
 def status_unversioned_file_in_current_dir(sbox):
-  "run status on an unversioned file in the current directory"
-
-  if sbox.build():
-    return 1
+  "status on unversioned file in current directory"
 
+  sbox.build()
   wc_dir = sbox.wc_dir
 
   was_cwd = os.getcwd()
@@ -50,28 +49,19 @@ def status_unversioned_file_in_current_dir(sbox):
 
     svntest.main.file_append('foo', 'a new file')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'stat', 'foo')
+    svntest.actions.run_and_verify_svn(None, [ "?      foo\n" ], [],
+                                       'stat', 'foo')
 
-    if len(stat_output) != 1: 
-      return 1
-
-    if len(err_output) != 0:
-      return 1
-
-    return 0
   finally:
     os.chdir(was_cwd)
 
 #----------------------------------------------------------------------
-
-# regression for issue #590
+# Regression for issue #590
 
 def status_update_with_nested_adds(sbox):
   "run 'status -u' when nested additions are pending"
 
-  if sbox.build():
-    return 1
-
+  sbox.build()
   wc_dir = sbox.wc_dir
 
   # Make a backup copy of the working copy
@@ -84,9 +74,8 @@ def status_update_with_nested_adds(sbox):
   os.makedirs(newdir_path)
   svntest.main.file_append (newfile_path, 'new text')
 
-  # Schedule newdir and newfile for addition
+  # Schedule newdir and newfile for addition (note that the add is recursive)
   svntest.main.run_svn(None, 'add', newdir_path)
-  svntest.main.run_svn(None, 'add', newfile_path)
 
   # Created expected output tree for commit
   expected_output = svntest.wc.State(wc_dir, {
@@ -104,10 +93,9 @@ def status_update_with_nested_adds(sbox):
     })
 
   # Commit.
-  if svntest.actions.run_and_verify_commit (wc_dir, expected_output,
-                                            expected_status, None,
-                                            None, None, None, None, wc_dir):
-    return 1
+  svntest.actions.run_and_verify_commit (wc_dir, expected_output,
+                                         expected_status, None,
+                                         None, None, None, None, wc_dir)
 
   # Now we go to the backup working copy, still at revision 1.
   # We will run 'svn st -u', and make sure that newdir/newfile is reported
@@ -125,69 +113,62 @@ def status_update_with_nested_adds(sbox):
   # an error happens, we'll catch it here.  So that's a good enough
   # regression test for now.  Someday, though, it would be nice to
   # positively match the mostly-empty lines.
-  return svntest.actions.run_and_verify_unquiet_status(wc_backup,
-                                                       expected_status)
-
+  svntest.actions.run_and_verify_unquiet_status(wc_backup,
+                                                expected_status)
+  
 #----------------------------------------------------------------------
 
 # svn status -vN should include all entries in a directory
 def status_shows_all_in_current_dir(sbox):
-  "status -vN and test if all items in the current directory show up"
-
-  if sbox.build():
-    return 1
+  "status -vN shows all items in current directory"
 
+  sbox.build()
   wc_dir = sbox.wc_dir
   was_cwd = os.getcwd()
 
   os.chdir(wc_dir)
+  try:
 
-  stat_output, err_output = svntest.main.run_svn(None, 'stat', '-vN')
-  if err_output:
-    return 1
+    output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                     'stat', '-vN')
 
-  entries_in_wc = len(os.listdir("."))
+    if (len(output) != len(os.listdir("."))):
+      raise svntest.Failure
 
-  os.chdir(was_cwd)
+  finally:
+    os.chdir(was_cwd)
 
-  if (len(stat_output) != entries_in_wc):
-    return 1
-
-  return 0
 
+#----------------------------------------------------------------------
 
 def status_missing_file(sbox):
   "status with a versioned file missing"
 
-  if sbox.build():
-    return 1
-
+  sbox.build()
   wc_dir = sbox.wc_dir
-  was_cwd = os.getcwd()
-
-  os.chdir(wc_dir)
-
-  os.remove('iota')
-
-  stat_output, err_output = svntest.main.run_svn(None, 'status')
-  if err_output:
-    return 1
-  for line in stat_output:
-    if not re.match("! +iota", line):
-      return 1
   
-  os.chdir(was_cwd)
+  was_cwd = os.getcwd()
+  
+  os.chdir(wc_dir)
+  try:
 
-  return 0
+    os.remove('iota')
 
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    for line in output:
+      if not re.match("! +iota", line):
+        raise svntest.Failure
+  finally:
+    os.chdir(was_cwd)
+
+#----------------------------------------------------------------------
 
 def status_type_change(sbox):
-  "status with versioned items whose working type has changed"
-
-  if sbox.build():
-    return 1
+  "status on versioned items whose type has changed"
 
+  sbox.build()
   wc_dir = sbox.wc_dir
+
   was_cwd = os.getcwd()
 
   os.chdir(wc_dir)
@@ -199,49 +180,48 @@ def status_type_change(sbox):
     os.rename('A', 'iota')
     os.rename('was_iota', 'A')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'status')
-    if err_output or len(stat_output) != 2:
-      return 1
-    for line in stat_output:
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    if len(output) != 2:
+      raise svntest.Failure
+    for line in output:
       if not re.match("~ +(iota|A)", line):
-        return 1
+        raise svntest.Failure
 
     # Now change the file that is obstructing the versioned dir into an
     # unversioned dir.
     os.remove('A')
     os.mkdir('A')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'status')
-    if err_output or len(stat_output) != 2:
-      return 1
-    for line in stat_output:
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    if len(output) != 2:
+      raise svntest.Failure
+    for line in output:
       if not re.match("~ +(iota|A)", line):
-        return 1
+        raise svntest.Failure
 
     # Now change the versioned dir that is obstructing the file into an
     # unversioned dir.
     svntest.main.safe_rmtree('iota')
     os.mkdir('iota')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'status')
-    if err_output or len(stat_output) != 2:
-      return 1
-    for line in stat_output:
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    if len(output) != 2:
+      raise svntest.Failure
+    for line in output:
       if not re.match("~ +(iota|A)", line):
-        return 1
+        raise svntest.Failure
 
   finally:
     os.chdir(was_cwd)
 
-  return 0
+#----------------------------------------------------------------------
 
 def status_type_change_to_symlink(sbox):
-  "status with versioned items replaced by symbolic links"
-
-  if sbox.build():
-    return 1
+  "status on versioned items replaced by symlinks"
 
+  sbox.build()
   wc_dir = sbox.wc_dir
+  
   was_cwd = os.getcwd()
 
   os.chdir(wc_dir)
@@ -253,12 +233,12 @@ def status_type_change_to_symlink(sbox):
     svntest.main.safe_rmtree('A/D')
     os.symlink('bar', 'A/D')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'status')
-    if err_output or len(stat_output) != 2:
-      return 1
-    for line in stat_output:
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    if len(output) != 2:
+      raise svntest.Failure
+    for line in output:
       if not re.match("~ +(iota|A/D)", line):
-        return 1
+        raise svntest.Failure
 
     # "valid" symlinks
     os.remove('iota')
@@ -266,103 +246,92 @@ def status_type_change_to_symlink(sbox):
     os.symlink('A/mu', 'iota')
     os.symlink('C', 'A/D')
 
-    stat_output, err_output = svntest.main.run_svn(None, 'status')
-    if err_output or len(stat_output) != 2:
-      return 1
-    for line in stat_output:
+    output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+    if len(output) != 2:
+      raise svntest.Failure
+    for line in output:
       if not re.match("~ +(iota|A/D)", line):
-        return 1
+        raise svntest.Failure
 
   finally:
     os.chdir(was_cwd)
 
-  return 0
-
+#----------------------------------------------------------------------
+# Regression test for revision 3686.
 
 def status_with_new_files_pending(sbox):
-  "status -u with new files pending in the repository (tests rev 3686)"
-
-  if sbox.build():
-    return 1
-
-  wc_dir = sbox.wc_dir
-  was_cwd = os.getcwd()
-
-  os.chdir(wc_dir)
-
-  svntest.main.file_append('newfile', 'this is a new file')
-  svntest.main.run_svn(None, 'add', 'newfile')
-  svntest.main.run_svn(None, 'ci', '-m', 'logmsg')
-  svntest.main.run_svn(None, 'up', '-r', '1')
-
-  stat_output, err_output = svntest.main.run_svn(None, 'status', '-u')
-  if err_output:
-    return 1
-
-  # The bug fixed in revision 3686 was a seg fault.  We don't have a
-  # reliable way to detect a seg fault here, since we haven't dealt
-  # with the popen2{Popen3,Popen4} mess in Python yet (the latter two
-  # are classes within the first, which is a module, and the Popen3
-  # class is not the same as os.popen3().  Got that?)  See the Python
-  # docs for details; in the meantime, no output means there was a
-  # problem.
-  for line in stat_output:
-    if line.find('newfile') != -1:
-      break;
-  else:
-    return 1
-
-  os.chdir(was_cwd)
-
-  return 0
-
-
-def status_for_unignored_file(sbox):
-  "status for unignored file"
+  "status -u with new files in the repository"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
+  
   was_cwd = os.getcwd()
 
   os.chdir(wc_dir)
-
   try:
     svntest.main.file_append('newfile', 'this is a new file')
-    svntest.main.run_svn(None, 'propset', 'svn:ignore', 'newfile', '.')
+    svntest.main.run_svn(None, 'add', 'newfile')
+    svntest.main.run_svn(None, 'ci', '-m', 'logmsg')
+    svntest.main.run_svn(None, 'up', '-r', '1')
+
+    output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                     'status', '-u')
+
+    # The bug fixed in revision 3686 was a seg fault.  We don't have a
+    # reliable way to detect a seg fault here, since we haven't dealt
+    # with the popen2{Popen3,Popen4} mess in Python yet (the latter two
+    # are classes within the first, which is a module, and the Popen3
+    # class is not the same as os.popen3().  Got that?)  See the Python
+    # docs for details; in the meantime, no output means there was a
+    # problem.
+    for line in output:
+      if line.find('newfile') != -1:
+        break
+    else:
+      raise svntest.Failure
+
+  finally:
+    os.chdir(was_cwd)
+
+#----------------------------------------------------------------------
+
+def status_for_unignored_file(sbox):
+  "status for unignored file and directory"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  
+  was_cwd = os.getcwd()
+
+  os.chdir(wc_dir)
+  try:
+    svntest.main.file_append('newfile', 'this is a new file')
+    os.makedirs('newdir')
+    svntest.main.run_svn(None, 'propset', 'svn:ignore', 'new*', '.')
 
     # status on the directory with --no-ignore
-    stat_output, err_output = svntest.main.run_svn(None, 'status', 
-                                                   '--no-ignore', '.')
-    if err_output:
-      raise svntest.Failure
-    status = 1
-    for line in stat_output:
-      if re.match("I +newfile", line):
-        status = 0
-
-    if (status == 1):
-      raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None,
+                                       ['I      newdir\n',
+                                        'I      newfile\n',
+                                        ' M     .\n'],
+                                       [],
+                                       'status', '--no-ignore', '.')
 
     # status specifying the file explicitly on the command line
-    stat_output, err_output = svntest.main.run_svn(None, 'status', 'newfile')
-
-    if err_output:
-      raise svntest.Failure
-    status = 1
-    for line in stat_output:
-      if re.match("I +newfile", line):
-        status = 0
-
-    if (status == 1):
-      raise svntest.Failure
+    svntest.actions.run_and_verify_svn(None,
+                                       ['I      newdir\n',
+                                        'I      newfile\n'],
+                                       [],
+                                       'status', 'newdir', 'newfile')
   
   finally:
     os.chdir(was_cwd)
 
 
+#----------------------------------------------------------------------
+
 def status_for_nonexistent_file(sbox):
-  "status for a file neither on disk nor under version control"
+  "status on missing and unversioned file"
 
   sbox.build()
 
@@ -372,13 +341,12 @@ def status_for_nonexistent_file(sbox):
   os.chdir(wc_dir)
 
   try:
-    stat_output, err_output = svntest.main.run_svn(None, 'status', 
-                                                   'nonexistent-file')
-    if err_output:
-      raise svntest.Failure
+    output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                     'status',
+                                                     'nonexistent-file')
 
     # there should *not* be a status line printed for the nonexistent file 
-    for line in stat_output:
+    for line in output:
       if re.match(" +nonexistent-file", line):
         raise svntest.Failure
   
@@ -386,8 +354,10 @@ def status_for_nonexistent_file(sbox):
     os.chdir(was_cwd)
 
 
+#----------------------------------------------------------------------
+
 def status_file_needs_update(sbox):
-  "status -u should show that outdated file needs update"
+  "status -u indicates out-of-dateness"
 
   # See this thread:
   #
@@ -413,10 +383,9 @@ def status_file_needs_update(sbox):
   # any issue for this bug, so this comment and the thread are your
   # audit trail :-).
 
-  if sbox.build():
-    return 1
-
+  sbox.build()
   wc_dir = sbox.wc_dir
+  
   other_wc = sbox.add_wc_path('other')
 
   svntest.actions.duplicate_dir(wc_dir, other_wc)
@@ -427,7 +396,6 @@ def status_file_needs_update(sbox):
   svntest.main.file_append('crontab.root', 'New file crontab.root.\n')
   svntest.main.run_svn(None, 'add', 'crontab.root')
   svntest.main.run_svn(None, 'ci', '-m', 'log msg')
-
   os.chdir(was_cwd)
   os.chdir(other_wc)
   svntest.main.run_svn(None, 'up')
@@ -441,20 +409,19 @@ def status_file_needs_update(sbox):
   # the -v flag, which we don't want, as this bug never appeared when
   # -v was passed.  So we run status by hand:
   os.chdir(was_cwd)
-  out, err = svntest.main.run_svn(None, 'status', '-u', other_wc)
-  if err:
-    return 1
+  out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                'status', '-u', other_wc)
 
-  saw_it = 0
   for line in out:
     if re.match("\\s+\\*.*crontab\\.root$", line):
-      saw_it = 1
-
-  return not saw_it
+      break
+  else:
+    raise svntest.Failure
 
+#----------------------------------------------------------------------
 
 def status_uninvited_parent_directory(sbox):
-  "status -u wc/added-and-outdated-file should show only that status"
+  "status -u on outdated, added file shows only that"
 
   # To reproduce, check out working copies wc1 and wc2, then do:
   #
@@ -479,10 +446,9 @@ def status_uninvited_parent_directory(sbox):
   # reverted because it caused other status problems (see the test
   # status_file_needs_update(), which fails when 4181 is present).
 
-  if sbox.build():
-    return 1
-
+  sbox.build()
   wc_dir = sbox.wc_dir
+  
   other_wc = sbox.add_wc_path('other')
 
   svntest.actions.duplicate_dir(wc_dir, other_wc)
@@ -504,19 +470,204 @@ def status_uninvited_parent_directory(sbox):
   # We don't want a full status tree here, just one line (or two, if
   # the bug is present).  So run status by hand:
   os.chdir(was_cwd)
-  out, err = svntest.main.run_svn(None, 'status', '-u',
-                                  os.path.join(other_wc, 'newfile'))
-  if err:
-    return 1
+  out, err = svntest.actions.run_and_verify_svn(
+    None, None, [],
+    'status', '-u', os.path.join(other_wc, 'newfile'))
 
-  saw_uninvited_parent_dir = 0
   for line in out:
     # The "/?" is just to allow for an optional trailing slash.
     if re.match("\\s+\\*.*\.other/?$", line):
-      saw_uninvited_parent_dir = 1
+      raise svntest.Failure
 
-  return saw_uninvited_parent_dir
+def status_on_forward_deletion(sbox):
+  "status -u on working copy deleted in HEAD"
+  # See issue #1289.
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  
+  top_url = svntest.main.current_repo_url
+  A_url = top_url + '/A'
 
+  svntest.main.run_svn(None, 'rm', '-m', 'Remove A.', A_url)
+
+  svntest.main.safe_rmtree(wc_dir)
+  os.mkdir(wc_dir)
+  saved_cwd = os.getcwd()
+  os.chdir(wc_dir)
+  try:
+    svntest.main.run_svn(None, 'co', '-r1', top_url, 'wc')
+    # If the bug is present, this will error with
+    #
+    #    subversion/libsvn_wc/lock.c:513: (apr_err=155005)
+    #    svn: Working copy not locked
+    #    svn: directory '' not locked
+    #
+    svntest.actions.run_and_verify_svn(None, None, [], 'st', '-u', 'wc')
+
+    # Try again another way; the error would look like this:
+    #
+    #    subversion/libsvn_repos/delta.c:207: (apr_err=160005)
+    #    svn: Invalid filesystem path syntax
+    #    svn: svn_repos_dir_delta: invalid editor anchoring; at least \
+    #       one of the input paths is not a directory and there was   \
+    #       no source entry.
+    #
+    # (Dang!  Hope a user never has to see that :-) ).
+    #
+    svntest.main.safe_rmtree('wc')
+    svntest.main.run_svn(None, 'co', '-r1', A_url, 'wc')
+    svntest.actions.run_and_verify_svn(None, None, [], 'st', '-u', 'wc')
+    
+  finally:
+    os.chdir(saved_cwd)
+
+#----------------------------------------------------------------------
+
+# Helper for timestamp_behaviour test
+def get_prop_timestamp(path):
+  "get the prop-time for path using svn info"
+  out, err = svntest.actions.run_and_verify_svn(None, None, [], 'info', path)
+  for line in out:
+    if re.match("^Properties Last Updated", line):
+      return line
+  print "Didn't find prop-time for " + path
+  raise svntest.Failure
+
+# Helper for timestamp_behaviour test
+def get_text_timestamp(path):
+  "get the text-time for path using svn info"
+  out, err = svntest.actions.run_and_verify_svn(None, None, [], 'info', path)
+  for line in out:
+    if re.match("^Text Last Updated", line):
+      return line
+  print "Didn't find text-time for " + path
+  raise svntest.Failure
+
+# Helper for timestamp_behaviour test
+def prop_time_behaviour(wc_dir, wc_path, status_path, expected_status):
+  "prop-time behaviour"
+
+  # Pristine prop-time
+  pre_prop_time = get_prop_timestamp(wc_path)
+
+  # Modifying the property does not affect the prop-time
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'name', 'xxx', wc_path)
+  expected_status.tweak(status_path, status=' M')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  prop_time = get_prop_timestamp(wc_path)
+  if prop_time != pre_prop_time:
+    raise svntest.Failure
+
+  # Manually reverting the property does not affect the prop-time
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'name', 'value', wc_path)
+  expected_status.tweak(status_path, status='  ')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  prop_time = get_prop_timestamp(wc_path)
+  if prop_time != pre_prop_time:
+    raise svntest.Failure
+
+  # svn revert changes the prop-time even though the properties don't change
+  svntest.actions.run_and_verify_svn(None, None, [], 'revert', wc_path)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  prop_time = get_prop_timestamp(wc_path)
+  if prop_time == pre_prop_time:
+    raise svntest.Failure
+
+
+# Is this really a status test?  I'm not sure, but I don't know where
+# else to put it.
+def timestamp_behaviour(sbox):
+  "timestamp behaviour"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  # Setup a file and directory with properties
+  A_path = os.path.join(wc_dir, 'A')
+  iota_path = os.path.join(wc_dir, 'iota')
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'propset', 'name', 'value',
+                                     A_path, iota_path)
+  svntest.actions.run_and_verify_svn(None, None, [], 'commit',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     '--message', 'log message',
+                                     wc_dir)
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+  expected_status.tweak(wc_rev=1)
+  expected_status.tweak('iota', 'A', wc_rev=2)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # Sleep to ensure timestamps change
+  time.sleep(2)
+
+  # Check behaviour of prop-time
+  prop_time_behaviour(wc_dir, iota_path, 'iota', expected_status)
+  prop_time_behaviour(wc_dir, A_path, 'A', expected_status)
+
+  # Check behaviour of text-time
+
+  # Pristine text and text-time
+  fp = open(iota_path, 'r')
+  pre_text = fp.readlines()
+  pre_text_time = get_text_timestamp(iota_path)
+
+  # Modifying the text does not affect text-time
+  svntest.main.file_append (iota_path, "some mod")
+  expected_status.tweak('iota', status='M ')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  text_time = get_text_timestamp(iota_path)
+  if text_time != pre_text_time:
+    raise svntest.Failure
+
+  # Manually reverting the text does not affect the text-time
+  fp = open(iota_path, 'w')
+  fp.writelines(pre_text)
+  fp.close()
+  expected_status.tweak('iota', status='  ')
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  text_time = get_text_timestamp(iota_path)
+  if text_time != pre_text_time:
+    raise svntest.Failure
+
+  # svn revert changes the text-time even though the text doesn't change
+  svntest.actions.run_and_verify_svn(None, None, [], 'revert', iota_path)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  text_time = get_text_timestamp(iota_path)
+  if text_time == pre_text_time:
+    raise svntest.Failure
+
+#----------------------------------------------------------------------
+
+def status_on_unversioned_dotdot(sbox):
+  "status on '..' where '..' is not versioned"
+  # See issue #1617.
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  
+  new_dir = os.path.join(wc_dir, 'new_dir')
+  new_subdir = os.path.join(new_dir, 'new_subdir')
+  os.mkdir(new_dir)
+  os.mkdir(new_subdir)
+  
+  saved_cwd = os.getcwd()
+  os.chdir(new_subdir)
+  try:
+    out, err = svntest.main.run_svn(1, 'st', '..')
+    matched = 0
+    for line in err:
+      if re.match(".*which is unsupported for this operation", line):
+        matched = 1
+        break
+    if not matched:
+      raise svntest.Failure
+  finally:
+    os.chdir(saved_cwd)
+
+#----------------------------------------------------------------------
 
 ########################################################################
 # Run the tests
@@ -535,6 +686,9 @@ test_list = [ None,
               status_for_nonexistent_file,
               status_file_needs_update,
               status_uninvited_parent_directory,
+              status_on_forward_deletion,
+              timestamp_behaviour,
+              status_on_unversioned_dotdot,
              ]
 
 if __name__ == '__main__':
