@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -178,10 +178,11 @@ def commit_routine_switching(wc_dir, verify):
   # Try to commit.  We expect this to fail because, if all the
   # switching went as expected, A/B/pi and A/D/G/pi point to the
   # same URL.  We don't allow this.
-  svntest.actions.run_and_verify_commit(wc_dir, None, None,
-                                        "commit to a URL more than once",
-                                        None, None, None, None,
-                                        wc_dir)
+  svntest.actions.run_and_verify_commit(
+    wc_dir, None, None,
+    "svn: Cannot commit both .* as they refer to the same URL$",
+    None, None, None, None,
+    wc_dir)
 
   # Okay, that all taken care of, let's revert the A/D/G/pi path and
   # move along.  Afterward, we should be okay to commit.  (Sorry,
@@ -543,7 +544,7 @@ def relocate_deleted_and_missing(sbox):
   repo_url = sbox.repo_url
   other_repo_dir, other_repo_url = sbox.add_repo_path('other')
   svntest.main.copy_repos(repo_dir, other_repo_dir, 2)
-  svntest.main.safe_rmtree(repo_dir)
+  svntest.main.safe_rmtree(repo_dir, 1)
   svntest.actions.run_and_verify_svn(None, None, [], 'switch', '--relocate',
                                      repo_url, other_repo_url, wc_dir)
 
@@ -562,8 +563,121 @@ def relocate_deleted_and_missing(sbox):
 
 #----------------------------------------------------------------------
 
+def delete_subdir(sbox):
+  "switch that deletes a sub-directory"
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  A_path = os.path.join(wc_dir, 'A')
+  A_url = svntest.main.current_repo_url + '/A'
+  A2_url = svntest.main.current_repo_url + '/A2'
+  A2_B_F_url = svntest.main.current_repo_url + '/A2/B/F'
+
+  svntest.actions.run_and_verify_svn(None,
+                                     ['\n', 'Committed revision 2.\n'], [],
+                                     'cp', '-m', 'make copy', A_url, A2_url)
+
+  svntest.actions.run_and_verify_svn(None,
+                                     ['\n', 'Committed revision 3.\n'], [],
+                                     'rm', '-m', 'delete subdir', A2_B_F_url)
+
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/B/F' : Item(status='D '),
+    })
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.remove('A/B/F')
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
+  expected_status.tweak('A', switched='S')
+  expected_status.remove('A/B/F')
+  expected_status.tweak('', 'iota', wc_rev=1)
+
+  # Used to fail with a 'directory not locked' error for A/B/F
+  svntest.actions.run_and_verify_switch(wc_dir, A_path, A2_url,
+                                        expected_output,
+                                        expected_disk,
+                                        expected_status)
+
+#----------------------------------------------------------------------
+# Issue 1532: Switch a file to a dir: can't switch it back to the file
+
+def file_dir_file(sbox):
+  "switch a file to a dir and back to the file"
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  file_path = os.path.join(wc_dir, 'iota')
+  file_url = svntest.main.current_repo_url + '/iota'
+  dir_url = svntest.main.current_repo_url + '/A/C'
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'switch', dir_url, file_path)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'switch', file_url, file_path)
+
+#----------------------------------------------------------------------
+
 ### ...and a slew of others...
 
+#----------------------------------------------------------------------
+def failed_anchor_is_target(sbox):
+  "anchor=target that fails due to local mods"
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  G_url = svntest.main.current_repo_url + '/A/D/G'
+  G_psi_url = G_url + '/psi'
+  svntest.actions.run_and_verify_svn(None,
+                                     ['\n', 'Committed revision 2.\n'], [],
+                                     'mkdir', '-m', 'log msg', G_psi_url)
+
+  H_path = os.path.join(wc_dir, 'A', 'D', 'H')
+  psi_path = os.path.join(H_path, 'psi')
+  svntest.main.file_append(psi_path, "more text")
+
+  # This switch will fail as it will not delete psi with local mods
+  out, err = svntest.main.run_svn(1, 'switch',
+                                  '--username', svntest.main.wc_author,
+                                  '--password', svntest.main.wc_passwd,
+                                  G_url, H_path)
+  if not err:
+    raise svntest.Failure
+
+  # Some items under H show up as switched because, while H itself was
+  # switched, the switch command failed before it reached all items
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+  expected_status.tweak(wc_rev=1)
+  expected_status.tweak('A/D/H', status='! ', switched='S', wc_rev=2)
+  expected_status.tweak('A/D/H/chi', 'A/D/H/omega', switched='S')
+  expected_status.tweak('A/D/H/psi', status='M ', switched='S')
+  expected_status.add({
+    'A/D/H/pi'      : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/D/H/tau'     : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/D/H/rho'     : Item(status='  ', wc_rev=2, repos_rev=2),
+    })
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # At one stage the failed switch left the wrong URL in the target
+  # directory H.
+  out, err = svntest.actions.run_and_verify_svn(None, None, [], 'info', H_path)
+  for line in out:
+    if line.find('URL: ' + G_url) != -1:
+      break
+  else:
+    raise svntest.Failure
+
+  # Revert local mod and repeat the switch
+  svntest.actions.run_and_verify_svn(None, None, [], 'revert', psi_path)
+  svntest.actions.run_and_verify_svn(None, None, [], 'switch',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     G_url, H_path)
+
+  expected_status.remove('A/D/H/chi', 'A/D/H/omega')
+  expected_status.tweak('A/D/H', status='  ') # remains switched
+  expected_status.tweak('A/D/H/psi', status='  ', switched=None, wc_rev=2)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
 ########################################################################
 # Run the tests
 
@@ -578,7 +692,10 @@ test_list = [ None,
               rev_update_switched_things,
               log_switched_file,
               relocate_deleted_and_missing,
-              ]
+              delete_subdir,
+              XFail(file_dir_file),
+              failed_anchor_is_target,
+             ]
 
 if __name__ == '__main__':
   svntest.main.run_tests(test_list)
