@@ -1,7 +1,7 @@
 /* changes-test.c --- test `changes' interfaces
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,6 +22,7 @@
 
 #include <apr.h>
 
+#include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_test.h"
 #include "../fs-helpers.h"
@@ -160,7 +161,7 @@ changes_add (const char **msg,
 {
   svn_fs_t *fs;
 
-  *msg = "Add changes to the changes table.";
+  *msg = "add changes to the changes table";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -186,7 +187,7 @@ changes_fetch_raw (const char **msg,
   int cur_change_index = 0;
   struct changes_args args;
 
-  *msg = "Fetch raw changes from the changes table.";
+  *msg = "fetch raw changes from the changes table";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -225,7 +226,7 @@ changes_fetch_raw (const char **msg,
                                   &args, pool));
       if (! args.raw_changes)
         return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
-                                  "got no changes for key `%s'", txn_id);
+                                  "got no changes for key '%s'", txn_id);
 
       for (j = 0; j < args.raw_changes->nelts; j++)
         {
@@ -239,41 +240,41 @@ changes_fetch_raw (const char **msg,
           if (strcmp (standard_changes[cur_change_index][0], txn_id))
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "missing some changes for key `%s'", txn_id);
+               "missing some changes for key '%s'", txn_id);
             
           /* Verify that the PATH matches. */
           if (strcmp (standard_changes[cur_change_index][1], change->path))
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "paths differ in change for key `%s'", txn_id);
+               "paths differ in change for key '%s'", txn_id);
 
           /* Verify that the NODE-REV-ID matches. */
           noderev_id = svn_fs_unparse_id (change->noderev_id, pool);
           if (strcmp (standard_changes[cur_change_index][2], noderev_id->data))
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "node revision ids differ in change for key `%s'", txn_id);
+               "node revision ids differ in change for key '%s'", txn_id);
 
           /* Verify that the change KIND matches. */
           kind = string_to_kind (standard_changes[cur_change_index][3]);
           if (kind != change->kind)
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "change kinds differ in change for key `%s'", txn_id);
+               "change kinds differ in change for key '%s'", txn_id);
 
           /* Verify that the change TEXT-MOD bit matches. */
           mod_bit = standard_changes[cur_change_index][4] ? 1 : 0;
           if (mod_bit != change->text_mod)
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "change text-mod bits differ in change for key `%s'", txn_id);
+               "change text-mod bits differ in change for key '%s'", txn_id);
 
           /* Verify that the change PROP-MOD bit matches. */
           mod_bit = standard_changes[cur_change_index][5] ? 1 : 0;
           if (mod_bit != change->prop_mod)
             return svn_error_createf 
               (SVN_ERR_TEST_FAILED, NULL,
-               "change prop-mod bits differ in change for key `%s'", txn_id);
+               "change prop-mod bits differ in change for key '%s'", txn_id);
 
           cur_change_index++;
         }
@@ -293,7 +294,7 @@ changes_delete (const char **msg,
   int num_txns = sizeof (standard_txns) / sizeof (const char *);
   struct changes_args args;
 
-  *msg = "Delete changes from the changes table.";
+  *msg = "delete changes from the changes table";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -317,7 +318,7 @@ changes_delete (const char **msg,
       if ((! args.raw_changes) || (args.raw_changes->nelts))
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "expected empty changes array for txn `%s'", args.key);
+           "expected empty changes array for txn '%s'", args.key);
     }
 
   return SVN_NO_ERROR;
@@ -444,7 +445,7 @@ compare_changes (apr_hash_t *ideals,
       if (! change)
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "missing expected change for path `%s' in txn_id `%s'", 
+           "missing expected change for path '%s' in txn_id '%s'", 
            path, txn_id);
             
       /* Verify that the NODE-REV-ID matches. */
@@ -452,25 +453,25 @@ compare_changes (apr_hash_t *ideals,
                               ideal_change->node_rev_id))
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "node revision ids differ in change for key `%s'", txn_id);
+           "node revision ids differ in change for key '%s'", txn_id);
 
       /* Verify that the change KIND matches. */
       if (change->change_kind != ideal_change->change_kind)
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "change kinds differ in change for key `%s'", txn_id);
+           "change kinds differ in change for key '%s'", txn_id);
 
       /* Verify that the change TEXT-MOD bit matches. */
       if (change->text_mod != ideal_change->text_mod)
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "change text-mod bits differ in change for key `%s'", txn_id);
+           "change text-mod bits differ in change for key '%s'", txn_id);
 
       /* Verify that the change PROP-MOD bit matches. */
       if (change->prop_mod != ideal_change->prop_mod)
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "change prop-mod bits differ in change for key `%s'", txn_id);
+           "change prop-mod bits differ in change for key '%s'", txn_id);
     }
 
   return SVN_NO_ERROR;
@@ -486,7 +487,7 @@ changes_fetch (const char **msg,
   int i;
   int num_txns = sizeof (standard_txns) / sizeof (const char *);
   struct changes_args args;
-  *msg = "Fetch compressed changes from the changes table.";
+  *msg = "fetch compressed changes from the changes table";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -527,11 +528,11 @@ changes_fetch (const char **msg,
       if (! args.changes)
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "got no changes for key `%s'", txn_id);
+           "got no changes for key '%s'", txn_id);
       if (apr_hash_count (ideals) != apr_hash_count (args.changes))
         return svn_error_createf 
           (SVN_ERR_TEST_FAILED, NULL,
-           "unexpected number of changes for key `%s'", txn_id);
+           "unexpected number of changes for key '%s'", txn_id);
       SVN_ERR (compare_changes (ideals, args.changes, txn_id, pool));
     }
 
@@ -539,6 +540,171 @@ changes_fetch (const char **msg,
 }
 
 
+static svn_error_t *
+changes_fetch_ordering (const char **msg, 
+                        svn_boolean_t msg_only,
+                        apr_pool_t *pool)
+{
+  svn_fs_t *fs;
+  svn_revnum_t youngest_rev = 0;
+  const char *txn_name;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root, *rev_root;
+  struct changes_args args;
+  apr_pool_t *subpool = svn_pool_create (pool);
+  apr_hash_index_t *hi;
+
+  *msg = "verify ordered-ness of fetched compressed changes";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* Create a new fs and repos */
+  SVN_ERR (svn_test__create_fs 
+           (&fs, "test-repo-changes-fetch-ordering", pool));
+
+  /*** REVISION 1: Make some files and dirs. ***/
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, youngest_rev, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  {
+    static svn_test__txn_script_command_t script_entries[] = {
+      { 'a', "dir1",        0 },
+      { 'a', "file1",       "This is the file 'file1'.\n" },
+      { 'a', "dir1/file2",  "This is the file 'file2'.\n" },
+      { 'a', "dir1/file3",  "This is the file 'file3'.\n" },
+      { 'a', "dir1/file4",  "This is the file 'file4'.\n" },
+    };
+    SVN_ERR (svn_test__txn_script_exec (txn_root, script_entries, 5, subpool));
+  }
+  SVN_ERR (svn_fs_commit_txn (NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear (subpool);
+
+  /*** REVISION 2: Delete and add some stuff, non-depth-first. ***/
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, youngest_rev, subpool));
+  SVN_ERR (svn_fs_txn_name (&txn_name, txn, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  {
+    static svn_test__txn_script_command_t script_entries[] = {
+      { 'd', "file1",       "This is the file 'file1'.\n" },
+      { 'd', "dir1/file2",  "This is the file 'file2'.\n" },
+      { 'd', "dir1/file3",  "This is the file 'file3'.\n" },
+      { 'a', "dir1/file5",  "This is the file 'file4'.\n" },
+      { 'a', "dir1/dir2",   0 },
+      { 'd', "dir1",        0 },
+      { 'a', "dir3",        0 },
+    };
+    SVN_ERR (svn_test__txn_script_exec (txn_root, script_entries, 7, subpool));
+  }
+  SVN_ERR (svn_fs_commit_txn (NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear (subpool);
+  
+  /*** TEST:  We should have only three changes, the deletion of 'file1'
+       the deletion of 'dir1', and the addition of 'dir3'. ***/
+  args.fs = fs;
+  args.key = txn_name;
+  SVN_ERR (svn_fs__retry_txn (fs, txn_body_changes_fetch, &args, subpool));
+  if ((! args.changes) || (apr_hash_count (args.changes) != 3))
+    return svn_error_create (SVN_ERR_TEST_FAILED, NULL,
+                             "expected changes");
+  for (hi = apr_hash_first (subpool, args.changes); 
+       hi; hi = apr_hash_next (hi))
+    {
+      const void *key;
+      void *val;
+      svn_fs_path_change_t *change;
+
+      /* KEY will be the path, VAL the change. */
+      apr_hash_this (hi, &key, NULL, &val);
+      change = val;
+
+      if ((change->change_kind == svn_fs_path_change_add) 
+          && (strcmp (key, "/dir3") == 0))
+        ;
+      else if ((change->change_kind == svn_fs_path_change_delete)
+               && ((strcmp (key, "/dir1") == 0)
+                   || (strcmp (key, "/file1") == 0)))
+        ;
+      else
+        return svn_error_create (SVN_ERR_TEST_FAILED, NULL, 
+                                 "got wrong changes");
+    }
+
+  /*** REVISION 3: Do the same stuff as in revision 1. ***/
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, youngest_rev, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  {
+    static svn_test__txn_script_command_t script_entries[] = {
+      { 'a', "dir1",        0 },
+      { 'a', "file1",       "This is the file 'file1'.\n" },
+      { 'a', "dir1/file2",  "This is the file 'file2'.\n" },
+      { 'a', "dir1/file3",  "This is the file 'file3'.\n" },
+      { 'a', "dir1/file4",  "This is the file 'file4'.\n" },
+    };
+    SVN_ERR (svn_test__txn_script_exec (txn_root, script_entries, 5, subpool));
+  }
+  SVN_ERR (svn_fs_commit_txn (NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear (subpool);
+
+  /*** REVISION 4: Do the same stuff as in revision 2, but use a copy
+       overwrite of the top directory (instead of a delete) to test
+       that the 'replace' change type works, too.  (And add 'dir4'
+       instead of 'dir3', since 'dir3' still exists).  ***/
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, youngest_rev, subpool));
+  SVN_ERR (svn_fs_txn_name (&txn_name, txn, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  SVN_ERR (svn_fs_revision_root (&rev_root, fs, 1, subpool));
+  {
+    static svn_test__txn_script_command_t script_entries[] = {
+      { 'd', "file1",       "This is the file 'file1'.\n" },
+      { 'd', "dir1/file2",  "This is the file 'file2'.\n" },
+      { 'd', "dir1/file3",  "This is the file 'file3'.\n" },
+      { 'a', "dir1/file5",  "This is the file 'file4'.\n" },
+      { 'a', "dir1/dir2",   0 },
+    };
+    SVN_ERR (svn_test__txn_script_exec (txn_root, script_entries, 5, subpool));
+    SVN_ERR (svn_fs_copy (rev_root, "dir1", txn_root, "dir1", subpool));
+    SVN_ERR (svn_fs_make_dir (txn_root, "dir4", subpool));
+  }
+  SVN_ERR (svn_fs_commit_txn (NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear (subpool);
+
+  /*** TEST:  We should have only three changes, the deletion of 'file1'
+       the replacement of 'dir1', and the addition of 'dir4'. ***/
+  args.fs = fs;
+  args.key = txn_name;
+  SVN_ERR (svn_fs__retry_txn (fs, txn_body_changes_fetch, &args, subpool));
+  if ((! args.changes) || (apr_hash_count (args.changes) != 3))
+    return svn_error_create (SVN_ERR_TEST_FAILED, NULL,
+                             "expected changes");
+  for (hi = apr_hash_first (subpool, args.changes); 
+       hi; hi = apr_hash_next (hi))
+    {
+      const void *key;
+      void *val;
+      svn_fs_path_change_t *change;
+
+      /* KEY will be the path, VAL the change. */
+      apr_hash_this (hi, &key, NULL, &val);
+      change = val;
+
+      if ((change->change_kind == svn_fs_path_change_add) 
+          && (strcmp (key, "/dir4") == 0))
+        ;
+      else if ((change->change_kind == svn_fs_path_change_replace) 
+               && (strcmp (key, "/dir1") == 0))
+        ;
+      else if ((change->change_kind == svn_fs_path_change_delete) 
+               && (strcmp (key, "/file1") == 0))
+        ;
+      else
+        return svn_error_create (SVN_ERR_TEST_FAILED, NULL, 
+                                 "got wrong changes");
+    }
+
+  return SVN_NO_ERROR;
+}
+
+
 
 /* The test table.  */
 
@@ -549,5 +715,6 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS (changes_fetch_raw),
     SVN_TEST_PASS (changes_delete),
     SVN_TEST_PASS (changes_fetch),
+    SVN_TEST_PASS (changes_fetch_ordering),
     SVN_TEST_NULL
   };
