@@ -2,7 +2,7 @@
  * translate-test.c -- test the eol and keyword translation subroutine
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -218,7 +218,7 @@ remove_file (const char *fname, apr_pool_t *pool)
         }
       else
         return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
-                                  "non-file `%s' is in the way", fname);
+                                  "non-file '%s' is in the way", fname);
     }
 
   return SVN_NO_ERROR;
@@ -280,6 +280,7 @@ substitute_and_verify (const char *test_name,
   keywords.date     = date   ? svn_string_create (date, pool)   : NULL;
   keywords.author   = author ? svn_string_create (author, pool) : NULL;
   keywords.url      = url    ? svn_string_create (url, pool)    : NULL;
+  keywords.id       = NULL;
 
   err = svn_subst_copy_and_translate (src_fname, dst_fname, dst_eol, repair,
                                       &keywords, expand, pool);
@@ -307,7 +308,10 @@ substitute_and_verify (const char *test_name,
              src_fname, buf);
         }
       else
-        return SVN_NO_ERROR;
+        {
+          svn_error_clear (err);
+          return SVN_NO_ERROR;
+        }
 
     }
   else if (err)
@@ -894,7 +898,7 @@ mixed_no_repair (const char **msg,
                  svn_boolean_t msg_only,
                  apr_pool_t *pool)
 {
-  *msg = "don't convert mixed line endings in absence of repair flag";
+  *msg = "keep mixed line endings without repair flag";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -919,7 +923,7 @@ expand_author (const char **msg,
                svn_boolean_t msg_only,
                apr_pool_t *pool)
 {
-  *msg = "expand author keyword";
+  *msg = "expand author";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -939,7 +943,7 @@ expand_date (const char **msg,
              svn_boolean_t msg_only,
              apr_pool_t *pool)
 {
-  *msg = "expand date keyword";
+  *msg = "expand date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -961,7 +965,7 @@ expand_author_date (const char **msg,
                     svn_boolean_t msg_only,
                     apr_pool_t *pool)
 {
-  *msg = "expand author and date keywords";
+  *msg = "expand author and date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -983,7 +987,7 @@ expand_author_rev (const char **msg,
                    svn_boolean_t msg_only,
                    apr_pool_t *pool)
 {
-  *msg = "expand author and rev keywords";
+  *msg = "expand author and rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1005,7 +1009,7 @@ expand_rev (const char **msg,
             svn_boolean_t msg_only,
             apr_pool_t *pool)
 {
-  *msg = "expand rev keyword";
+  *msg = "expand rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1027,7 +1031,7 @@ expand_rev_url (const char **msg,
                 svn_boolean_t msg_only,
                 apr_pool_t *pool)
 {
-  *msg = "expand rev and url keywords";
+  *msg = "expand rev and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1049,7 +1053,7 @@ expand_author_date_rev_url (const char **msg,
                             svn_boolean_t msg_only,
                             apr_pool_t *pool)
 {
-  *msg = "expand author, date, rev, and url keywords";
+  *msg = "expand author, date, rev, and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1082,7 +1086,7 @@ lf_to_crlf_expand_author (const char **msg,
                           svn_boolean_t msg_only,
                           apr_pool_t *pool)
 {
-  *msg = "lf_to_crlf, plus expand author keyword";
+  *msg = "lf_to_crlf; expand author";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1100,7 +1104,7 @@ mixed_to_lf_expand_author_date (const char **msg,
                                 svn_boolean_t msg_only,
                                 apr_pool_t *pool)
 {
-  *msg = "mixed_to_lf, plus expand author and date keywords";
+  *msg = "mixed_to_lf; expand author and date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1118,7 +1122,7 @@ crlf_to_cr_expand_author_rev (const char **msg,
                               svn_boolean_t msg_only,
                               apr_pool_t *pool)
 {
-  *msg = "crlf_to_cr, plus expand author and rev keywords";
+  *msg = "crlf_to_cr; expand author and rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1136,7 +1140,7 @@ cr_to_crlf_expand_rev (const char **msg,
                        svn_boolean_t msg_only,
                        apr_pool_t *pool)
 {
-  *msg = "cr_to_crlf, plus expand rev keyword";
+  *msg = "cr_to_crlf; expand rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1154,7 +1158,7 @@ cr_to_crlf_expand_rev_url (const char **msg,
                            svn_boolean_t msg_only,
                            apr_pool_t *pool)
 {
-  *msg = "cr_to_crlf, plus expand rev and url keywords";
+  *msg = "cr_to_crlf; expand rev and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1172,7 +1176,7 @@ mixed_to_crlf_expand_author_date_rev_url (const char **msg,
                                           svn_boolean_t msg_only,
                                           apr_pool_t *pool)
 {
-  *msg = "mixed_to_crlf, plus expand author, date, rev, and url keywords";
+  *msg = "mixed_to_crlf; expand author, date, rev, and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1198,7 +1202,7 @@ unexpand_author (const char **msg,
                  svn_boolean_t msg_only,
                  apr_pool_t *pool)
 {
-  *msg = "unexpand author keyword";
+  *msg = "unexpand author";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1218,7 +1222,7 @@ unexpand_date (const char **msg,
                svn_boolean_t msg_only,
                apr_pool_t *pool)
 {
-  *msg = "unexpand date keyword";
+  *msg = "unexpand date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1240,7 +1244,7 @@ unexpand_author_date (const char **msg,
                       svn_boolean_t msg_only,
                       apr_pool_t *pool)
 {
-  *msg = "unexpand author and date keywords";
+  *msg = "unexpand author and date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1262,7 +1266,7 @@ unexpand_author_rev (const char **msg,
                      svn_boolean_t msg_only,
                      apr_pool_t *pool)
 {
-  *msg = "unexpand author and rev keywords";
+  *msg = "unexpand author and rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1284,7 +1288,7 @@ unexpand_rev (const char **msg,
               svn_boolean_t msg_only,
               apr_pool_t *pool)
 {
-  *msg = "unexpand rev keyword";
+  *msg = "unexpand rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1306,7 +1310,7 @@ unexpand_rev_url (const char **msg,
                   svn_boolean_t msg_only,
                   apr_pool_t *pool)
 {
-  *msg = "unexpand rev and url keywords";
+  *msg = "unexpand rev and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1328,7 +1332,7 @@ unexpand_author_date_rev_url (const char **msg,
                               svn_boolean_t msg_only,
                               apr_pool_t *pool)
 {
-  *msg = "unexpand author, date, rev, and url keywords";
+  *msg = "unexpand author, date, rev, and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1361,7 +1365,7 @@ lf_to_crlf_unexpand_author (const char **msg,
                             svn_boolean_t msg_only,
                             apr_pool_t *pool)
 {
-  *msg = "lf_to_crlf, plus unexpand author keyword";
+  *msg = "lf_to_crlf; unexpand author";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1379,7 +1383,7 @@ mixed_to_lf_unexpand_author_date (const char **msg,
                                   svn_boolean_t msg_only,
                                   apr_pool_t *pool)
 {
-  *msg = "mixed_to_lf, plus unexpand author and date keywords";
+  *msg = "mixed_to_lf; unexpand author and date";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1397,7 +1401,7 @@ crlf_to_cr_unexpand_author_rev (const char **msg,
                                 svn_boolean_t msg_only,
                                 apr_pool_t *pool)
 {
-  *msg = "crlf_to_cr, plus unexpand author and rev keywords";
+  *msg = "crlf_to_cr; unexpand author and rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1415,7 +1419,7 @@ cr_to_crlf_unexpand_rev (const char **msg,
                          svn_boolean_t msg_only,
                          apr_pool_t *pool)
 {
-  *msg = "cr_to_crlf, plus unexpand rev keyword";
+  *msg = "cr_to_crlf; unexpand rev";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1433,7 +1437,7 @@ cr_to_crlf_unexpand_rev_url (const char **msg,
                              svn_boolean_t msg_only,
                              apr_pool_t *pool)
 {
-  *msg = "cr_to_crlf, plus unexpand rev and url keywords";
+  *msg = "cr_to_crlf; unexpand rev and url";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1451,7 +1455,7 @@ mixed_to_crlf_unexpand_author_date_rev_url (const char **msg,
                                             svn_boolean_t msg_only,
                                             apr_pool_t *pool)
 {
-  *msg = "mixed_to_crlf, plus unexpand author, date, rev, and url keywords";
+  *msg = "mixed_to_crlf; unexpand author, date, rev, url";
 
   if (msg_only)
     return SVN_NO_ERROR;
