@@ -8,6 +8,17 @@ use strict;
 #    $ showchange.pl REVISION [PATH|URL]
 
 
+if ((scalar(@ARGV) == 0)
+    or ($ARGV[0] eq '-?')
+    or ($ARGV[0] eq '-h')
+    or ($ARGV[0] eq '--help')) {
+    print <<EOF;
+Show the log message and diff for a revision.
+usage: $0 REVISION [PATH|URL]
+EOF
+    exit 0;
+}
+
 my $revision = shift || die ("Revision argument required.\n");
 
 my $url = shift || "";
